@@ -21,6 +21,7 @@
 
 #include <calf/audio_fx.h>
 #include <calf/giface.h>
+#include <limits.h>
 #include <stdlib.h>
 #include <time.h>
 #include <math.h>
@@ -166,7 +167,7 @@ void biquad_filter_module::sanitize()
 }
 
 int biquad_filter_module::process_channel(uint16_t channel_no, const float *in, float *out, uint32_t numsamples, int inmask) {
-    dsp::biquad_d1<float> *filter;
+    dsp::biquad_d1 *filter;
     switch (channel_no) {
     case 0:
         filter = left;
@@ -354,8 +355,9 @@ tap_distortion::tap_distortion()
     is_active = false;
     srate = 0;
     meter = 0.f;
-    prev_med = prev_out = 0.f;
+    rdrive = rbdr = kpa = kpb = kna = knb = ap = an = imr = kc = srct = sq = pwrq = prev_med = prev_out = 0.f;
     drive_old = blend_old = -1.f;
+    over = 1;
 }
 
 void tap_distortion::activate()
@@ -395,24 +397,29 @@ void tap_distortion::set_params(float blend, float drive)
 void tap_distortion::set_sample_rate(uint32_t sr)
 {
     srate = sr;
+    over = srate * 2 > 96000 ? 1 : 2;
+    resampler.set_params(srate, over, 2);
 }
 
 float tap_distortion::process(float in)
 {
+    double *samples = resampler.upsample((double)in);
     meter = 0.f;
-    float out = 0.f;
-    float proc = in;
-    float med;
-    if (proc >= 0.0f) {
-        med = (D(ap + proc * (kpa - proc)) + kpb) * pwrq;
-    } else {
-        med = (D(an - proc * (kna + proc)) + knb) * pwrq * -1.0f;
+    for (int o = 0; o < over; o++) {
+        float proc = samples[o];
+        float med;
+        if (proc >= 0.0f) {
+            med = (D(ap + proc * (kpa - proc)) + kpb) * pwrq;
+        } else {
+            med = (D(an - proc * (kna + proc)) + knb) * pwrq * -1.0f;
+        }
+        proc = srct * (med - prev_med + prev_out);
+        prev_med = M(med);
+        prev_out = M(proc);
+        samples[o] = proc;
+        meter = std::max(meter, proc);
     }
-    proc = srct * (med - prev_med + prev_out);
-    prev_med = M(med);
-    prev_out = M(proc);
-    out = proc;
-    meter = proc;
+    float out = (float)resampler.downsample(samples);
     return out;
 }
 
@@ -511,7 +518,22 @@ void simple_lfo::set_params(float f, int m, float o, uint32_t sr, float a)
     srate = sr;
     amount = a;
 }
-
+void simple_lfo::set_freq(float f)
+{
+    freq = f;
+}
+void simple_lfo::set_mode(int m)
+{
+    mode = m;
+}
+void simple_lfo::set_offset(float o)
+{
+    offset = o;
+}
+void simple_lfo::set_amount(float a)
+{
+    amount = a;
+}
 bool simple_lfo::get_graph(float *data, int points, cairo_iface *context, int *mode) const
 {
     if (!is_active)
@@ -566,6 +588,12 @@ lookahead_limiter::lookahead_limiter() {
     asc_changed = false;
     asc_coeff = 1.f;
 }
+lookahead_limiter::~lookahead_limiter()
+{
+    free(buffer);
+    free(nextpos);
+    free(nextdelta);
+}
 
 void lookahead_limiter::activate()
 {
@@ -594,12 +622,13 @@ void lookahead_limiter::set_sample_rate(uint32_t sr)
     // rebuild buffer
     overall_buffer_size = (int)(srate * (100.f / 1000.f) * channels) + channels; // buffer size attack rate multiplied by 2 channels
     buffer = (float*) calloc(overall_buffer_size, sizeof(float));
-    memset(buffer, 0, overall_buffer_size * sizeof(float)); // reset buffer to zero
     pos = 0;
 
-    nextpos = (int*) calloc(overall_buffer_size, sizeof(int));
     nextdelta = (float*) calloc(overall_buffer_size, sizeof(float));
+    nextpos = (int*) malloc(overall_buffer_size * sizeof(int));
     memset(nextpos, -1, overall_buffer_size * sizeof(int));
+    
+    reset();
 }
 
 void lookahead_limiter::set_params(float l, float a, float r, float w, bool ar, float arc, bool d)
@@ -859,3 +888,513 @@ bool lookahead_limiter::get_asc() {
     asc_active = false;
     return true;
 }
+
+
+////////////////////////////////////////////////////////////////////////////////
+
+transients::transients() {
+    envelope        = 0.f;
+    attack          = 0.f;
+    release         = 0.f;
+    attack_coef     = 0.f;
+    release_coef    = 0.f;
+    att_time        = 0.f;
+    att_level       = 0.f;
+    rel_time        = 0.f;
+    rel_level       = 0.f;
+    sust_thres      = 1.f;
+    maxdelta        = 0.f;
+    new_return      = 1.f;
+    old_return      = 1.f;
+    lookahead       = 0;
+    lookpos         = 0;
+    channels        = 1;
+    sustain_ended   = false;
+}
+transients::~transients()
+{
+    free(lookbuf);
+}
+void transients::set_channels(int ch) {
+    channels = ch;
+    lookbuf = (float*) calloc(looksize * channels, sizeof(float));
+    lookpos = 0;
+}
+void transients::set_sample_rate(uint32_t sr) {
+    srate = sr;
+    attack_coef  = exp(log(0.01) / (0.001 * srate));
+    release_coef = exp(log(0.01) / (0.2f  * srate));
+    // due to new calculation in attack, we sometimes get harsh
+    // gain reduction/boost. 
+    // to prevent "clicks" a maxdelta is set, which allows the signal
+    // to raise/fall ~6dB/ms. 
+    maxdelta = pow(4, 1.f / (0.001 * srate));
+    calc_relfac();
+}
+void transients::set_params(float att_t, float att_l, float rel_t, float rel_l, float sust_th, int look) {
+    lookahead  = look;
+    sust_thres = sust_th;
+    att_time   = att_t;
+    rel_time   = rel_t;
+    att_level  = att_l > 0 ? 0.25f * pow(att_l * 8, 2)
+                          : -0.25f * pow(att_l * 4, 2);
+    rel_level  = rel_l > 0 ? 0.5f  * pow(rel_l * 8, 2)
+                          : -0.25f * pow(rel_l * 4, 2);
+    calc_relfac();
+}
+void transients::calc_relfac()
+{
+    relfac = pow(0.5f, 1.f / (0.001 * rel_time * srate));
+}
+void transients::process(float *in, float s) {
+    s = fabs(s);
+    // fill lookahead buffer
+    for (int i = 0; i < channels; i++) {
+        lookbuf[lookpos + i] = in[i];
+    }
+    
+    // envelope follower
+    // this is the real envelope follower curve. It raises as
+    // fast as the signal is raising and falls much slower
+    // depending on the sample rate and the ffactor
+    // (the falling factor)
+    if(s > envelope)
+        envelope = attack_coef * (envelope - s) + s;
+    else
+        envelope = release_coef * (envelope - s) + s;
+    
+    // attack follower
+    // this is a curve which follows the envelope slowly.
+    // It never can rise above the envelope. It reaches 70.7%
+    // of the envelope in a certain amount of time set by the user
+    
+    double attdelta = (envelope - attack)
+                   * 0.707
+                   / (srate * att_time * 0.001);
+    if (sustain_ended == true and envelope / attack - 1 > 0.2f)
+        sustain_ended = false;
+    attack += attdelta;
+    
+    // never raise above envelope
+    attack = std::min(envelope, attack);
+    
+    // release follower
+    // this is a curve which is always above the envelope. It
+    // starts to fall when the envelope falls beneath the
+    // sustain threshold
+        
+    if ((envelope / release) - sust_thres < 0 and sustain_ended == false) 
+        sustain_ended = true; 
+    double reldelta = sustain_ended ? relfac : 1; 
+                   
+    // release delta can never raise above 0
+    release *= reldelta;
+    
+    // never fall below envelope
+    release = std::max(envelope, release);
+    
+    // difference between attack and envelope
+    double attdiff = attack > 0 ? log(envelope / attack) : 0;
+    
+    // difference between release and envelope
+    double reldiff = envelope > 0 ? log(release / envelope) : 0;
+    
+    // amplification factor from attack and release curve
+    double ampfactor = attdiff * att_level + reldiff * rel_level;
+    old_return = new_return;
+    new_return = 1 + (ampfactor < 0 ? std::max(-1 + 1e-15, exp(ampfactor) - 1) : ampfactor);
+    if (new_return / old_return > maxdelta) 
+        new_return = old_return * maxdelta;
+    else if (new_return / old_return < 1 / maxdelta)
+        new_return = old_return / maxdelta;
+        
+    int pos = (lookpos + looksize * channels - lookahead * channels) % (looksize * channels);
+    
+    for (int i = 0; i < channels; i++)
+        in[i] = lookbuf[pos + i] * new_return;
+    
+    // advance lookpos
+    lookpos = (lookpos + channels) % (looksize * channels);
+}
+
+
+//////////////////////////////////////////////////////////////////
+
+crossover::crossover() {
+    bands     = -1;
+    mode      = -1;
+    redraw_graph = 1;
+}
+void crossover::set_sample_rate(uint32_t sr) {
+    srate = sr;
+}
+void crossover::init(int c, int b, uint32_t sr) {
+    channels = std::min(8, c);
+    bands    = std::min(8, b);
+    srate    = sr;
+    for(int b = 0; b < bands; b ++) {
+        // reset frequency settings
+        freq[b]     = 1.0;
+        active[b]   = true;
+        level[b]    = 1.0;
+        for (int c = 0; c < channels; c ++) {
+            // reset outputs
+            out[c][b] = 0.f;
+        }
+    }
+}
+float crossover::set_filter(int b, float f, bool force) {
+    // keep between neighbour bands
+    if (b)
+        f = std::max((float)freq[b-1] * 1.1f, f);
+    if (b < bands - 2)
+        f = std::min((float)freq[b+1] * 0.9f, f);
+    // restrict to 10-20k
+    f = std::max(10.f, std::min(20000.f, f));
+    // nothing changed? return
+    if (freq[b] == f and !force)
+        return freq[b];
+    freq[b] = f;
+    float q;
+    switch (mode) {
+        case 0:
+        default:
+            q = 0.5;
+            break;
+        case 1:
+            q = 0.7071068123730965;
+            break;
+        case 2:
+            q = 0.54;
+            break;
+    }
+    for (int c = 0; c < channels; c ++) {
+        if (!c) {
+            lp[c][b][0].set_lp_rbj(freq[b], q, (float)srate);
+            hp[c][b][0].set_hp_rbj(freq[b], q, (float)srate);
+        } else {
+            lp[c][b][0].copy_coeffs(lp[c-1][b][0]);
+            hp[c][b][0].copy_coeffs(hp[c-1][b][0]);
+        }
+        if (mode > 1) {
+            if (!c) {
+                lp[c][b][1].set_lp_rbj(freq[b], 1.34, (float)srate);
+                hp[c][b][1].set_hp_rbj(freq[b], 1.34, (float)srate);
+            } else {
+                lp[c][b][1].copy_coeffs(lp[c-1][b][1]);
+                hp[c][b][1].copy_coeffs(hp[c-1][b][1]);
+            }
+            lp[c][b][2].copy_coeffs(lp[c][b][0]);
+            hp[c][b][2].copy_coeffs(hp[c][b][0]);
+            lp[c][b][3].copy_coeffs(lp[c][b][1]);
+            hp[c][b][3].copy_coeffs(hp[c][b][1]);
+        } else {
+            lp[c][b][1].copy_coeffs(lp[c][b][0]);
+            hp[c][b][1].copy_coeffs(hp[c][b][0]);
+        }
+    }
+    redraw_graph = std::min(2, redraw_graph + 1);
+    return freq[b];
+}
+void crossover::set_mode(int m) {
+    if(mode == m)
+        return;
+    mode = m;
+    for(int i = 0; i < bands - 1; i ++) {
+        set_filter(i, freq[i], true);
+    }
+    redraw_graph = std::min(2, redraw_graph + 1);
+}
+void crossover::set_active(int b, bool a) {
+    if (active[b] == a)
+        return;
+    active[b] = a;
+    redraw_graph = std::min(2, redraw_graph + 1);
+}
+void crossover::set_level(int b, float l) {
+    if (level[b] == l)
+        return;
+    level[b] = l;
+    redraw_graph = std::min(2, redraw_graph + 1);
+}
+void crossover::process(float *data) {
+    for (int c = 0; c < channels; c++) {
+        for(int b = 0; b < bands; b ++) {
+            out[c][b] = data[c];
+            for (int f = 0; f < get_filter_count(); f++){
+                if(b + 1 < bands) {
+                    out[c][b] = lp[c][b][f].process(out[c][b]);
+                    lp[c][b][f].sanitize();
+                }
+                if(b - 1 >= 0) {
+                    out[c][b] = hp[c][b - 1][f].process(out[c][b]);
+                    hp[c][b - 1][f].sanitize();
+                }
+            }
+            out[c][b] *= level[b];
+        }
+    }
+}
+float crossover::get_value(int c, int b) {
+    return out[c][b];
+}
+bool crossover::get_graph(int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
+{
+    if (subindex >= bands) {
+        redraw_graph = std::max(0, redraw_graph - 1);
+        return false;
+    }
+    float ret;
+    double freq;
+    for (int i = 0; i < points; i++) {
+        ret = 1.f;
+        freq = 20.0 * pow (20000.0 / 20.0, i * 1.0 / points);
+        for(int f = 0; f < get_filter_count(); f ++) {
+            if(subindex < bands -1)
+                ret *= lp[0][subindex][f].freq_gain(freq, (float)srate);
+            if(subindex > 0)
+                ret *= hp[0][subindex - 1][f].freq_gain(freq, (float)srate);
+        }
+        ret *= level[subindex];
+        context->set_source_rgba(0.15, 0.2, 0.0, !active[subindex] ? 0.3 : 0.8);
+        data[i] = dB_grid(ret);
+    }
+    return true;
+}
+bool crossover::get_layers(int index, int generation, unsigned int &layers) const
+{
+    layers = 0 | (redraw_graph or !generation ? LG_CACHE_GRAPH : 0) | (!generation ? LG_CACHE_GRID : 0);
+    return redraw_graph or !generation;
+}
+
+int crossover::get_filter_count() const
+{
+    switch (mode) {
+        case 0:
+        default:
+            return 1;
+        case 1:
+            return 2;
+        case 2:
+            return 4;
+    }
+}
+
+//////////////////////////////////////////////////////////////////
+
+bitreduction::bitreduction()
+{
+    coeff        = 1;
+    morph        = 0;
+    mode         = 0;
+    dc           = 0;
+    sqr          = 0;
+    aa           = 0;
+    aa1          = 0;
+    redraw_graph = true;
+    bypass       = true;
+}
+void bitreduction::set_sample_rate(uint32_t sr)
+{
+    srate = sr;
+}
+void bitreduction::set_params(float b, float mo, bool bp, uint32_t md, float d, float a)
+{
+    morph        = 1 - mo;
+    bypass       = bp;
+    dc           = d;
+    aa           = a;
+    mode         = md;
+    coeff        = powf(2.0f, b) - 1;
+    sqr          = sqrt(coeff / 2);
+    aa1          = (1.f - aa) / 2.f;
+    redraw_graph = true;
+}
+float bitreduction::add_dc(float s, float dc) const
+{
+    return s > 0 ? s *= dc : s /= dc;
+}
+float bitreduction::remove_dc(float s, float dc) const
+{
+    return s > 0 ? s /= dc : s *= dc;
+}
+float bitreduction::waveshape(float in) const
+{
+    double y;
+    double k;
+    
+    // add dc
+    in = add_dc(in, dc);
+    
+    // main rounding calculation depending on mode
+    
+    // the idea for anti-aliasing:
+    // you need a function f which brings you to the scale, where you want to round
+    // and the function f_b (with f(f_b)=id) which brings you back to your original scale.
+    //
+    // then you can use the logic below in the following way:
+    // y = f(in) and k = roundf(y)
+    // if (y > k + aa1)
+    //      k = f_b(k) + ( f_b(k+1) - f_b(k) ) *0.5 * (sin(x - PI/2) + 1)
+    // if (y < k + aa1)
+    //      k = f_b(k) - ( f_b(k+1) - f_b(k) ) *0.5 * (sin(x - PI/2) + 1)
+    //
+    // whereas x = (fabs(f(in) - k) - aa1) * PI / aa 
+    // for both cases.
+    
+    switch (mode) {
+        case 0:
+        default:
+            // linear
+            y = (in) * coeff;
+            k = roundf(y);
+            if (k - aa1 <= y and y <= k + aa1) {
+                k /= coeff;
+            } else if (y > k + aa1) {
+                k = k / coeff + ((k + 1) / coeff - k / coeff) * 0.5 * (sin(M_PI * (fabs(y - k) - aa1) / aa - M_PI_2) + 1);
+            } else {
+                k = k / coeff - (k / coeff - (k - 1) / coeff) * 0.5 * (sin(M_PI * (fabs(y - k) - aa1) / aa - M_PI_2) + 1);
+            }
+            break;
+        case 1:
+            // logarithmic
+            y = sqr * log(fabs(in)) + sqr * sqr;
+            k = roundf(y);
+            if(!in) {
+                k = 0;
+            } else if (k - aa1 <= y and y <= k + aa1) {
+                k = in / fabs(in) * exp(k / sqr - sqr);
+            } else if (y > k + aa1) {
+                k = in / fabs(in) * (exp(k / sqr - sqr) + (exp((k + 1) / sqr - sqr) - exp(k / sqr - sqr)) * 0.5 * (sin((fabs(y - k) - aa1) / aa * M_PI - M_PI_2) + 1));
+            } else {
+                k = in / fabs(in) * (exp(k / sqr - sqr) - (exp(k / sqr - sqr) - exp((k - 1) / sqr - sqr)) * 0.5 * (sin((fabs(y - k) - aa1) / aa * M_PI - M_PI_2) + 1));
+            }
+            break;
+    }
+            
+    // morph between dry and wet signal
+    k += (in - k) * morph;
+    
+    // remove dc
+    k = remove_dc(k, dc);
+    
+    return k;
+}
+float bitreduction::process(float in)
+{
+    return waveshape(in);
+}
+
+bool bitreduction::get_layers(int index, int generation, unsigned int &layers) const
+{
+    layers = redraw_graph or !generation ? LG_CACHE_GRAPH | LG_CACHE_GRID : 0;
+    return redraw_graph or !generation;
+}
+bool bitreduction::get_graph(int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
+{
+    if (subindex >= 2) {
+        redraw_graph = false;
+        return false;
+    }
+    for (int i = 0; i < points; i++) {
+         data[i] = sin(((float)i / (float)points * 360.) * M_PI / 180.);
+        if (subindex and !bypass)
+            data[i] = waveshape(data[i]);
+        else {
+            context->set_line_width(1);
+            context->set_source_rgba(0.15, 0.2, 0.0, 0.15);
+        }
+    }
+    return true;
+}
+bool bitreduction::get_gridline(int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
+{
+    if (phase or subindex)
+        return false;
+    pos = 0;
+    vertical = false;
+    return true;
+}
+
+//////////////////////////////////////////////////////////////////
+
+resampleN::resampleN()
+{
+    factor  = 2;
+    srate   = 0;
+    filters = 2;
+}
+resampleN::~resampleN()
+{
+    free(tmp);
+}
+void resampleN::set_params(uint32_t sr, int fctr = 2, int fltrs = 2)
+{
+    srate   = sr;
+    factor  = std::min(16, std::max(1, fctr));
+    filters = std::min(4, std::max(1, fltrs));
+    // set all filters
+    filter[0][0].set_lp_rbj(std::max(25000., (double)srate / 2), 0.8, (float)srate * factor);
+    for (int i = 1; i < filters; i++) {
+        filter[0][i].copy_coeffs(filter[0][0]);
+        filter[1][i].copy_coeffs(filter[0][0]);
+    }
+}
+double *resampleN::upsample(double sample)
+{
+    tmp[0] = sample;
+    if (factor > 1) {
+        for (int f = 0; f < filters; f++)
+            tmp[0] = filter[0][f].process(sample);
+        for (int i = 1; i < factor; i++) {
+            tmp[i] = 0;
+            for (int f = 0; f < filters; f++)
+                tmp[i] = filter[0][f].process(sample);
+        }
+    }
+    return tmp;
+}
+double resampleN::downsample(double *sample)
+{
+    if (factor > 1) {
+        for(int i = 0; i < factor; i++) {
+            for (int f = 0; f < filters; f++) {
+                sample[i] = filter[1][f].process(sample[i]);
+            }
+        }
+    }
+    return sample[0];
+}
+
+//////////////////////////////////////////////////////////////////
+
+samplereduction::samplereduction()
+{
+    target  = 0;
+    real    = 0;
+    samples = 0;
+    last    = 0;
+    amount  = 0;
+    round   = 0;
+}
+void samplereduction::set_params(float am)
+{
+    amount  = am;
+    round   = roundf(amount);
+    //samples = round;
+}
+double samplereduction::process(double in)
+{
+    samples ++;
+    if (samples >= round) {
+        target += amount;
+        real += round;
+        if (target + amount >= real + 1) {
+            last = in;
+            target = 0;
+            real   = 0;
+        }
+        samples = 0;
+    }
+    return last;
+}
