@@ -25,6 +25,7 @@
 #include "delay.h"
 #include "fixed_point.h"
 #include "inertia.h"
+#include "giface.h"
 #include "onepole.h"
 #include <complex>
 
@@ -441,7 +442,7 @@ public:
 class biquad_filter_module: public filter_module_iface
 {
 private:
-    dsp::biquad_d1<float> left[3], right[3];
+    dsp::biquad_d1 left[3], right[3];
     int order;
 
 public:
@@ -512,37 +513,6 @@ public:
     }
 };
 
-/// Tom Szilagyi's distortion code, used with permission
-/// KF: I'm not 100% sure how this is supposed to work, but it does.
-/// I'm planning to rewrite it using more modular approach when I have more time.
-class tap_distortion {
-private:
-    float blend_old, drive_old;
-    float meter;
-    float rdrive, rbdr, kpa, kpb, kna, knb, ap, an, imr, kc, srct, sq, pwrq;
-    float prev_med, prev_out;
-public:
-    uint32_t srate;
-    bool is_active;
-    tap_distortion();
-    void activate();
-    void deactivate();
-    void set_params(float blend, float drive);
-    void set_sample_rate(uint32_t sr);
-    float process(float in);
-    float get_distortion_level();
-    static inline float M(float x)
-    {
-        return (fabs(x) > 0.000000001f) ? x : 0.0f;
-    }
-
-    static inline float D(float x)
-    {
-        x = fabs(x);
-        return (x > 0.000000001f) ? sqrtf(x) : 0.0f;
-    }
-};
-
 /// LFO module by Markus
 /// This module provides simple LFO's (sine=0, triangle=1, square=2, saw_up=3, saw_down=4)
 /// get_value() returns a value between -1 and 1
@@ -555,6 +525,10 @@ private:
 public:
     simple_lfo();
     void set_params(float f, int m, float o, uint32_t sr, float amount = 1.f);
+    void set_freq(float f);
+    void set_mode(int m);
+    void set_amount(float a);
+    void set_offset(float o);
     float get_value();
     void advance(uint32_t count);
     void set_phase(float ph);
@@ -602,14 +576,15 @@ public:
     float asc_coeff;
     bool _asc_used;
     static inline void denormal(volatile float *f) {
-	    *f += 1e-18;
-	    *f -= 1e-18;
+        *f += 1e-18;
+        *f -= 1e-18;
     }
     inline float get_rdelta(float peak, float _limit, float _att, bool _asc = true);
     void reset();
     void reset_asc();
     bool get_asc();
     lookahead_limiter();
+    ~lookahead_limiter();
     void set_multi(bool set);
     void process(float &left, float &right, float *multi_buffer);
     void set_sample_rate(uint32_t sr);
@@ -619,6 +594,198 @@ public:
     void deactivate();
 };
 
+class transients {
+private:
+    double attack_coef, release_coef;
+public:
+    double envelope, attack, release;
+    bool sustain_ended;
+    double old_return, new_return, maxdelta, relfac;
+    float att_time, att_level, rel_time, rel_level, sust_thres;
+    static const int looksize = 101;
+    int lookahead, lookpos;
+    float *lookbuf;
+    int channels;
+    uint32_t srate;
+    transients();
+    ~transients();
+    void calc_relfac();
+    void process(float *in, float s);
+    void waveshape(float *in);
+    void set_channels(int ch);
+    void set_sample_rate(uint32_t sr);
+    void set_params(float att_t, float att_l, float rel_t, float rel_l, float sust_th, int look);
+    int cnt;
+};
+
+
+class crossover {
+private:
+public:
+    int channels, bands, mode;
+    float freq[8], active[8], level[8], out[8][8];
+    dsp::biquad_d2 lp[8][8][4], hp[8][8][4];
+    mutable int redraw_graph;
+    uint32_t srate;
+    crossover();
+    void process(float *data);
+    float get_value(int c, int b);
+    void set_sample_rate(uint32_t sr);
+    float set_filter(int b, float f, bool force = false);
+    void set_level(int b, float l);
+    void set_active(int b, bool a);
+    void set_mode(int m);
+    int get_filter_count() const;
+    void init(int c, int b, uint32_t sr);
+    virtual bool get_graph(int subindex, int phase, float *data, int points, calf_plugins::cairo_iface *context, int *mode) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
+};
+
+class bitreduction
+{
+private:
+public:
+    float morph, coeff, dc, sqr, aa, aa1;
+    bool bypass;
+    uint32_t mode, srate;
+    mutable bool redraw_graph;
+    bitreduction();
+    void set_sample_rate(uint32_t sr);
+    float add_dc(float s, float dc) const;
+    float remove_dc(float s, float dc) const;
+    void set_params(float b, float m, bool bp, uint32_t mode, float dc, float aa);
+    float waveshape(float in) const;
+    float process(float in);
+    virtual bool get_graph(int subindex, int phase, float *data, int points, calf_plugins::cairo_iface *context, int *mode) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
+    bool get_gridline(int subindex, int phase, float &pos, bool &vertical, std::string &legend, calf_plugins::cairo_iface *context) const;
+};
+
+class resampleN
+{
+private:
+public:
+    uint32_t srate; // sample rate; source for upsampling, target for downsampling
+    int factor; // number of added/removed samples, max 16
+    int filters; // how many lowpasses should be used, max 4
+    double tmp[16];
+    dsp::biquad_d2 filter[2][4];
+    resampleN();
+    ~resampleN();
+    void set_params(uint32_t sr, int factor, int filters);
+    double *upsample(double sample);
+    double downsample(double *sample);
+};
+
+class samplereduction
+{
+private:
+public:
+    float target, real, amount;
+    uint samples, round, cnt;
+    double last;
+    samplereduction();
+    void set_params(float am);
+    double process(double in);
+};
+
+
+/// Tom Szilagyi's distortion code, used with permission
+/// KF: I'm not 100% sure how this is supposed to work, but it does.
+/// I'm planning to rewrite it using more modular approach when I have more time.
+class tap_distortion {
+private:
+    float blend_old, drive_old;
+    float meter;
+    float rdrive, rbdr, kpa, kpb, kna, knb, ap, an, imr, kc, srct, sq, pwrq;
+    int over;
+    float prev_med, prev_out;
+    resampleN resampler;
+public:
+    uint32_t srate;
+    bool is_active;
+    tap_distortion();
+    void activate();
+    void deactivate();
+    void set_params(float blend, float drive);
+    void set_sample_rate(uint32_t sr);
+    float process(float in);
+    float get_distortion_level();
+    static inline float M(float x)
+    {
+        return (fabs(x) > 0.00000001f) ? x : 0.0f;
+    }
+
+    static inline float D(float x)
+    {
+        x = fabs(x);
+        return (x > 0.00000001f) ? sqrtf(x) : 0.0f;
+    }
+};
+
+/*
+   Class for warping clicks avoiding between chunks
+   Used in reverse delay
+*/
+class overlap_window
+{
+private:
+    float val;
+    float step;
+    float acc;
+    unsigned int active_samples;
+    unsigned int full_samples;
+    unsigned int counter;
+
+public:
+    overlap_window() {
+        val = 0;
+        step = 0;
+        acc = 0;
+        active_samples = 0;
+        full_samples = 0;
+        counter = 0;
+    }
+    void set_coef(float t /* 0..1 */, unsigned int full_samples) {
+        set_full(0, full_samples, t*full_samples);
+    }
+    bool set_full(float min_val, unsigned int full_samples, unsigned int active_samples) {
+        if(active_samples >= full_samples) return false;
+
+        acc = min_val;
+        val = min_val;
+        this->full_samples = full_samples;
+        this->active_samples = active_samples;
+        counter = 0;
+        step = (1 - min_val)/(active_samples/2);
+
+        return true;
+    }
+    float get() {
+        if(counter >= 0 && counter < active_samples/2) {
+            acc += step;
+            counter++;
+            return acc;
+        }
+        else if(counter >= active_samples/2 && counter <= full_samples - active_samples/2) {
+            counter++;
+            return 1;
+        }
+        else if(counter > full_samples - active_samples/2 && counter < full_samples) {
+            acc -= step;
+            counter++;
+            return acc;
+        }
+        else if(counter >= full_samples) {
+            float ret_val = acc;
+            acc = val;
+            counter = 0;
+            return ret_val;
+        }
+        return 1;
+    }
+};
+
 #if 0
 { to keep editor happy
 #endif
