@@ -27,6 +27,7 @@
 #include <exception>
 #include <string>
 #include <vector>
+#include <cairo/cairo.h>
 
 namespace osctl {
     struct osc_client;
@@ -38,6 +39,8 @@ enum {
     MAX_SAMPLE_RUN = 256
 };
     
+struct automation_range;
+
 /// Values ORed together for flags field in parameter_properties
 enum parameter_flags
 {
@@ -57,17 +60,17 @@ enum parameter_flags
   PF_SCALE_QUAD = 0x50, ///< quadratic scale (decent for some gain/amplitude values)
   PF_SCALE_LOG_INF = 0x60, ///< log scale + +inf (FAKE_INFINITY)
 
-  PF_CTLMASK =     0x0F00, ///< bit mask for control type
-  PF_CTL_DEFAULT = 0x0000, ///< try to figure out automatically
-  PF_CTL_KNOB =    0x0100, ///< knob
-  PF_CTL_FADER =   0x0200, ///< fader (slider)
-  PF_CTL_TOGGLE =  0x0300, ///< toggle button
-  PF_CTL_COMBO =   0x0400, ///< combo box
-  PF_CTL_RADIO =   0x0500, ///< radio button
-  PF_CTL_BUTTON =  0x0600, ///< push button
-  PF_CTL_METER  =  0x0700, ///< volume meter
-  PF_CTL_LED    =  0x0800, ///< light emitting diode
-  PF_CTL_LABEL  =  0x0900, ///< label
+  PF_CTLMASK =        0x000F00, ///< bit mask for control type
+  PF_CTL_DEFAULT =    0x000000, ///< try to figure out automatically
+  PF_CTL_KNOB =       0x000100, ///< knob
+  PF_CTL_FADER =      0x000200, ///< fader (slider)
+  PF_CTL_TOGGLE =     0x000300, ///< toggle button
+  PF_CTL_COMBO =      0x000400, ///< combo box
+  PF_CTL_RADIO =      0x000500, ///< radio button
+  PF_CTL_BUTTON =     0x000600, ///< push button
+  PF_CTL_METER  =     0x000700, ///< volume meter
+  PF_CTL_LED    =     0x000800, ///< light emitting diode
+  PF_CTL_LABEL  =     0x000900, ///< label
   
   PF_CTLOPTIONS     = 0x00F000, ///< bit mask for control (widget) options
   PF_CTLO_HORIZ     = 0x001000, ///< horizontal version of the control (unused)
@@ -75,14 +78,15 @@ enum parameter_flags
   PF_CTLO_LABEL     = 0x004000, ///< add a text display to the control (meters only)
   PF_CTLO_REVERSE   = 0x008000, ///< use VU_MONOCHROME_REVERSE mode (meters only)
 
+  PF_PROP_MASK     =  0x3F0000, ///< bit mask for properties
   PF_PROP_NOBOUNDS =  0x010000, ///< no epp:hasStrictBounds
   PF_PROP_EXPENSIVE = 0x020000, ///< epp:expensive, may trigger expensive calculation
-  PF_PROP_OUTPUT_GAIN=0x050000, ///< epp:outputGain + skip epp:hasStrictBounds
-  PF_PROP_OUTPUT    = 0x080000, ///< output port
-  PF_PROP_OPTIONAL  = 0x100000, ///< connection optional
-  PF_PROP_GRAPH     = 0x200000, ///< add graph
-  
-  PF_UNITMASK     = 0xFF000000,  ///< bit mask for units   \todo reduce to use only 5 bits
+  PF_PROP_OUTPUT_GAIN=0x040000, ///< epp:outputGain + skip epp:hasStrictBounds
+  PF_PROP_OPTIONAL  = 0x080000, ///< connection optional
+  PF_PROP_GRAPH     = 0x100000, ///< add graph
+  PF_PROP_OUTPUT    = 0x200000, ///< output port (flag, cannot be combined with others)
+
+  PF_UNITMASK     = 0x0F000000,  ///< bit mask for units   \todo reduce to use only 5 bits
   PF_UNIT_DB      = 0x01000000,  ///< decibels
   PF_UNIT_COEF    = 0x02000000,  ///< multiply-by factor
   PF_UNIT_HZ      = 0x03000000,  ///< Hertz
@@ -94,6 +98,16 @@ enum parameter_flags
   PF_UNIT_DEG     = 0x09000000,  ///< degrees
   PF_UNIT_NOTE    = 0x0A000000,  ///< MIDI note number
   PF_UNIT_RPM     = 0x0B000000,  ///< revolutions per minute
+  PF_UNIT_SAMPLES = 0x0C000000,  ///< samples
+  
+  PF_SYNC_BPM     = 0x10000000, ///< sync a bpm setting with the host environment
+
+  PF_DIGITMASK    = 0xE0000000,
+  PF_DIGIT_0      = 0x20000000,
+  PF_DIGIT_1      = 0x40000000,
+  PF_DIGIT_2      = 0x60000000,
+  PF_DIGIT_3      = 0x80000000,
+  PF_DIGIT_ALL    = 0xA0000000,
 };
 
 /// A fake infinity value (because real infinity may break some hosts)
@@ -134,6 +148,8 @@ struct parameter_properties
     double to_01(float value) const;
     /// stringify (in sensible way)
     std::string to_string(float value) const;
+    /// convert string (from text entry) to value
+    float string_to_value(const char* string) const;
     /// get required width (for reserving GUI space)
     int get_char_count() const;
     /// get increment step based on step value (if specified) and other factors
@@ -142,8 +158,12 @@ struct parameter_properties
 
 struct cairo_iface
 {
+    cairo_t *context;
+    int size_x, size_y, pad_x, pad_y;
     virtual void set_source_rgba(float r, float g, float b, float a = 1.f) = 0;
     virtual void set_line_width(float width) = 0;
+    virtual void set_dash(const double *dash, int length) = 0;
+    virtual void draw_label(const char *label, float x, float y, int pos, float margin, float align) = 0;
     virtual ~cairo_iface() {}
 };
 
@@ -153,52 +173,75 @@ struct progress_report_iface
     virtual ~progress_report_iface() {}
 };
 
+/// possible bit masks for get_layers
+enum layers_flags {
+    LG_NONE            = 0x000000,
+    LG_CACHE_GRID      = 0x000001,
+    LG_REALTIME_GRID   = 0x000002,
+    LG_CACHE_GRAPH     = 0x000004,
+    LG_REALTIME_GRAPH  = 0x000008,
+    LG_CACHE_DOT       = 0x000010,
+    LG_REALTIME_DOT    = 0x000020,
+    LG_CACHE_MOVING    = 0x000040,
+    LG_REALTIME_MOVING = 0x000080
+};
+
+/// possible values for get_moving
+enum moving_directions {
+    LG_MOVING_LEFT     = 0x000000,
+    LG_MOVING_RIGHT    = 0x000001,
+    LG_MOVING_UP       = 0x000002,
+    LG_MOVING_DOWN     = 0x000004
+};
+
 /// 'provides live line graph values' interface
 struct line_graph_iface
 {
     /// Obtain subindex'th graph of parameter 'index'
     /// @param index parameter/graph number (usually tied to particular plugin control port)
     /// @param subindex graph number (there may be multiple overlaid graphs for one parameter, eg. for monosynth 2x12dB filters)
+    /// @param phase 0 if in cache phase or 1 if in realtime phase
     /// @param data buffer for normalized output values
     /// @param points number of points to fill
     /// @param context cairo context to adjust (for multicolour graphs etc.)
     /// @retval true graph data was returned; subindex+1 graph may or may not be available
     /// @retval false graph data was not returned; subindex+1 graph does not exist either
-    virtual bool get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode = 0) const { return false; }
-
-    /// Obtain subindex'th dot of parameter 'index'
-    /// @param index parameter/dot number (usually tied to particular plugin control port)
-    /// @param subindex dot number (there may be multiple dots graphs for one parameter)
-    virtual bool get_dot(int index, int subindex, float &x, float &y, int &size, cairo_iface *context) const { return false; }
+    virtual bool get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode = 0) const { return false; }
     
-    /// Obtain subindex'th dot of parameter 'index'
+    /// Obtain subindex'th moving surface of parameter 'index'
     /// @param index parameter/dot number (usually tied to particular plugin control port)
-    /// @param subindex dot number (there may be multiple dots graphs for one parameter)
-    virtual bool get_gridline(int index, int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const { return false; }
-    
-    /// Obtain subindex'th static graph of parameter index (static graphs are only dependent on parameter value, not plugin state)
-    /// @param index parameter/graph number (usually tied to particular plugin control port)
-    /// @param subindex graph number (there may be multiple overlaid graphs for one parameter, eg. for monosynth 2x12dB filters)
-    /// @param value parameter value to pick the graph for
+    /// @param subindex mvoing line number (there may be multiple graphs for one parameter)
+    /// @param direction 0 if horizontal or 1 if vertical movement
     /// @param data buffer for normalized output values
-    /// @param points number of points to fill
-    /// @param context cairo context to adjust (for multicolour graphs etc.)
-    /// @retval true graph data was returned; subindex+1 graph may or may not be available
-    /// @retval false graph data was not returned; subindex+1 graph does not exist either
-    virtual bool get_static_graph(int index, int subindex, float value, float *data, int points, cairo_iface *context) const { return false; }
+    /// @param x number of points direction to fill in x
+    /// @param y number of points direction to fill in y
+    virtual bool get_moving(int index, int subindex, int &direction, float *data, int x, int y, int &offset, uint32_t &color) const { return false; }
     
-    /// Return which graphs need to be redrawn and which can be cached for later reuse
-    /// @param index Parameter/graph number (usually tied to particular plugin control port)
-    /// @param generation 0 (at start) or the last value returned by the function (corresponds to a set of input values)
-    /// @param subindex_graph First graph that has to be redrawn (because it depends on values that might have changed)
-    /// @param subindex_dot First dot that has to be redrawn
-    /// @param subindex_gridline First gridline/legend that has to be redrawn
-    /// @retval Current generation (to pass when calling the function next time); if different than passed generation value, call the function again to retrieve which graph offsets should be put into cache
-    virtual int get_changed_offsets(int index, int generation, int &subindex_graph, int &subindex_dot, int &subindex_gridline) const { subindex_graph = subindex_dot = subindex_gridline = 0; return 0; }
+    /// Obtain subindex'th dot of parameter 'index'
+    /// @param index parameter/dot number (usually tied to particular plugin control port)
+    /// @param subindex dot number (there may be multiple dots graphs for one parameter)
+    /// @param phase 0 if in cache phase or 1 if in realtime phase
+    virtual bool get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const { return false; }
     
-    /// Return if a graph should redraw completely
-    /// @param index Parameter/graph number (usually tied to particular plugin control port)
-    virtual bool get_clear_all(int index) const { return false; }
+    /// Obtain subindex'th dot of parameter 'index'
+    /// @param index parameter/dot number (usually tied to particular plugin control port)
+    /// @param subindex dot number (there may be multiple dots graphs for one parameter)
+    /// @param phase 0 if in cache phase or 1 if in realtime phase
+    virtual bool get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const { return false; }
+    
+    /// Retrun which layers need to be redrawn in the next GTK drawing cycle
+    /// @param index Parameter/graph identifier (usually tied to particular plugin control port)
+    /// @param generation The overall amount of drawing cycles since the last full refresh of all surfaces
+    /// @param layers Bitmask defining the layers to be redrawn (see layers_flags above)
+    /// @retval true there's at least one layer to be redrawn; false nothing to draw in this cycle
+    virtual bool get_layers(int index, int generation, unsigned int &layers) const { return false; }
+    
+    /// Return a label for the crosshairs they are enabled
+    /// @param x Position of the mouse pointer in x direction
+    /// @param y Position of the mouse pointer in y direction
+    /// @param sx Horizontal size of the widget in pixels
+    /// @param sy Vertical size of the widget in pixels
+    virtual std::string get_crosshair_label( int x, int y, int sx, int sy, cairo_iface *context ) const { std::string s = ""; return s; }
     
     /// Standard destructor to make compiler happy
     virtual ~line_graph_iface() {}
@@ -288,6 +331,7 @@ struct ladspa_plugin_info
 /// An interface returning metadata about a plugin
 struct plugin_metadata_iface
 {
+    enum { simulate_stereo_input = true };
     /// @return plugin long name
     virtual const char *get_name() const = 0;
     /// @return plugin LV2 label
@@ -329,9 +373,11 @@ struct plugin_metadata_iface
     /// does the plugin require string port extension? (or DSSI configure) may be slow
     virtual bool requires_configure() const = 0;
     /// obtain array of names of configure variables (or NULL is none needed)
-    virtual const char *const *get_configure_vars() const { return NULL; }
+    virtual void get_configure_vars(std::vector<std::string> &names) const { names.clear(); }
     /// @return table_metadata_iface if any
     virtual const table_metadata_iface *get_table_metadata_iface(const char *key) const { return NULL; }
+    /// @return whether to auto-connect right input with left input if unconnected
+    virtual bool get_simulate_stereo_input() const = 0;
 
     /// Do-nothing destructor to silence compiler warning
     virtual ~plugin_metadata_iface() {}
@@ -368,6 +414,19 @@ struct plugin_ctl_iface
     virtual const line_graph_iface *get_line_graph_iface() const = 0;
     /// @return phase_graph_iface if any
     virtual const phase_graph_iface *get_phase_graph_iface() const = 0;
+    /// @return serial number of last automation write (JACK host only)
+    virtual int get_write_serial(int param_no) { return 0; }
+    
+    /// Add or update parameter automation routing
+    virtual void add_automation(uint32_t source, const automation_range &dest) {}
+    /// Remove parameter automation routing        
+    virtual void delete_automation(uint32_t source, int param_no) {}
+    /// Retrieve automation list for a given parameter
+    /// @param param_no parameter to retrieve automation list for, or -1 for all
+    virtual void get_automation(int param_no, std::multimap<uint32_t, automation_range> &dests) {}
+    /// Return the source identifier for the most recently seen control change that could be used for automation
+    virtual uint32_t get_last_automation_source() { return 0xFFFFFFFF; }
+    virtual void send_automation_configures(send_configure_iface *) {}
     /// Do-nothing destructor to silence compiler warning
     virtual ~plugin_ctl_iface() {}
 };
@@ -433,7 +492,7 @@ struct audio_module_iface
     /// Reset parameter values for epp:trigger type parameters (ones activated by oneshot push button instead of check box)
     virtual void params_reset() = 0;
     /// Called after instantiating (after all the feature pointers are set - including interfaces like progress_report_iface)
-    virtual void post_instantiate() = 0;
+    virtual void post_instantiate(uint32_t sample_rate) = 0;
     /// Return the arrays of port buffer pointers
     virtual void get_port_arrays(float **&ins_ptrs, float **&outs_ptrs, float **&params_ptrs) = 0;
     /// Return metadata object
@@ -465,6 +524,8 @@ public:
     float *ins[Metadata::in_count]; 
     float *outs[Metadata::out_count];
     float *params[Metadata::param_count];
+    bool questionable_data_reported_in;
+    bool questionable_data_reported_out;
 
     progress_report_iface *progress_report;
 
@@ -473,6 +534,8 @@ public:
         memset(ins, 0, sizeof(ins));
         memset(outs, 0, sizeof(outs));
         memset(params, 0, sizeof(params));
+        questionable_data_reported_in = false;
+        questionable_data_reported_out = false;
     }
 
     /// Handle MIDI Note On
@@ -508,7 +571,7 @@ public:
     /// Reset parameter values for epp:trigger type parameters (ones activated by oneshot push button instead of check box)
     void params_reset() {}
     /// Called after instantiating (after all the feature pointers are set - including interfaces like progress_report_iface)
-    void post_instantiate() {}
+    void post_instantiate(uint32_t) {}
     /// Handle 'message context' port message
     /// @arg output_ports pointer to bit array of output port "changed" flags, note that 0 = first audio input, not first parameter (use input_count + output_count)
     uint32_t message_run(const void *valid_ports, void *output_ports) { 
@@ -539,15 +602,54 @@ public:
     /// utility function: call process, and if it returned zeros in output masks, zero out the relevant output port buffers
     uint32_t process_slice(uint32_t offset, uint32_t end)
     {
+        bool had_errors = false;
+        for (int i=0; i<Metadata::in_count; ++i) {
+            float *indata = ins[i];
+            if (indata) {
+                float errval = 0;
+                for (uint32_t j = offset; j < end; j++)
+                {
+                    if (!std::isfinite(indata[j]) || fabs(indata[j]) > 4294967296.0) {
+                        errval = indata[j];
+                        had_errors = true;
+                    }
+                }
+                if (had_errors && !questionable_data_reported_in) {
+                    fprintf(stderr, "Warning: Plugin %s got questionable value %f on its input %d\n", Metadata::get_name(), errval, i);
+                    questionable_data_reported_in = true;
+                }
+            }
+        }
         uint32_t total_out_mask = 0;
         while(offset < end)
         {
             uint32_t newend = std::min(offset + MAX_SAMPLE_RUN, end);
-            uint32_t out_mask = process(offset, newend - offset, -1, -1);
+            uint32_t out_mask = !had_errors ? process(offset, newend - offset, -1, -1) : 0;
             total_out_mask |= out_mask;
             zero_by_mask(out_mask, offset, newend - offset);
             offset = newend;
         }
+        for (int i=0; i<Metadata::out_count; ++i) {
+            if (total_out_mask & (1 << i))
+            {
+                had_errors = false;
+                float *outdata = outs[i];
+                float errval = 0;
+                for (uint32_t j = offset; j < end; j++)
+                {
+                    if (!std::isfinite(outdata[j]) || fabs(outdata[j]) > 4294967296.0) {
+                        errval = outdata[j];
+                        had_errors = true;
+                    }
+                }
+                if (had_errors && !questionable_data_reported_out) {
+                    fprintf(stderr, "Warning: Plugin %s generated questionable value %f on its output %d - this is most likely a bug in the plugin!\n", Metadata::get_name(), errval, i);
+                    questionable_data_reported_out = true;
+                }
+                if (had_errors)
+                    dsp::zero(outs[i] + offset, end - offset);
+            }
+        }
         return total_out_mask;
     }
     /// @return line_graph_iface if any
@@ -629,6 +731,7 @@ public:
     bool is_noisy(int param_no) const { return false; }
     const ladspa_plugin_info &get_plugin_info() const { return plugin_info; }
     bool requires_configure() const { return false; }
+    bool get_simulate_stereo_input() const { return Metadata::simulate_stereo_input; }
 };
 
 #define CALF_PORT_NAMES(name) template<> const char *::plugin_metadata<name##_metadata>::port_names[]
@@ -654,9 +757,9 @@ static bool get_graph(Fx &fx, int subindex, float *data, int points, float res =
 {
     for (int i = 0; i < points; i++)
     {
-        typedef std::complex<double> cfloat;
+        //typedef std::complex<double> cfloat;
         double freq = 20.0 * pow (20000.0 / 20.0, i * 1.0 / points);
-        data[i] = dB_grid(fx.freq_gain(subindex, freq, fx.srate), res, ofs);
+        data[i] = dB_grid(fx.freq_gain(subindex, freq), res, ofs);
     }
     return true;
 }
@@ -671,12 +774,18 @@ static inline float dB_grid_inv(float pos)
 class frequency_response_line_graph: public line_graph_iface 
 {
 public:
-    bool get_gridline(int index, int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
-    virtual int get_changed_offsets(int index, int generation, int &subindex_graph, int &subindex_dot, int &subindex_gridline) const;
+    mutable bool redraw_graph;
+    virtual bool get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
+    virtual bool get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const;
+    virtual bool get_layers(int index, int generation, unsigned int &layers) const;
+    virtual float freq_gain(int index, double freq) const { return 0; };
+    virtual std::string get_crosshair_label( int x, int y, int sx, int sy, cairo_iface *context ) const;
 };
 
 /// set drawing color based on channel index (0 or 1)
-void set_channel_color(cairo_iface *context, int channel);
+void set_channel_color(cairo_iface *context, int channel, float alpha = 0.6);
+void set_channel_dash(cairo_iface *context, int channel);
+void draw_cairo_label(cairo_iface *context, const char *label, float x, float y, int pos, float margin, float align);
 
 struct preset_access_iface
 {
@@ -706,6 +815,8 @@ public:
     mod_matrix_metadata(unsigned int _rows, const char **_src_names, const char **_dest_names);
     virtual const table_column_info *get_table_columns() const;
     virtual uint32_t get_table_rows() const;
+    /// Return a list of configure variables used by the modulation matrix
+    void get_configure_vars(std::vector<std::string> &names) const;
 
 protected:
     /// Column descriptions for table widget
@@ -738,6 +849,40 @@ public:
 };
 #endif
 
+struct automation_range
+{
+    float min_value;
+    float max_value;
+    int param_no;
+    
+    automation_range(float l, float u, int param)
+    : min_value(l)
+    , max_value(u)
+    , param_no(param)
+    {}
+    void send_configure(const plugin_metadata_iface *metadata, uint32_t from_controller, send_configure_iface *sci);
+    static automation_range *new_from_configure(const plugin_metadata_iface *metadata, const char *key, const char *value, uint32_t &from_controller);
+};
+
+struct automation_map: public std::multimap<uint32_t, automation_range>
+{
+};
+
+inline float subindex_to_freq(int subindex)
+{
+  float freq = 100;
+  if (subindex < 9)
+    freq = 10 * (subindex + 1);
+  else if (subindex < 18)
+    freq = 100 * (subindex - 9 + 1);
+  else if (subindex < 27)
+    freq = 1000 * (subindex - 18 + 1);
+  else
+    freq = 10000 * (subindex - 27 + 1);
+
+  return freq;
+}
+
 };
 
 #endif
