@@ -28,33 +28,59 @@
 #include <gtk/gtk.h>
 #include "giface.h"
 #include "gui_config.h"
+#include "jackhost.h"
 
 namespace calf_plugins {
 
-class plugin_gui;
-
-struct control_base
+class window_update_controller
 {
-    std::string control_name;
+    int refresh_counter;
+public:
+    window_update_controller() : refresh_counter() {}
+    bool check_redraw(GtkWidget *toplevel);
+};
+
+class plugin_gui;
+class jack_host;
+
+class control_base
+{
+public:
     typedef std::map<std::string, std::string> xml_attribute_map;
+
+    GtkWidget *widget;
+    std::string control_name;
     xml_attribute_map attribs;
     plugin_gui *gui;
+public:
     void require_attribute(const char *name);
     void require_int_attribute(const char *name);
     int get_int(const char *name, int def_value = 0);
     float get_float(const char *name, float def_value = 0.f);
-    /// called after creation, so that all standard properties can be set
-    virtual void set_std_properties() = 0;
+
+public:
+    virtual GtkWidget *create(plugin_gui *_gui) { return NULL; }
+    virtual bool is_container() { return GTK_IS_CONTAINER(widget); };
+    virtual void set_visibilty(bool state);
+    virtual void add(control_base *ctl) { gtk_container_add(GTK_CONTAINER(widget), ctl->widget); }
+    /// called from created() to set all the properties
+    virtual void set_std_properties();
+    /// called after the control is created
+    virtual void created();
+    virtual ~control_base() {}
 };
 
 #define _GUARD_CHANGE_ if (in_change) return; guard_change __gc__(this);
 
-struct param_control: public control_base
+class param_control: public control_base
 {    
+protected:
+    GtkWidget *entrywin;
+public:
     int param_no;
     std::string param_variable;
-    GtkWidget *label, *widget;
     int in_change;
+    bool has_entry;
     float old_displayed_value;
     
     struct guard_change {
@@ -66,32 +92,33 @@ struct param_control: public control_base
     param_control();
     inline const parameter_properties &get_props();
     
-    virtual void init_xml(const char *element) {}
-    virtual GtkWidget *create_label();
-    virtual void update_label();
+    virtual GtkWidget *create(plugin_gui *_gui);
     /// called to create a widget for a control
     virtual GtkWidget *create(plugin_gui *_gui, int _param_no)=0;
     /// called to transfer the value from control to parameter(s)
     virtual void get()=0;
     /// called to transfer the value from parameter(s) to control
     virtual void set()=0;
+    /// called after the control is created
+    virtual void created();
     /// called on DSSI configure()
     virtual void configure(const char *key, const char *value) {}
+    /// called from created() to add hooks for parameters
     virtual void hook_params();
+    /// called from created() to add context menu handlers
+    virtual void add_context_menu_handler();
     virtual void on_idle() {}
-    virtual void set_std_properties();
     virtual ~param_control();
+    virtual void do_popup_menu();
+    static gboolean on_button_press_event(GtkWidget *widget, GdkEventButton *event, void *user_data);
+    static gboolean on_popup_menu(GtkWidget *widget, void *user_data);
+    virtual void create_value_entry(GtkWidget *widget, int x, int y);
+    virtual void destroy_value_entry();
+    static gboolean value_entry_action(GtkEntry *widget, GdkEvent *event, void *user_data);
+    static gboolean value_entry_unfocus(GtkWidget *widget, GdkEventFocus *event, void *user_data);
+    static gboolean value_entry_click(GtkWidget *widget, GdkEventButton *event, void *user_data);
 };
 
-struct control_container: public control_base
-{
-    GtkContainer *container;
-    
-    virtual GtkWidget *create(plugin_gui *_gui, const char *element, xml_attribute_map &attributes)=0;
-    virtual void add(GtkWidget *w, control_base *base) { gtk_container_add(container, w); }
-    virtual void set_std_properties();
-    virtual ~control_container() {}
-};
 
 class plugin_gui_window;
 
@@ -101,14 +128,29 @@ protected:
     int param_count;
     std::multimap<int, param_control *> par2ctl;
     XML_Parser parser;
-    param_control *current_control;
-    std::vector<control_container *> container_stack;
-    control_container *top_container;
+    control_base *top_container;
     std::map<std::string, int> param_name_map;
     int ignore_stack;
     int last_status_serial_no;
     std::map<int, GSList *> param_radio_groups;
     GtkWidget *leftBox, *rightBox;
+    int context_menu_param_no;
+    uint32_t context_menu_last_designator;
+    std::vector<control_base *> stack;
+
+    struct automation_menu_entry {
+        plugin_gui *gui;
+        int source;
+        automation_menu_entry(plugin_gui *_gui, int _source)
+        : gui(_gui), source(_source) {}
+    };
+    std::vector<automation_menu_entry *> automation_menu_callback_data;
+
+    static void on_automation_add(GtkWidget *widget, void *user_data);
+    static void on_automation_delete(GtkWidget *widget, void *user_data);
+    static void on_automation_set_lower(GtkWidget *widget, void *user_data);
+    static void on_automation_set_upper(GtkWidget *widget, void *user_data);
+    void on_automation_set_lower_or_upper(automation_menu_entry *ame, bool is_upper);
 public:
     plugin_gui_window *window;
     GtkWidget *container;
@@ -116,15 +158,23 @@ public:
     plugin_ctl_iface *plugin;
     preset_access_iface *preset_access;
     std::vector<param_control *> params;
+    std::vector<int> read_serials;
+
+    /* For optional lv2ui:show interface. */
+    bool optclosed;
+    GtkWidget* optwidget;
+    GtkWidget* optwindow;
+    const char* opttitle;
 
     plugin_gui(plugin_gui_window *_window);
     GtkWidget *create_from_xml(plugin_ctl_iface *_plugin, const char *xml);
-    param_control *create_control_from_xml(const char *element, const char *attributes[]);
-    control_container *create_container_from_xml(const char *element, const char *attributes[]);
+    control_base *create_widget_from_xml(const char *element, const char *attributes[]);
 
     void add_param_ctl(int param, param_control *ctl) { par2ctl.insert(std::pair<int, param_control *>(param, ctl)); }
+    void remove_param_ctl(int param, param_control *ctl);
     void refresh();
     void refresh(int param_no, param_control *originator = NULL);
+    int get_param_no_by_name(std::string param_name);
     void xml_element_start(const char *element, const char *attributes[]);
     void set_param_value(int param_no, float value, param_control *originator = NULL);
     /// Called on change of configure variable
@@ -138,6 +188,12 @@ public:
     void set_radio_group(int param, GSList *group);
     /// Show rack ear widgets
     void show_rack_ears(bool show);
+    /// Pop-up a context menu for a control
+    void on_control_popup(param_control *ctl, int param_no);
+    /// Clean up callback data allocated for the automation pop-up menu
+    void cleanup_automation_entries();
+    /// Destroy all the widgets in the container
+    void destroy_child_widgets(GtkWidget *parent);
     ~plugin_gui();
     static void xml_element_start(void *data, const char *element, const char *attributes[]);
     static void xml_element_end(void *data, const char *element);
@@ -199,7 +255,7 @@ struct main_window_iface: public progress_report_iface
     /// Create the actual window associated with this interface
     virtual void create() = 0;
     /// Add the plugin to the window
-    virtual void add_plugin(plugin_ctl_iface *plugin) = 0;
+    virtual void add_plugin(jack_host *plugin) = 0;
     /// Remove the plugin from the window
     virtual void del_plugin(plugin_ctl_iface *plugin) = 0;
     /// Refresh the plugin UI
@@ -245,6 +301,7 @@ class plugin_gui_window: public calf_utils::config_listener_iface
 {
 private:
     void cleanup();
+    window_update_controller refresh_controller;
 public:
     plugin_gui *gui;
     GtkWindow *toplevel;
@@ -289,6 +346,39 @@ struct activate_command_params
 
 void activate_command(GtkAction *action, activate_command_params *params);
 
+class cairo_impl: public calf_plugins::cairo_iface
+{
+public:
+    cairo_t *context;
+    virtual void set_source_rgba(float r, float g, float b, float a = 1.f) { cairo_set_source_rgba(context, r, g, b, a); }
+    virtual void set_line_width(float width) { cairo_set_line_width(context, width); }
+    virtual void set_dash(const double *dash, int length) { cairo_set_dash(context, dash, length, 0); }
+    virtual void draw_label(const char *label, float x, float y, int pos, float margin, float align) {
+        cairo_text_extents_t extents;
+        cairo_text_extents(context, label, &extents);
+        switch(pos) {
+            case 0:
+            default:
+                // top
+                cairo_move_to(context, x - extents.width / 2, y - margin);
+                break;
+            case 1:
+                // right
+                cairo_move_to(context, x + margin, y + 2);
+                break;
+            case 2:
+                // bottom
+                cairo_move_to(context, x - extents.width / 2, y + margin + extents.height);
+                break;
+            case 3:
+                // left
+                cairo_move_to(context, x - margin - extents.width, y + 2);
+                break;
+        }
+        cairo_show_text(context, label);
+    }
+};
+
 };
 
 #endif
