@@ -30,20 +30,21 @@
 #include "loudness.h"
 #include "metadata.h"
 #include "plugin_tools.h"
+#include "bypass.h"
 
 namespace calf_plugins {
 
-/// Not a true _audio_module style class, just pretends to be one!
-/// Main gain reduction routine by Thor called by various audio modules
-
+/**********************************************************************
+ * GAIN REDUCTION by Thor Harald Johanssen
+**********************************************************************/
 class gain_reduction_audio_module
 {
 private:
     float linSlope, detected, kneeSqrt, kneeStart, linKneeStart, kneeStop;
     float compressedKneeStop, adjKneeStart, thres;
     float attack, release, threshold, ratio, knee, makeup, detection, stereo_link, bypass, mute, meter_out, meter_comp;
-    mutable float old_threshold, old_ratio, old_knee, old_makeup, old_bypass, old_mute, old_detection, old_stereo_link;
-    mutable volatile int last_generation;
+    float old_threshold, old_ratio, old_knee, old_makeup, old_bypass, old_mute, old_detection, old_stereo_link;
+    mutable bool redraw_graph;
     uint32_t srate;
     bool is_active;
     inline float output_level(float slope) const;
@@ -62,18 +63,54 @@ public:
     bool get_graph(int subindex, float *data, int points, cairo_iface *context, int *mode) const;
     bool get_dot(int subindex, float &x, float &y, int &size, cairo_iface *context) const;
     bool get_gridline(int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
-    int  get_changed_offsets(int generation, int &subindex_graph, int &subindex_dot, int &subindex_gridline) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
 };
 
-/// Not a true _audio_module style class, just pretends to be one!
-/// Main gate routine by Damien called by various audio modules
+/**********************************************************************
+ * GAIN REDUCTION 2 by Damien Zammit
+**********************************************************************/
+
+class gain_reduction2_audio_module
+{
+private:
+    float linSlope, detected, kneeSqrt, kneeStart, linKneeStart, kneeStop;
+    float compressedKneeStop, adjKneeStart, thres;
+    float attack, release, threshold, ratio, knee, makeup, detection, stereo_link, bypass, mute, meter_out, meter_comp;
+    float old_threshold, old_ratio, old_knee, old_makeup, old_bypass, old_mute, old_detection, old_stereo_link;
+    mutable bool redraw_graph;
+    float old_y1,old_yl,old_detected;
+    uint32_t srate;
+    bool is_active;
+    inline float output_level(float inputt) const;
+    inline float output_gain(float inputt) const;
+public:
+    gain_reduction2_audio_module();
+    void set_params(float att, float rel, float thr, float rat, float kn, float mak, float byp, float mu);
+    void update_curve();
+    void process(float &left);
+    void activate();
+    void deactivate();
+    int id;
+    void set_sample_rate(uint32_t sr);
+    float get_output_level();
+    float get_comp_level();
+    bool get_graph(int subindex, float *data, int points, cairo_iface *context, int *mode) const;
+    bool get_dot(int subindex, float &x, float &y, int &size, cairo_iface *context) const;
+    bool get_gridline(int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
+};
+
+/**********************************************************************
+ * EXPANDER by Damien Zammit
+**********************************************************************/
+
 class expander_audio_module {
 private:
     float linSlope, peak, detected, kneeSqrt, kneeStart, linKneeStart, kneeStop, linKneeStop;
     float compressedKneeStop, adjKneeStart, range, thres, attack_coeff, release_coeff;
     float attack, release, threshold, ratio, knee, makeup, detection, stereo_link, bypass, mute, meter_out, meter_gate;
-    mutable float old_threshold, old_ratio, old_knee, old_makeup, old_bypass, old_range, old_trigger, old_mute, old_detection, old_stereo_link;
-    mutable volatile int last_generation;
+    float old_threshold, old_ratio, old_knee, old_makeup, old_bypass, old_range, old_trigger, old_mute, old_detection, old_stereo_link;
+    mutable bool redraw_graph;
     inline float output_level(float slope) const;
     inline float output_gain(float linSlope, bool rms) const;
 public:
@@ -92,33 +129,39 @@ public:
     bool get_graph(int subindex, float *data, int points, cairo_iface *context, int *mode) const;
     bool get_dot(int subindex, float &x, float &y, int &size, cairo_iface *context) const;
     bool get_gridline(int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
-    int  get_changed_offsets(int generation, int &subindex_graph, int &subindex_dot, int &subindex_gridline) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
 };
 
-/// Compressor by Thor
+/**********************************************************************
+ * COMPRESSOR by Thor Harald Johanssen
+**********************************************************************/
+
 class compressor_audio_module: public audio_module<compressor_metadata>, public line_graph_iface  {
 private:
     typedef compressor_audio_module AM;
-    stereo_in_out_metering<compressor_metadata> meters;
     gain_reduction_audio_module compressor;
+    dsp::bypass bypass;
+    vumeters meters;
 public:
     typedef std::complex<double> cfloat;
     uint32_t srate;
     bool is_active;
-    mutable volatile int last_generation, last_calculated_generation;
     compressor_audio_module();
     void activate();
     void deactivate();
     void params_changed();
     void set_sample_rate(uint32_t sr);
     uint32_t process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask);
-    bool get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const;
-    bool get_dot(int index, int subindex, float &x, float &y, int &size, cairo_iface *context) const;
-    bool get_gridline(int index, int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
-    int  get_changed_offsets(int index, int generation, int &subindex_graph, int &subindex_dot, int &subindex_gridline) const;
+    bool get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const;
+    bool get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const;
+    bool get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
 };
 
-/// Sidecain Compressor by Markus Schmidt (based on Thor's compressor and Krzysztof's filters)
+/**********************************************************************
+ * SIDECHAIN COMPRESSOR by Markus Schmidt
+**********************************************************************/
+
 class sidechaincompressor_audio_module: public audio_module<sidechaincompressor_metadata>, public frequency_response_line_graph  {
 private:
     typedef sidechaincompressor_audio_module AM;
@@ -134,51 +177,52 @@ private:
         BANDPASS_1,
         BANDPASS_2
     };
-    enum CalfScRoute {
-        STEREO,
-        RIGHT_LEFT,
-        LEFT_RIGHT
-    };
-    mutable float f1_freq_old, f2_freq_old, f1_level_old, f2_level_old;
-    mutable float f1_freq_old1, f2_freq_old1, f1_level_old1, f2_level_old1;
+    float f1_freq_old, f2_freq_old, f1_level_old, f2_level_old;
+    float f1_freq_old1, f2_freq_old1, f1_level_old1, f2_level_old1;
     CalfScModes sc_mode;
-    mutable CalfScModes sc_mode_old, sc_mode_old1;
+    CalfScModes sc_mode_old, sc_mode_old1;
+    mutable bool redraw_graph;
     float f1_active, f2_active;
-    stereo_in_out_metering<sidechaincompressor_metadata> meters;
     gain_reduction_audio_module compressor;
-    dsp::biquad_d2<float> f1L, f1R, f2L, f2R;
+    dsp::biquad_d2 f1L, f1R, f2L, f2R;
+    dsp::bypass bypass;
+    vumeters meters;
 public:
     typedef std::complex<double> cfloat;
     uint32_t srate;
     bool is_active;
-    mutable volatile int last_generation, last_calculated_generation;
     sidechaincompressor_audio_module();
     void activate();
     void deactivate();
     void params_changed();
     cfloat h_z(const cfloat &z) const;
-    float freq_gain(int index, double freq, uint32_t sr) const;
+    float freq_gain(int index, double freq) const;
     void set_sample_rate(uint32_t sr);
     uint32_t process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask);
-    bool get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const;
-    bool get_dot(int index, int subindex, float &x, float &y, int &size, cairo_iface *context) const;
-    bool get_gridline(int index, int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
-    int  get_changed_offsets(int index, int generation, int &subindex_graph, int &subindex_dot, int &subindex_gridline) const;
+    bool get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const;
+    bool get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const;
+    bool get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
 };
 
-/// Multibandcompressor by Markus Schmidt (based on Thor's compressor and Krzysztof's filters)
-class multibandcompressor_audio_module: public audio_module<multibandcompressor_metadata>, public line_graph_iface {
+/**********************************************************************
+ * MULTIBAND COMPRESSOR by Markus Schmidt
+**********************************************************************/
+
+class multibandcompressor_audio_module: public audio_module<multibandcompressor_metadata>, public frequency_response_line_graph {
 private:
     typedef multibandcompressor_audio_module AM;
     static const int strips = 4;
     bool solo[strips];
+    float xout[strips], xin[2];
     bool no_solo;
-    uint32_t clip_inL, clip_inR, clip_outL, clip_outR;
     float meter_inL, meter_inR, meter_outL, meter_outR;
     gain_reduction_audio_module strip[strips];
-    dsp::biquad_d2<float> lpL[strips - 1][3], lpR[strips - 1][3], hpL[strips - 1][3], hpR[strips - 1][3];
-    float freq_old[strips - 1], sep_old[strips - 1], q_old[strips - 1];
-    int mode, mode_old;
+    dsp::crossover crossover;
+    dsp::bypass bypass;
+    int mode, page, bypass_;
+    mutable int redraw;
+    vumeters meters;
 public:
     uint32_t srate;
     bool is_active;
@@ -189,13 +233,42 @@ public:
     uint32_t process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask);
     void set_sample_rate(uint32_t sr);
     const gain_reduction_audio_module *get_strip_by_param_index(int index) const;
-    virtual bool get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const;
-    virtual bool get_dot(int index, int subindex, float &x, float &y, int &size, cairo_iface *context) const;
-    virtual bool get_gridline(int index, int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
-    virtual int  get_changed_offsets(int index, int generation, int &subindex_graph, int &subindex_dot, int &subindex_gridline) const;
+    virtual bool get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const;
+    virtual bool get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const;
+    virtual bool get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
+    virtual bool get_layers(int index, int generation, unsigned int &layers) const;
 };
 
-/// Deesser by Markus Schmidt (based on Thor's compressor and Krzyexpander_audio_modulesztof's filters)
+/**********************************************************************
+ * MONO COMPRESSOR by Damien Zammit
+**********************************************************************/
+
+class monocompressor_audio_module: public audio_module<monocompressor_metadata>, public line_graph_iface  {
+private:
+    typedef monocompressor_audio_module AM;
+    gain_reduction2_audio_module monocompressor;
+    vumeters meters;
+    dsp::bypass bypass;
+public:
+    typedef std::complex<double> cfloat;
+    uint32_t srate;
+    bool is_active;
+    monocompressor_audio_module();
+    void activate();
+    void deactivate();
+    void params_changed();
+    void set_sample_rate(uint32_t sr);
+    uint32_t process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask);
+    bool get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const;
+    bool get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const;
+    bool get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
+};
+
+/**********************************************************************
+ * DEESSER by Markus Schmidt
+**********************************************************************/
+
 class deesser_audio_module: public audio_module<deesser_metadata>, public frequency_response_line_graph  {
 private:
     enum CalfDeessModes {
@@ -205,53 +278,56 @@ private:
     mutable float f1_freq_old, f2_freq_old, f1_level_old, f2_level_old, f2_q_old;
     mutable float f1_freq_old1, f2_freq_old1, f1_level_old1, f2_level_old1, f2_q_old1;
     uint32_t detected_led;
-    float detected, clip_out;
-    uint32_t clip_led;
+    float detected;
     gain_reduction_audio_module compressor;
-    dsp::biquad_d2<float> hpL, hpR, lpL, lpR, pL, pR;
+    dsp::biquad_d2 hpL, hpR, lpL, lpR, pL, pR;
+    dsp::bypass bypass;
+    vumeters meters;
 public:
     uint32_t srate;
     bool is_active;
-    mutable volatile int last_generation, last_calculated_generation;
     deesser_audio_module();
     void activate();
     void deactivate();
     void params_changed();
-    float freq_gain(int index, double freq, uint32_t sr) const
+    float freq_gain(int index, double freq) const
     {
-        return hpL.freq_gain(freq, sr) * pL.freq_gain(freq, sr);
+        return hpL.freq_gain(freq, srate) * pL.freq_gain(freq, srate);
     }
     void set_sample_rate(uint32_t sr);
     uint32_t process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask);
-    bool get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const;
-    bool get_gridline(int index, int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
-    int  get_changed_offsets(int index, int generation, int &subindex_graph, int &subindex_dot, int &subindex_gridline) const;
 };
 
-/// Gate by Damien
+/**********************************************************************
+ * GATE AUDIO MODULE Damien Zammit
+**********************************************************************/
+
 class gate_audio_module: public audio_module<gate_metadata>, public line_graph_iface  {
 private:
     typedef gate_audio_module AM;
-    stereo_in_out_metering<gate_metadata> meters;
     expander_audio_module gate;
+    dsp::bypass bypass;
+    vumeters meters;
 public:
     typedef std::complex<double> cfloat;
     uint32_t srate;
     bool is_active;
-    mutable volatile int last_generation, last_calculated_generation;
     gate_audio_module();
     void activate();
     void deactivate();
     void params_changed();
     void set_sample_rate(uint32_t sr);
     uint32_t process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask);
-    bool get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const;
-    bool get_dot(int index, int subindex, float &x, float &y, int &size, cairo_iface *context) const;
-    bool get_gridline(int index, int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
-    int  get_changed_offsets(int index, int generation, int &subindex_graph, int &subindex_dot, int &subindex_gridline) const;
+    bool get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const;
+    bool get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const;
+    bool get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
 };
 
-/// Sidecain Gate by Markus Schmidt (based on Damiens's gate and Krzysztof's filters)
+/**********************************************************************
+ * SIDECHAIN GATE by Markus Schmidt
+**********************************************************************/
+
 class sidechaingate_audio_module: public audio_module<sidechaingate_metadata>, public frequency_response_line_graph  {
 private:
     typedef sidechaingate_audio_module AM;
@@ -267,52 +343,53 @@ private:
         BANDPASS_1,
         BANDPASS_2
     };
-    enum CalfScRoute {
-        STEREO,
-        RIGHT_LEFT,
-        LEFT_RIGHT
-    };
-    mutable float f1_freq_old, f2_freq_old, f1_level_old, f2_level_old;
-    mutable float f1_freq_old1, f2_freq_old1, f1_level_old1, f2_level_old1;
+    float f1_freq_old, f2_freq_old, f1_level_old, f2_level_old;
+    float f1_freq_old1, f2_freq_old1, f1_level_old1, f2_level_old1;
     CalfScModes sc_mode;
-    mutable CalfScModes sc_mode_old, sc_mode_old1;
+    CalfScModes sc_mode_old, sc_mode_old1;
+    mutable bool redraw_graph;
     float f1_active, f2_active;
-    stereo_in_out_metering<sidechaingate_metadata> meters;
     expander_audio_module gate;
-    dsp::biquad_d2<float> f1L, f1R, f2L, f2R;
+    dsp::biquad_d2 f1L, f1R, f2L, f2R;
+    dsp::bypass bypass;
+    vumeters meters;
 public:
     typedef std::complex<double> cfloat;
     uint32_t srate;
     bool is_active;
-    mutable volatile int last_generation, last_calculated_generation;
     sidechaingate_audio_module();
     void activate();
     void deactivate();
     void params_changed();
     cfloat h_z(const cfloat &z) const;
-    float freq_gain(int index, double freq, uint32_t sr) const;
+    float freq_gain(int index, double freq) const;
     void set_sample_rate(uint32_t sr);
     uint32_t process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask);
-    bool get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const;
-    bool get_dot(int index, int subindex, float &x, float &y, int &size, cairo_iface *context) const;
-    bool get_gridline(int index, int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
-    int  get_changed_offsets(int index, int generation, int &subindex_graph, int &subindex_dot, int &subindex_gridline) const;
+    bool get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const;
+    bool get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const;
+    bool get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
 };
 
 
-/// Multibandgate by Markus Schmidt (based on Damiens's gate and Krzysztof's filters)
-class multibandgate_audio_module: public audio_module<multibandgate_metadata>, public line_graph_iface {
+/**********************************************************************
+ * MULTIBAND GATE by Markus Schmidt
+**********************************************************************/
+
+class multibandgate_audio_module: public audio_module<multibandgate_metadata>, public frequency_response_line_graph {
 private:
     typedef multibandgate_audio_module AM;
     static const int strips = 4;
     bool solo[strips];
+    float xout[strips], xin[2];
     bool no_solo;
-    uint32_t clip_inL, clip_inR, clip_outL, clip_outR;
     float meter_inL, meter_inR, meter_outL, meter_outR;
     expander_audio_module gate[strips];
-    dsp::biquad_d2<float> lpL[strips - 1][3], lpR[strips - 1][3], hpL[strips - 1][3], hpR[strips - 1][3];
-    float freq_old[strips - 1], sep_old[strips - 1], q_old[strips - 1];
-    int mode, mode_old;
+    dsp::crossover crossover;
+    dsp::bypass bypass;
+    int mode, page, bypass_;
+    mutable int redraw;
+    vumeters meters;
 public:
     uint32_t srate;
     bool is_active;
@@ -323,10 +400,53 @@ public:
     uint32_t process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask);
     void set_sample_rate(uint32_t sr);
     const expander_audio_module *get_strip_by_param_index(int index) const;
-    virtual bool get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const;
-    virtual bool get_dot(int index, int subindex, float &x, float &y, int &size, cairo_iface *context) const;
-    virtual bool get_gridline(int index, int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
-    virtual int  get_changed_offsets(int index, int generation, int &subindex_graph, int &subindex_dot, int &subindex_gridline) const;
+    virtual bool get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const;
+    virtual bool get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const;
+    virtual bool get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
+};
+
+/**********************************************************************
+ * TRANSIENT DESIGNER by Christian Holschuh and Markus Schmidt
+**********************************************************************/
+
+class transientdesigner_audio_module:
+    public audio_module<transientdesigner_metadata>, public frequency_response_line_graph
+{
+    typedef transientdesigner_audio_module AM;
+    static const int channels = 2;
+    uint32_t srate;
+    bool active;
+    mutable bool redraw;
+    float meter_inL, meter_inR, meter_outL, meter_outR;
+    dsp::transients transients;
+    dsp::bypass bypass;
+    dsp::biquad_d2 hp[3], lp[3];
+    float hp_f_old, hp_m_old, lp_f_old, lp_m_old;
+    int display_old;
+    mutable int pixels;
+    mutable float *pbuffer;
+    mutable int pbuffer_pos;
+    mutable int pbuffer_size;
+    mutable int pbuffer_sample;
+    mutable int pbuffer_draw;
+    mutable bool pbuffer_available;
+    bool attacked;
+    uint32_t attcount;
+    int attack_pos;
+    float display_max;
+    vumeters meters;
+public:
+    transientdesigner_audio_module();
+    ~transientdesigner_audio_module();
+    void params_changed();
+    void activate();
+    void set_sample_rate(uint32_t sr);
+    void deactivate();
+    uint32_t process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask);
+    bool get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const;
+    bool get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
 };
 
 };
