@@ -206,7 +206,7 @@ class scanner_vibrato
 protected:
     enum { ScannerSize = 18 };
     float lfo_phase;
-    dsp::biquad_d2<float> scanner[ScannerSize];
+    dsp::biquad_d2 scanner[ScannerSize];
     organ_vibrato legacy;
 public:
     void reset();
@@ -215,13 +215,13 @@ public:
 
 class organ_voice: public dsp::voice, public organ_voice_base {
 protected:    
-    enum { Channels = 2, BlockSize = 64, EnvCount = organ_parameters::EnvCount, FilterCount = organ_parameters::FilterCount };
+    enum { Channels = 2, BlockSize = 64, EnvCount = organ_parameters::EnvCount, FilterCount = organ_parameters::FilterCount, MaxSampleRun = calf_plugins::MAX_SAMPLE_RUN };
     union {
         float output_buffer[BlockSize][Channels];
         float aux_buffers[3][BlockSize][Channels];
     };
     dsp::fixed_point<int64_t, 52> phase, dphase;
-    dsp::biquad_d1<float> filterL[2], filterR[2];
+    dsp::biquad_d1 filterL[2], filterR[2];
     adsr envs[EnvCount];
     dsp::inertia<dsp::linear_ramp> expression;
     scanner_vibrato vibrato;
@@ -245,7 +245,7 @@ public:
     void note_off(int /* vel */);
     virtual float get_priority() { return stolen ? 20000 : (perc_released ? 1 : (sostenuto ? 200 : 100)); }
     virtual void steal();
-    void render_block();
+    void render_block(int current_snapshot);
     
     virtual int get_current_note() {
         return note;
@@ -293,6 +293,7 @@ struct drawbar_organ: public dsp::basic_synth, public calf_plugins::organ_enums
      drawbar_organ(organ_parameters *_parameters)
     : parameters(_parameters)
     , percussion(_parameters) {
+        init_voices(36);
     }
     void render_separate(float *output[], int nsamples);
     dsp::voice *alloc_voice();
@@ -322,18 +323,20 @@ public:
     dsp::organ_parameters par_values;
     uint32_t srate;
     bool panic_flag;
+    mutable bool redraw;
+    
     /// Value for configure variable map_curve
     std::string var_map_curve;
 
     organ_audio_module();
     
-    void post_instantiate();
+    void post_instantiate(uint32_t sample_rate);
 
     void set_sample_rate(uint32_t sr) {
         srate = sr;
     }
     void params_changed();
-
+    
     void activate();
     void deactivate();
     uint32_t process(uint32_t offset, uint32_t nsamples, uint32_t inputs_mask, uint32_t outputs_mask);
@@ -342,7 +345,8 @@ public:
     /// Practically all the stuff here is noisy
     bool is_noisy(int param_no) { return true; }
     void execute(int cmd_no);
-    bool get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const;
+    bool get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const;
     char *configure(const char *key, const char *value);
     void send_configures(send_configure_iface *);
     uint32_t message_run(const void *valid_inputs, void *output_ports);
