@@ -15,12 +15,12 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef __CALF_OSC_H
-#define __CALF_OSC_H
+#ifndef CALF_OSC_H
+#define CALF_OSC_H
 
 #include "fft.h"
 #include <map>
@@ -53,10 +53,16 @@ struct simple_oscillator
     {
         phasedelta = (int)(freq * 65536.0 * 256.0 * 16.0 * odsr) << 4;
     }
+    /// Make one phase increment
+    inline void step()
+    {
+        phase += phasedelta;
+    }
+    /// Make one phase increment and return a value from -0.5 to 0.5
     inline float get()
     {
         float value = (phase >> 16 ) / 65535.0 - 0.5;
-        phase += phasedelta;
+        step();
         return value;
     }
 };
@@ -146,7 +152,7 @@ struct bandlimiter
                 new_spec[SIZE - i] = 0.f;
         }
         // convert back to time domain (IFFT) and extract only real part
-        fft.calculate(new_spec.data(), iffted.data(), true);
+        fft.calculate(&new_spec.front(), &iffted.front(), true);
         for (int i = 0; i < SIZE; i++)
             output[i] = iffted[i].real();
     }
@@ -183,11 +189,19 @@ struct waveform_family: public std::map<uint32_t, float *>
         for (unsigned int i = 0; i < cutoff; i++)
             vmax = std::max(vmax, abs(bl.spectrum[i]));
         float vthres = vmax / 1024.0;  // -60dB
+        float cumul = 0.f;
         while(cutoff > (SIZE / limit)) {
             if (!foldover)
             {
-                while(cutoff > 1 && abs(bl.spectrum[cutoff - 1]) < vthres)
+                // skip harmonics too quiet to be heard, but measure their loudness cumulatively,
+                // because even if a single harmonic is too quiet, a whole bunch of them may add up 
+                // to considerable amount of space
+                cumul = 0.f;
+                while(cutoff > 1 && cumul + abs(bl.spectrum[cutoff - 1]) < vthres)
+                {
+                    cumul += abs(bl.spectrum[cutoff - 1]);
                     cutoff--;
+                }
             }
             float *wf = new float[SIZE+1];
             bl.make_waveform(wf, cutoff, foldover);
@@ -215,6 +229,13 @@ struct waveform_family: public std::map<uint32_t, float *>
     }
 };
 
+#if 0
+// cubic interpolation
+static inline float cerp(float pm1, float p0, float p1, float p2, float t)
+{
+    return (-t*(t-1)*(t-2) * pm1 + 3*(t+1)*(t-1)*(t-2) * p0 - 3*(t+1)*t*(t-2) * p1 + (t+1)*t*(t-1) * p2) * (1.0 / 6.0);
+}
+#endif
 /**
  * Simple table-based lerping oscillator. Uses waveform of size 2^SIZE_BITS.
  * Combine with waveform_family if bandlimited waveforms are needed. Because
@@ -226,10 +247,79 @@ struct waveform_oscillator: public simple_oscillator
 {
     enum { SIZE = 1 << SIZE_BITS, MASK = SIZE - 1, SCALE = 1 << (32 - SIZE_BITS) };
     float *waveform;
+    waveform_oscillator()
+    {
+        waveform = NULL;
+    }
+    
+    /// Get the value from single oscillator at current position
     inline float get()
     {
         uint32_t wpos = phase >> (32 - SIZE_BITS);
-        float value = dsp::lerp(waveform[wpos], waveform[(wpos + 1) & MASK], (phase & (SCALE - 1)) * (1.0f / SCALE));
+        return dsp::lerp(waveform[wpos], waveform[(wpos + 1) & MASK], (phase & (SCALE - 1)) * (1.0f / SCALE));
+    }
+    /// Add/substract two phase-shifted values
+    inline float get_phaseshifted(uint32_t shift, float mix)
+    {
+        uint32_t wpos = phase >> (32 - SIZE_BITS);
+        float value1 = dsp::lerp(waveform[wpos], waveform[(wpos + 1) & MASK], (phase & (SCALE - 1)) * (1.0f / SCALE));
+        wpos = (phase + shift) >> (32 - SIZE_BITS);
+        float value2 = dsp::lerp(waveform[wpos], waveform[(wpos + 1) & MASK], ((phase + shift) & (SCALE - 1)) * (1.0f / SCALE));
+        return value1 + mix * value2;
+    }
+    /// Add/substract two phase-shifted values
+    inline float get_phaseshifted2(uint32_t shift, int32_t gshift, float mix)
+    {
+        uint32_t wpos = (phase + gshift) >> (32 - SIZE_BITS);
+        float value1 = dsp::lerp(waveform[wpos], waveform[(wpos + 1) & MASK], (phase & (SCALE - 1)) * (1.0f / SCALE));
+        wpos = (phase + gshift + shift) >> (32 - SIZE_BITS);
+        float value2 = dsp::lerp(waveform[wpos], waveform[(wpos + 1) & MASK], ((phase + shift) & (SCALE - 1)) * (1.0f / SCALE));
+        return value1 + mix * value2;
+    }
+    /// Get the value of a hard synced osc (65536 = 1:1 ratio)
+    inline float get_phasedist(uint32_t sync, uint32_t shift, float mix)
+    {
+        uint32_t phase_mod = (uint64_t(phase) * sync >> 16);
+        
+        uint32_t wpos = phase_mod >> (32 - SIZE_BITS);
+        float value1 = dsp::lerp(waveform[wpos], waveform[(wpos + 1) & MASK], (phase & (SCALE - 1)) * (1.0f / SCALE));
+        wpos = (phase_mod + shift) >> (32 - SIZE_BITS);
+        float value2 = dsp::lerp(waveform[wpos], waveform[(wpos + 1) & MASK], ((phase + shift) & (SCALE - 1)) * (1.0f / SCALE));
+        return value1 + mix * value2;
+    }
+    /// One step
+    inline void advance()
+    {
+        phase += phasedelta;
+    }
+};
+
+/**
+ * Simple triangle LFO without any smoothing or anything of this sort. 
+ */
+struct triangle_lfo: public simple_oscillator
+{
+    /// Previous value (not stored here, but may be used by calling code)
+    float last;
+    
+    triangle_lfo()
+    {
+        reset();
+    }
+    void reset()
+    {
+        simple_oscillator::reset();
+        last = 0;
+    }
+    inline float get()
+    {
+        uint32_t phase2 = phase;
+        // start at 90 degrees point of the "/\" wave (-1 to +1)
+        phase2 += 1<<30; 
+        // if in second half, invert the wave (so it falls back into 0..0x7FFFFFFF)
+        phase2 ^= ((int32_t)phase2)>>31;
+        
+        float value = (phase2 >> 6) / 16777216.0 - 1.0;
         phase += phasedelta;
         return value;
     }
