@@ -38,7 +38,7 @@ calf_vumeter_expose (GtkWidget *widget, GdkEventExpose *event)
     g_assert(CALF_IS_VUMETER(widget));
 
     CalfVUMeter *vu = CALF_VUMETER(widget);
-    GtkStyle	*style;
+    GtkStyle    *style;
     style = gtk_widget_get_style(widget);
     cairo_t *c = gdk_cairo_create(GDK_DRAWABLE(widget->window));
     
@@ -207,18 +207,47 @@ calf_vumeter_expose (GtkWidget *widget, GdkEventExpose *event)
             cairo_stroke(cache_cr);
         }
         // create blinder pattern
-        vu->pat = cairo_pattern_create_linear (led_x, led_y, led_x, led_y + led_h);
-        cairo_pattern_add_color_stop_rgba (vu->pat, 0, 0.5, 0.5, 0.5, 0.6);
-        cairo_pattern_add_color_stop_rgba (vu->pat, 0.4, 0, 0, 0, 0.6);
-        cairo_pattern_add_color_stop_rgba (vu->pat, 0.401, 0, 0, 0, 0.8);
-        cairo_pattern_add_color_stop_rgba (vu->pat, 1, 0, 0, 0, 0.6);
-        cairo_destroy( cache_cr );
+        cairo_pattern_t *pat = cairo_pattern_create_linear (led_x, led_y, led_x, led_y + led_h);
+        cairo_pattern_add_color_stop_rgba (pat, 0, 1, 1, 1, 0.25);
+        cairo_pattern_add_color_stop_rgba (pat, 0.5, 0.5, 0.5, 0.5, 0.0);
+        cairo_pattern_add_color_stop_rgba (pat, 1, 0.0, 0.0, 0.0, 0.25);
+        cairo_rectangle(cache_cr, led_x, led_y, led_w, led_h);
+        cairo_set_source(cache_cr, pat);
+        cairo_fill(cache_cr);
+        
+        // create overlay
+        vu->cache_overlay = cairo_surface_create_similar(window_surface, 
+                                  CAIRO_CONTENT_COLOR,
+                                  widget->allocation.width,
+                                  widget->allocation.height);
+        cairo_t *over_cr = cairo_create(vu->cache_overlay);
+        
+        // copy surface to overlay
+        cairo_set_source_surface(over_cr, vu->cache_surface, 0, 0);
+        cairo_rectangle(over_cr, 0, 0, width, height);
+        cairo_fill(over_cr);
+        
+        // create blinder pattern
+        pat = cairo_pattern_create_linear (led_x, led_y, led_x, led_y + led_h);
+        cairo_pattern_add_color_stop_rgba (pat, 0, 0.2, 0.2, 0.2, 0.7);
+        cairo_pattern_add_color_stop_rgba (pat, 0.4, 0.05, 0.05, 0.05, 0.7);
+        cairo_pattern_add_color_stop_rgba (pat, 0.401, 0.05, 0.05, 0.05, 0.9);
+        cairo_pattern_add_color_stop_rgba (pat, 1, 0.05, 0.05, 0.05, 0.75);
+        
+        // draw on top of overlay
+        cairo_set_source(over_cr, pat);
+        cairo_rectangle(over_cr, 0, 0, width, height);
+        cairo_paint(over_cr);
+        
+        // clean up
+        cairo_destroy(cache_cr);
+        cairo_destroy(over_cr);
     }
     
     // draw LED blinder
     cairo_set_source_surface( c, vu->cache_surface, 0,0 );
     cairo_paint( c );
-    cairo_set_source( c, vu->pat );
+    cairo_set_source_surface( c, vu->cache_overlay, 0, 0 );
     
     // get microseconds
     timeval tv;
@@ -228,7 +257,7 @@ calf_vumeter_expose (GtkWidget *widget, GdkEventExpose *event)
     // limit to 1.f
     float value_orig = std::max(std::min(vu->value, 1.f), 0.f);
     float value = 0.f;
-    
+
     // falloff?
     if(vu->vumeter_falloff > 0.f and vu->mode != VU_MONOCHROME_REVERSE) {
         // fall off a bit
@@ -249,6 +278,10 @@ calf_vumeter_expose (GtkWidget *widget, GdkEventExpose *event)
         value = value_orig;
         vu->falling = false;
     }
+    
+    float draw = 0.f;
+    float draw_last = 0.f;
+    
     if(vu->vumeter_hold > 0.0) {
         // peak hold timer
         if(time - (long)(vu->vumeter_hold * 1000 * 1000) > vu->last_hold) {
@@ -265,15 +298,17 @@ calf_vumeter_expose (GtkWidget *widget, GdkEventExpose *event)
                 vu->last_hold = time;
                 vu->holding = true;
             }
+            draw = log10(1 + value * 9);
+            draw_last = log10(1 + vu->last_value * 9);
             
             // blinder left -> hold LED
-            int hold_x = round((vu->last_value) * (led_w + led_m)); // add last led_m removed earlier
+            int hold_x = round((draw_last) * (led_w + led_m)); // add last led_m removed earlier
             hold_x -= hold_x % led_s + led_m;
             hold_x = std::max(0, hold_x);
             cairo_rectangle( c, led_x, led_y, hold_x, led_h);
             
             // blinder hold LED -> value
-            int val_x = round((1 - value) * (led_w + led_m)); // add last led_m removed earlier
+            int val_x = round((1 - draw) * (led_w + led_m)); // add last led_m removed earlier
             val_x -= val_x % led_s;
             int blind_x = std::min(hold_x + led_s, led_w);
             int blind_w = std::min(std::max(led_w - val_x - hold_x - led_s, 0), led_w);
@@ -285,7 +320,8 @@ calf_vumeter_expose (GtkWidget *widget, GdkEventExpose *event)
                 vu->last_hold = time;
                 vu->holding = true;
             }
-            int val_x = round((1 - value) / 2.f * (led_w + led_m)); // add last led_m removed earlier
+            draw = log10(1 + value * 9);
+            int val_x = round((1 - draw) / 2.f * (led_w + led_m)); // add last led_m removed earlier
             cairo_rectangle(c, led_x, led_y, val_x, led_h);
             cairo_rectangle(c, led_x + led_w - val_x, led_y, val_x, led_h);
             
@@ -296,10 +332,12 @@ calf_vumeter_expose (GtkWidget *widget, GdkEventExpose *event)
                 vu->last_hold = time;
                 vu->holding = true;
             }
+            draw = log10(1 + value * 9);
+            draw_last = log10(1 + vu->last_value * 9);
             
-            int hold_x = round((1 - vu->last_value) * (led_w + led_m)); // add last led_m removed earlier
+            int hold_x = round((1 - draw_last) * (led_w + led_m)); // add last led_m removed earlier
             hold_x -= hold_x % led_s;
-            int val_x = round(value * (led_w + led_m)); // add last led_m removed earlier
+            int val_x = round(draw * (led_w + led_m)); // add last led_m removed earlier
             val_x -= val_x % led_s;
             int blind_w = led_w - hold_x - led_s - val_x;
             blind_w = std::min(std::max(blind_w, 0), led_w);
@@ -308,14 +346,15 @@ calf_vumeter_expose (GtkWidget *widget, GdkEventExpose *event)
         }
     } else {
         // darken normally
+        float draw = log10(1 + value * 9);
         if( vu->mode == VU_MONOCHROME_REVERSE )
-            cairo_rectangle( c, led_x, led_y, value * led_w, led_h);
+            cairo_rectangle( c, led_x, led_y, draw * led_w, led_h);
         else if( vu->mode == VU_STANDARD_CENTER ) {
-            int val_x = round((1 - value) / 2.f * (led_w + led_m)); // add last led_m removed earlier
+            int val_x = round((1 - draw) / 2.f * (led_w + led_m)); // add last led_m removed earlier
             cairo_rectangle(c, led_x, led_y, val_x, led_h);
             cairo_rectangle(c, led_x + led_w - val_x, led_y, val_x, led_h);
         } else
-            cairo_rectangle( c, led_x + value * led_w, led_y, led_w * (1 - value), led_h);
+            cairo_rectangle( c, led_x + draw * led_w, led_y, led_w * (1 - draw), led_h);
     }
     cairo_fill( c );
     
@@ -326,16 +365,16 @@ calf_vumeter_expose (GtkWidget *widget, GdkEventExpose *event)
         or (vu->value < vu->disp_value and vu->mode == VU_MONOCHROME_REVERSE))
             vu->disp_value = vu->value;
         if (vu->disp_value < 1.0 / 32768.0)
-            sprintf(str, "-inf");
+            snprintf(str, sizeof(str), "-inf");
         else
-            sprintf(str, "%0.2f", dsp::amp2dB(vu->disp_value));
+            snprintf(str, sizeof(str), "%0.2f", dsp::amp2dB(vu->disp_value));
         // draw value as number
         cairo_text_extents(c, str, &extents);
         cairo_move_to(c, text_x + (text_w - extents.width) / 2.0, text_y);
         if(vu->disp_value > 1.f and vu->mode != VU_MONOCHROME_REVERSE)
             cairo_set_source_rgba (c, 1, 0, 0, 0.8);
         else
-            cairo_set_source_rgba (c, 0, 0.7, 1, 0.8);
+            cairo_set_source_rgba (c, 0, 0.9, 1, 0.8);
         cairo_show_text(c, str);
         cairo_fill(c);
     }
@@ -356,6 +395,17 @@ calf_vumeter_size_request (GtkWidget *widget,
     requisition->height = self->vumeter_height;
 }
 
+static void
+calf_vumeter_unrealize (GtkWidget *widget, CalfVUMeter *vu)
+{
+    if( vu->cache_surface )
+        cairo_surface_destroy( vu->cache_surface );
+    vu->cache_surface = NULL;
+    if( vu->cache_overlay )
+        cairo_surface_destroy( vu->cache_overlay );
+    vu->cache_overlay = NULL;
+}
+
 static void
 calf_vumeter_size_allocate (GtkWidget *widget,
                            GtkAllocation *allocation)
@@ -367,9 +417,7 @@ calf_vumeter_size_allocate (GtkWidget *widget,
 
     parent_class->size_allocate( widget, allocation );
 
-    if( vu->cache_surface )
-        cairo_surface_destroy( vu->cache_surface );
-    vu->cache_surface = NULL;
+    calf_vumeter_unrealize(widget, vu);
 }
 
 static void
@@ -394,6 +442,7 @@ calf_vumeter_init (CalfVUMeter *self)
     self->meter_width = 0;
     self->disp_value = 0.f;
     self->value = 0.f;
+    g_signal_connect(GTK_OBJECT(widget), "unrealize", G_CALLBACK(calf_vumeter_unrealize), (gpointer)self);
 }
 
 GtkWidget *
