@@ -22,8 +22,10 @@
 #include <calf/ctl_vumeter.h>
 #include <calf/giface.h>
 #include <calf/gui.h>
+#include <calf/custom_ctl.h>
 #include <calf/preset.h>
 #include <calf/gtk_main_win.h>
+#include <calf/jackhost.h>
 
 using namespace calf_plugins;
 using namespace std;
@@ -108,12 +110,14 @@ void gtk_main_window::on_preferences_action(GtkWidget *widget, gtk_main_window *
     gtk_spin_button_set_increments(GTK_SPIN_BUTTON(gtk_builder_get_object(prefs_builder, "float-size")), 1, 1);
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_builder_get_object(prefs_builder, "rack-float")), main->get_config()->rack_float);
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_builder_get_object(prefs_builder, "float-size")), main->get_config()->float_size);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(prefs_builder, "show-vu-meters")), main->get_config()->vu_meters);
     int response = gtk_dialog_run(GTK_DIALOG(preferences_dlg));
     if (response == GTK_RESPONSE_OK)
     {
         main->get_config()->rack_ears = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(prefs_builder, "show-rack-ears")));
         main->get_config()->rack_float = gtk_spin_button_get_value(GTK_SPIN_BUTTON(gtk_builder_get_object(prefs_builder, "rack-float")));
         main->get_config()->float_size = gtk_spin_button_get_value(GTK_SPIN_BUTTON(gtk_builder_get_object(prefs_builder, "float-size")));
+        main->get_config()->vu_meters = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(prefs_builder, "show-vu-meters")));
         main->get_config()->save(main->get_config_db());
     }
     gtk_widget_destroy(preferences_dlg);
@@ -125,7 +129,7 @@ void gtk_main_window::on_exit_action(GtkWidget *widget, gtk_main_window *main)
     gtk_widget_destroy(GTK_WIDGET(main->toplevel));
 }
 
-void gtk_main_window::add_plugin(plugin_ctl_iface *plugin)
+void gtk_main_window::add_plugin(jack_host *plugin)
 {
     if (toplevel)
     {
@@ -148,7 +152,7 @@ void gtk_main_window::del_plugin(plugin_ctl_iface *plugin)
     if (strip->gui_win)
         strip->gui_win->close();
     vector<GtkWidget *> to_destroy;
-    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = plugins.begin(); i != plugins.end(); i++)
+    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = plugins.begin(); i != plugins.end(); ++i)
     {
         if (i->second == strip)
             to_destroy.push_back(i->second->strip_table);
@@ -175,7 +179,7 @@ void gtk_main_window::set_window(plugin_ctl_iface *plugin, plugin_gui_window *gu
 
 void gtk_main_window::refresh_all_presets(bool builtin_too)
 {
-    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = plugins.begin(); i != plugins.end(); i++)
+    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = plugins.begin(); i != plugins.end(); ++i)
     {
         if (i->second && i->second->gui_win) {
             char ch = '0';
@@ -186,7 +190,7 @@ void gtk_main_window::refresh_all_presets(bool builtin_too)
 }
 
 static gboolean
-gui_button_pressed(GtkWidget *button, gtk_main_window::plugin_strip *strip)
+gui_button_pressed(GtkWidget *button, plugin_strip *strip)
 {
     GtkToggleButton *tb = GTK_TOGGLE_BUTTON(button);
     if ((gtk_toggle_button_get_active(tb) != 0) == (strip->gui_win != NULL))
@@ -199,17 +203,30 @@ gui_button_pressed(GtkWidget *button, gtk_main_window::plugin_strip *strip)
     }
     return TRUE;
 }
-
 static gboolean
-extra_button_pressed(GtkWidget *button, gtk_main_window::plugin_strip *strip)
+connect_button_pressed(GtkWidget *button, plugin_strip *strip)
 {
+    //GtkToggleButton *tb = GTK_TOGGLE_BUTTON(button);
+    if (strip->connector) {
+        strip->connector->close();
+        strip->connector = NULL;
+    } else {
+        strip->connector = new calf_connector(strip);
+    }
+    return TRUE;
+}
+static gboolean
+extra_button_pressed(GtkWidget *button, plugin_strip *strip)
+{
+    if (strip->connector)
+        strip->connector->close();
     strip->main_win->owner->remove_plugin(strip->plugin);
     return TRUE;
 }
 
 void gtk_main_window::show_rack_ears(bool show)
 {
-    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = plugins.begin(); i != plugins.end(); i++)
+    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = plugins.begin(); i != plugins.end(); ++i)
     {
         if (show)
         {
@@ -224,12 +241,34 @@ void gtk_main_window::show_rack_ears(bool show)
     }
 }
 
-gtk_main_window::plugin_strip *gtk_main_window::create_strip(plugin_ctl_iface *plugin)
+void gtk_main_window::show_vu_meters(bool show)
+{
+    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = plugins.begin(); i != plugins.end(); ++i)
+    {
+        if (show)
+        {
+            if (i->second->inBox)
+                gtk_widget_show(i->second->inBox);
+            if (i->second->outBox)
+                gtk_widget_show(i->second->outBox);
+        }
+        else
+        {
+            if (i->second->inBox)
+                gtk_widget_hide(i->second->inBox);
+            if (i->second->outBox)
+                gtk_widget_hide(i->second->outBox);
+        }
+    }
+}
+
+plugin_strip *gtk_main_window::create_strip(jack_host *plugin)
 {
     plugin_strip *strip = new plugin_strip;
     strip->main_win = this;
     strip->plugin = plugin;
     strip->gui_win = NULL;
+    strip->connector = NULL;
     strip->id = plugins.size();
     
     GtkAttachOptions ao = (GtkAttachOptions)(GTK_EXPAND | GTK_FILL);
@@ -288,43 +327,52 @@ gtk_main_window::plugin_strip *gtk_main_window::create_strip(plugin_ctl_iface *p
     
     // title @ 1, 1
     const plugin_metadata_iface *metadata = plugin->get_metadata_iface();
-    //sprintf(buf, "<span size=\"12000\">%s</span>", metadata->get_label());
     GtkWidget *title = gtk_label_new(NULL);
     gtk_widget_set_name(GTK_WIDGET(title), "Calf-Rack-Title");
     gtk_label_set_markup(GTK_LABEL(title), metadata->get_label());
-    gtk_table_attach(GTK_TABLE(strip->strip_table), title, 1, 2, row, row + 1, ao, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL | GTK_SHRINK) , 10, 10);
+    gtk_label_set_justify(GTK_LABEL(title), GTK_JUSTIFY_RIGHT);
+    gtk_table_attach(GTK_TABLE(strip->strip_table), title, 1, 2, row, row + 1, (GtkAttachOptions)0, (GtkAttachOptions)0 , 10, 0);
     gtk_widget_show(title);
     
     // open button
-    GtkWidget *label = gtk_toggle_button_new_with_label("Edit");
+    GtkWidget *label = calf_toggle_button_new("Edit");
     strip->button = label;
     gtk_widget_set_size_request(GTK_WIDGET(label), 80, -1);
-    gtk_signal_connect(GTK_OBJECT(label), "toggled", G_CALLBACK(gui_button_pressed), 
+    g_signal_connect(GTK_OBJECT(label), "toggled", G_CALLBACK(gui_button_pressed), 
         (plugin_ctl_iface *)strip);
     gtk_widget_show(strip->button);
 
+    // connect button
+    GtkWidget *con = calf_toggle_button_new("Connect");
+    strip->con = con;
+    gtk_widget_set_size_request(GTK_WIDGET(con), -1, -1);
+    g_signal_connect(GTK_OBJECT(con), "toggled", G_CALLBACK(connect_button_pressed), 
+        (plugin_ctl_iface *)strip);
+    gtk_widget_show(strip->con);
+    
     // delete buton
-    GtkWidget *extra = gtk_button_new_with_label("Remove");
+    GtkWidget *extra = calf_button_new("Remove");
     strip->extra = extra;
-    gtk_widget_set_size_request(GTK_WIDGET(extra), 80, -1);
-    gtk_signal_connect(GTK_OBJECT(extra), "clicked", G_CALLBACK(extra_button_pressed), 
+    //gtk_widget_set_size_request(GTK_WIDGET(extra), 100, -1);
+    g_signal_connect(GTK_OBJECT(extra), "clicked", G_CALLBACK(extra_button_pressed), 
         (plugin_ctl_iface *)strip);
     gtk_widget_show(strip->extra);
     
     // button box @ 1, 2
-    GtkWidget *buttonBox = gtk_hbox_new(TRUE, 10);
-    gtk_box_pack_start(GTK_BOX(buttonBox), GTK_WIDGET(strip->button), TRUE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(buttonBox), GTK_WIDGET(strip->extra), TRUE, TRUE, 0);
-    gtk_table_attach(GTK_TABLE(strip->strip_table), buttonBox, 1, 2, row + 1, row + 2, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL | GTK_SHRINK), GTK_EXPAND, 10, 10);
+    GtkWidget *buttonBox = gtk_hbox_new(FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(buttonBox), GTK_WIDGET(strip->button), FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(buttonBox), GTK_WIDGET(strip->con), FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(buttonBox), GTK_WIDGET(strip->extra), FALSE, FALSE, 0);
+    gtk_table_attach(GTK_TABLE(strip->strip_table), buttonBox, 1, 2, row + 1, row + 2, (GtkAttachOptions)0, (GtkAttachOptions)0, 10, 10);
     gtk_widget_show(buttonBox);
     
     // midi box
     if (metadata->get_midi()) {
         label = calf_led_new();
         GtkWidget *midiBox = gtk_vbox_new(FALSE, 1);
-        gtk_box_pack_start(GTK_BOX(midiBox), GTK_WIDGET(gtk_label_new("MIDI")), TRUE, TRUE, 0);
-        gtk_box_pack_start(GTK_BOX(midiBox), GTK_WIDGET(label), TRUE, TRUE, 0);
-        gtk_table_attach(GTK_TABLE(strip->strip_table), midiBox, 2, 3, row, row + 1, GTK_FILL, GTK_EXPAND, 5, 3);
+        gtk_box_pack_start(GTK_BOX(midiBox), GTK_WIDGET(gtk_label_new("MIDI")), FALSE, FALSE, 0);
+        gtk_box_pack_start(GTK_BOX(midiBox), GTK_WIDGET(label), FALSE, FALSE, 0);
+        gtk_table_attach(GTK_TABLE(strip->strip_table), midiBox, 2, 3, row, row + 1, (GtkAttachOptions)0, (GtkAttachOptions)0, 5, 3);
         gtk_widget_set_size_request(GTK_WIDGET(label), 25, 25);
         strip->midi_in = label;
         gtk_widget_show_all(midiBox);
@@ -337,79 +385,74 @@ gtk_main_window::plugin_strip *gtk_main_window::create_strip(plugin_ctl_iface *p
     }
     strip->midi_in = label;
     
-
-    for (int i = 0; i < 2; i++)
-        strip->audio_in[i] = strip->audio_out[i] = NULL;
+    strip->inBox = NULL;
+    strip->outBox = NULL;
+    strip->audio_in.clear();
+    strip->audio_out.clear();
         
-    if (metadata->get_input_count() == 2) {
+    if (metadata->get_input_count()) {
         
         GtkWidget *inBox  = gtk_vbox_new(FALSE, 1);
         
-        gtk_box_pack_start(GTK_BOX(inBox), gtk_label_new("audio in"),TRUE, TRUE, 0);
+        gtk_box_pack_start(GTK_BOX(inBox), gtk_label_new("AUDIO IN"),FALSE, FALSE, 0);
         
-        label = calf_vumeter_new();
-        calf_vumeter_set_falloff(CALF_VUMETER(label), 2.5);
-        calf_vumeter_set_hold(CALF_VUMETER(label), 1.5);
-        calf_vumeter_set_width(CALF_VUMETER(label), 120);
-        calf_vumeter_set_height(CALF_VUMETER(label), 12);
-        calf_vumeter_set_position(CALF_VUMETER(label), 2);
-        gtk_box_pack_start(GTK_BOX(inBox), label,FALSE, FALSE, 0);
-        strip->audio_in[0] = label;
+        for (int i = 0; i < metadata->get_input_count(); i++)
+        {
+            label = calf_vumeter_new();
+            calf_vumeter_set_falloff(CALF_VUMETER(label), 2.5);
+            calf_vumeter_set_hold(CALF_VUMETER(label), 1.5);
+            calf_vumeter_set_width(CALF_VUMETER(label), 100);
+            calf_vumeter_set_height(CALF_VUMETER(label), 12);
+            calf_vumeter_set_position(CALF_VUMETER(label), 2);
+            gtk_box_pack_start(GTK_BOX(inBox), label, TRUE, TRUE, 0);
+            strip->audio_in.push_back(label);
+        }
         
-        label = calf_vumeter_new();
-        calf_vumeter_set_falloff(CALF_VUMETER(label), 2.5);
-        calf_vumeter_set_hold(CALF_VUMETER(label), 1.5);
-        calf_vumeter_set_width(CALF_VUMETER(label), 120);
-        calf_vumeter_set_height(CALF_VUMETER(label), 12);
-        calf_vumeter_set_position(CALF_VUMETER(label), 2);
-        gtk_box_pack_start(GTK_BOX(inBox), label,FALSE, FALSE, 0);
-        strip->audio_in[1] = label;
+        strip->inBox = gtk_alignment_new(0.f, 0.f, 1.f, 0.f);
+        gtk_container_add(GTK_CONTAINER(strip->inBox), inBox);
         
-        gtk_widget_show_all(inBox);
-        gtk_table_attach(GTK_TABLE(strip->strip_table), inBox, 3, 4, row, row + 1, GTK_FILL, GTK_SHRINK, 5, 3);
+        gtk_table_attach(GTK_TABLE(strip->strip_table), strip->inBox, 3, 4, row, row + 1, ao, ao, 5, 3);
         
-        gtk_widget_set_size_request(GTK_WIDGET(inBox), 160, -1);
+        if (get_config()->vu_meters)
+            gtk_widget_show_all(strip->inBox);
+            
+        gtk_widget_set_size_request(GTK_WIDGET(strip->inBox), 120, -1);
     }
 
-    if (metadata->get_output_count() == 2) {
+    if (metadata->get_output_count()) {
         
         GtkWidget *outBox  = gtk_vbox_new(FALSE, 1);
         
-        gtk_box_pack_start(GTK_BOX(outBox), gtk_label_new("audio out"),TRUE, TRUE, 0);
+        gtk_box_pack_start(GTK_BOX(outBox), gtk_label_new("AUDIO OUT"),TRUE, TRUE, 0);
         
-        label = calf_vumeter_new();
-        calf_vumeter_set_falloff(CALF_VUMETER(label), 2.5);
-        calf_vumeter_set_hold(CALF_VUMETER(label), 1.5);
-        calf_vumeter_set_width(CALF_VUMETER(label), 120);
-        calf_vumeter_set_height(CALF_VUMETER(label), 12);
-        calf_vumeter_set_position(CALF_VUMETER(label), 2);
-        gtk_box_pack_start(GTK_BOX(outBox), label,FALSE, FALSE, 0);
-        strip->audio_out[0] = label;
+        for (int i = 0; i < metadata->get_output_count(); i++)
+        {
+            label = calf_vumeter_new();
+            calf_vumeter_set_falloff(CALF_VUMETER(label), 2.5);
+            calf_vumeter_set_hold(CALF_VUMETER(label), 1.5);
+            calf_vumeter_set_width(CALF_VUMETER(label), 100);
+            calf_vumeter_set_height(CALF_VUMETER(label), 12);
+            calf_vumeter_set_position(CALF_VUMETER(label), 2);
+            gtk_box_pack_start(GTK_BOX(outBox), label,FALSE, FALSE, 0);
+            strip->audio_out.push_back(label);
+        }
         
-        label = calf_vumeter_new();
-        calf_vumeter_set_falloff(CALF_VUMETER(label), 2.5);
-        calf_vumeter_set_hold(CALF_VUMETER(label), 1.5);
-        calf_vumeter_set_width(CALF_VUMETER(label), 120);
-        calf_vumeter_set_height(CALF_VUMETER(label), 12);
-        calf_vumeter_set_position(CALF_VUMETER(label), 2);
-        gtk_box_pack_start(GTK_BOX(outBox), label,FALSE, FALSE, 0);
-        strip->audio_out[1] = label;
+        strip->outBox = gtk_alignment_new(0.f, 0.f, 1.f, 0.f);
+        gtk_container_add(GTK_CONTAINER(strip->outBox), outBox);
         
-        gtk_widget_show_all(outBox);
-        gtk_table_attach(GTK_TABLE(strip->strip_table), outBox, 4, 5, row, row + 1, GTK_FILL, GTK_SHRINK, 5, 3);
+        gtk_table_attach(GTK_TABLE(strip->strip_table), strip->outBox, 4, 5, row, row + 1, ao, ao, 5, 3);
         
-        gtk_widget_set_size_request(GTK_WIDGET(outBox), 160, -1);
+        if (get_config()->vu_meters)
+            gtk_widget_show_all(strip->outBox);
+            
+        gtk_widget_set_size_request(GTK_WIDGET(strip->outBox), 120, -1);
     }
 
     // other stuff bottom right
-    GtkWidget *paramBox = gtk_hbox_new(TRUE, 10);
-    
-    gtk_box_pack_start(GTK_BOX(paramBox), gtk_label_new(NULL), TRUE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(paramBox), gtk_label_new(NULL), TRUE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(paramBox), gtk_label_new(NULL), TRUE, TRUE, 0);
+    GtkWidget *paramBox = gtk_hbox_new(FALSE, 0);
     
     GtkWidget *logoImg     = gtk_image_new_from_file(PKGLIBDIR "/logo_button.png");
-    gtk_box_pack_end(GTK_BOX(paramBox), GTK_WIDGET(logoImg), FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(paramBox), GTK_WIDGET(logoImg), TRUE, TRUE, 0);
     
     gtk_table_attach(GTK_TABLE(strip->strip_table), paramBox, 3, 5, row + 1, row + 2, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 10, 0);
     gtk_widget_show_all(GTK_WIDGET(paramBox));
@@ -434,7 +477,7 @@ void gtk_main_window::sort_strips()
     int float_size = get_config()->float_size; // amout of rows/cols before line break
     int posx, posy;
     gtk_table_resize(GTK_TABLE(strips_table), (int)(plugins.size() / float_size + 1), float_size);
-    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = plugins.begin(); i != plugins.end(); i++)
+    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = plugins.begin(); i != plugins.end(); ++i)
     {
         switch (rack_float) {
             case 0:
@@ -453,7 +496,7 @@ void gtk_main_window::sort_strips()
             g_object_ref(i->second->strip_table);
             gtk_container_remove(GTK_CONTAINER(strips_table), GTK_WIDGET(i->second->strip_table));
         }
-        gtk_table_attach(GTK_TABLE(strips_table), i->second->strip_table, posx, posx + 1, posy, posy + 1, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL | GTK_SHRINK), (GtkAttachOptions)(0), 0, 0);
+        gtk_table_attach(GTK_TABLE(strips_table), i->second->strip_table, posx, posx + 1, posy, posy + 1, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL | GTK_SHRINK), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL | GTK_SHRINK), 0, 0);
         if(rem) g_object_unref(i->second->strip_table);
     }
 }
@@ -468,7 +511,8 @@ void gtk_main_window::update_strip(plugin_ctl_iface *plugin)
 void gtk_main_window::open_gui(plugin_ctl_iface *plugin)
 {
     plugin_gui_window *gui_win = new plugin_gui_window(this, this);
-    gui_win->create(plugin, (owner->get_client_name() + " - " + plugin->get_metadata_iface()->get_label()).c_str(), plugin->get_metadata_iface()->get_id());
+    std::string title = "Calf - ";
+    gui_win->create(plugin, (title + plugin->get_metadata_iface()->get_label()).c_str(), plugin->get_metadata_iface()->get_id()); //(owner->get_client_name() + " - " + plugin->get_metadata_iface()->get_label()).c_str(), plugin->get_metadata_iface()->get_id());
     gtk_widget_show(GTK_WIDGET(gui_win->toplevel));
     plugins[plugin]->gui_win = gui_win; 
 }
@@ -485,6 +529,35 @@ static const char *plugin_post_xml =
 "  </menubar>\n"
 "</ui>\n"
 ;
+#define countof(X) ( (size_t) ( sizeof(X)/sizeof*(X) ) )
+void gtk_main_window::register_icons()
+{
+    const char *names[]={"Allpass", "Amplifier", "Analyser",
+                         "Bandpass", "Chorus", "Comb", "Compressor",
+                         "Constant", "Converter", "Delay", "Distortion",
+                         "Dynamics", "Envelope", "EQ", "Expander",
+                         "Filter", "Flanger", "Function", "Gate",
+                         "Generator", "Highpass", "Instrument",
+                         "Limiter", "Mixer", "Modulator", "MultiEQ",
+                         "Oscillator", "ParaEQ", "Phaser", "Pitch",
+                         "Reverb", "Simulator", "Spatial", "Spectral",
+                         "Utility", "Waveshaper"};
+    factory = gtk_icon_factory_new ();
+    for (size_t i = 0; i < countof(names); i++) {
+        char name[1024];
+        strcpy(name, "LV2-");
+        strcat(name, names[i]);
+        if (!gtk_icon_factory_lookup(factory, name)) {
+            std::string iname = std::string(PKGLIBDIR) + "icons/LV2/" + names[i] + ".svg";
+            GdkPixbuf *buf    = gdk_pixbuf_new_from_file_at_size(iname.c_str(), 64, 64, NULL);
+            GtkIconSet *icon  = gtk_icon_set_new_from_pixbuf(buf);
+            gtk_icon_factory_add (factory, name, icon);
+            gtk_icon_set_unref(icon);
+            g_object_unref(buf);
+        }
+    }
+    gtk_icon_factory_add_default(factory);
+}
 
 void gtk_main_window::add_plugin_action(GtkWidget *src, gpointer data)
 {
@@ -501,13 +574,47 @@ std::string gtk_main_window::make_plugin_list(GtkActionGroup *actions)
 {
     string s = plugin_pre_xml;
     const plugin_registry::plugin_vector &plugins = plugin_registry::instance().get_all();
-    for(unsigned int i = 0; i < plugins.size(); i++)
+    std::string type   = "";
+    std::string tmp    = "";
+    std::string last   = "";
+    unsigned int count = 0;
+    unsigned int size  = plugins.size();
+    
+    const plugin_metadata_iface *p = plugins[0];
+    
+    for(unsigned int i = 0; i <= size; i++)
     {
-        const plugin_metadata_iface *p = plugins[i];
-        string action_name = "Add" + string(p->get_id())+"Action";
-        s += string("<menuitem action=\"") + action_name + "\" />";
-        GtkActionEntry ae = { action_name.c_str(), NULL, p->get_label(), NULL, NULL, (GCallback)add_plugin_action };
-        gtk_action_group_add_actions_full(actions, &ae, 1, (gpointer)new add_plugin_params(this, p->get_id()), action_destroy_notify);
+        if (i < size) {
+            p = plugins[i];
+            type = (p->get_plugin_info()).plugin_type;
+            type = type.substr(0, type.length() - 6);
+        }
+        if (type != last or i >= size or !i) {
+            
+            if (i) {
+                if (count > 1) {
+                    s += "<menu action='" + last + "'>" + tmp + "</menu>";
+                    GtkAction *a = gtk_action_new(last.c_str(), last.c_str(), NULL, ("LV2-" + last).c_str());
+                    gtk_action_group_add_action(actions, a);
+                } else {
+                    s += tmp;
+                }
+            }
+            tmp = "";
+            last = type;
+            count = 0;
+        }
+        if (i < size) {
+            std::string action = "Add" + string(p->get_id()) + "Action";
+            std::string stock  = "LV2-" + type;
+            // TODO:
+            // add lv2 stock icons to plug-ins and not just to menus
+            // GTK_STOCK_OPEN -> ("LV2_" + type).c_str()
+            GtkActionEntry ae  = { action.c_str(), stock.c_str(), p->get_label(), NULL, NULL, (GCallback)add_plugin_action };
+            gtk_action_group_add_actions_full(actions, &ae, 1, (gpointer)new add_plugin_params(this, p->get_id()), action_destroy_notify);
+            tmp   += string("<menuitem always-show-image=\"true\" action=\"") + action + "\" />";
+            count += 1;
+        }
     }
     return s + plugin_post_xml;
 }
@@ -519,10 +626,14 @@ static void window_destroy_cb(GtkWindow *window, gpointer data)
 
 void gtk_main_window::create()
 {
+    register_icons();
     toplevel = GTK_WINDOW(gtk_window_new (GTK_WINDOW_TOPLEVEL));
-    gtk_window_set_title(toplevel, (owner->get_client_name() + " - Calf JACK Host").c_str());
+    std::string title = "Calf JACK Host";
+    gtk_window_set_title(toplevel, title.c_str()); //(owner->get_client_name() + " - Calf JACK Host").c_str());
     gtk_widget_set_name(GTK_WIDGET(toplevel), "Calf-Rack");
     gtk_window_set_icon_name(toplevel, "calf");
+    gtk_window_set_role(toplevel, "calf_rack");
+    
     is_closed = false;
     gtk_window_set_resizable(toplevel, false);
     
@@ -556,7 +667,7 @@ void gtk_main_window::create()
             gtk_misc_set_alignment(GTK_MISC(c->widget), 0.5, 0);
         }
     }
-    for (std::vector<plugin_ctl_iface *>::iterator i = plugin_queue.begin(); i != plugin_queue.end(); i++)
+    for (std::vector<jack_host *>::iterator i = plugin_queue.begin(); i != plugin_queue.end(); ++i)
     {
         plugin_strip *st = create_strip(*i);
         plugins[*i] = st;
@@ -575,13 +686,14 @@ void gtk_main_window::create()
     
     notifier = get_config_db()->add_listener(this);
     on_config_change();
-    gtk_signal_connect(GTK_OBJECT(toplevel), "destroy", G_CALLBACK(window_destroy_cb), this);
+    g_signal_connect(GTK_OBJECT(toplevel), "destroy", G_CALLBACK(window_destroy_cb), this);
 }
 
 void gtk_main_window::on_config_change()
 {
     get_config()->load(get_config_db());
     show_rack_ears(get_config()->rack_ears);    
+    show_vu_meters(get_config()->vu_meters);
     sort_strips();
 }
 
@@ -603,7 +715,7 @@ void gtk_main_window::on_closed()
     is_closed = true;
     toplevel = NULL;
 
-    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = plugins.begin(); i != plugins.end(); i++)
+    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = plugins.begin(); i != plugins.end(); ++i)
     {
         if (i->second && i->second->gui_win) {
             i->second->gui_win->close();
@@ -614,28 +726,39 @@ void gtk_main_window::on_closed()
 
 static inline float LVL(float value)
 {
-    return sqrt(value) * 0.75;
+    return value; //sqrt(value) * 0.75;
 }
 
 gboolean gtk_main_window::on_idle(void *data)
 {
     gtk_main_window *self = (gtk_main_window *)data;
+
     self->owner->on_idle();
-    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = self->plugins.begin(); i != self->plugins.end(); i++)
+    
+    if (!self->refresh_controller.check_redraw(GTK_WIDGET(self->toplevel)))
+        return TRUE;
+
+    for (std::map<plugin_ctl_iface *, plugin_strip *>::iterator i = self->plugins.begin(); i != self->plugins.end(); ++i)
     {
         if (i->second)
         {
             plugin_ctl_iface *plugin = i->first;
             plugin_strip *strip = i->second;
             int idx = 0;
-            if (plugin->get_metadata_iface()->get_input_count() == 2) {
-                calf_vumeter_set_value(CALF_VUMETER(strip->audio_in[0]), LVL(plugin->get_level(idx++)));
-                calf_vumeter_set_value(CALF_VUMETER(strip->audio_in[1]), LVL(plugin->get_level(idx++)));
+            if (strip->inBox && gtk_widget_is_drawable (strip->inBox)) {
+                for (int i = 0; i < (int)strip->audio_in.size(); i++) {
+                    calf_vumeter_set_value(CALF_VUMETER(strip->audio_in[i]), LVL(plugin->get_level(idx++)));
+                }
             }
-            if (plugin->get_metadata_iface()->get_output_count() == 2) {
-                calf_vumeter_set_value(CALF_VUMETER(strip->audio_out[0]), LVL(plugin->get_level(idx++)));
-                calf_vumeter_set_value(CALF_VUMETER(strip->audio_out[1]), LVL(plugin->get_level(idx++)));
+            else
+                idx += strip->audio_in.size();
+            if (strip->outBox && gtk_widget_is_drawable (strip->outBox)) {
+                for (int i = 0; i < (int)strip->audio_out.size(); i++) {
+                    calf_vumeter_set_value(CALF_VUMETER(strip->audio_out[i]), LVL(plugin->get_level(idx++)));
+                }
             }
+            else
+                idx += strip->audio_out.size();
             if (plugin->get_metadata_iface()->get_midi()) {
                 calf_led_set_value (CALF_LED (strip->midi_in), plugin->get_level(idx++));
             }
@@ -736,8 +859,8 @@ void gtk_main_window::report_progress(float percentage, const std::string &messa
             gtk_window_set_modal (GTK_WINDOW (progress_window), TRUE);
             if (toplevel)
                 gtk_window_set_transient_for (GTK_WINDOW (progress_window), toplevel);
+            gtk_widget_show(progress_window);
         }
-        gtk_widget_show(progress_window);
         GtkWidget *pbar = gtk_bin_get_child (GTK_BIN (progress_window));
         if (!message.empty())
             gtk_progress_bar_set_text (GTK_PROGRESS_BAR (pbar), message.c_str());
