@@ -37,14 +37,17 @@ plugin_gui::plugin_gui(plugin_gui_window *_window)
 {
     ignore_stack = 0;
     top_container = NULL;
-    current_control = NULL;
     param_count = 0;
     container = NULL;
     effect_name = NULL;
     preset_access = new gui_preset_access(this);
+    optclosed = false;
+    optwidget = NULL;
+    optwindow = NULL;
+    opttitle = NULL;
 }
 
-param_control *plugin_gui::create_control_from_xml(const char *element, const char *attributes[])
+control_base *plugin_gui::create_widget_from_xml(const char *element, const char *attributes[])
 {
     if (!strcmp(element, "knob"))
         return new knob_param_control;
@@ -60,6 +63,8 @@ param_control *plugin_gui::create_control_from_xml(const char *element, const ch
         return new radio_param_control;
     if (!strcmp(element, "toggle"))
         return new toggle_param_control;
+    if (!strcmp(element, "tap"))
+        return new tap_button_param_control;
     if (!strcmp(element, "spin"))
         return new spin_param_control;
     if (!strcmp(element, "button"))
@@ -88,11 +93,8 @@ param_control *plugin_gui::create_control_from_xml(const char *element, const ch
         return new filechooser_param_control;
     if (!strcmp(element, "listview"))
         return new listview_param_control;
-    return NULL;
-}
-
-control_container *plugin_gui::create_container_from_xml(const char *element, const char *attributes[])
-{
+    if (!strcmp(element, "notebook"))
+        return new notebook_param_control;
     if (!strcmp(element, "table"))
         return new table_container;
     if (!strcmp(element, "vbox"))
@@ -103,8 +105,6 @@ control_container *plugin_gui::create_container_from_xml(const char *element, co
         return new alignment_container;
     if (!strcmp(element, "frame"))
         return new frame_container;
-    if (!strcmp(element, "notebook"))
-        return new notebook_container;
     if (!strcmp(element, "scrolled"))
         return new scrolled_container;
     return NULL;
@@ -116,6 +116,18 @@ void plugin_gui::xml_element_start(void *data, const char *element, const char *
     gui->xml_element_start(element, attributes);
 }
 
+int plugin_gui::get_param_no_by_name(string param_name)
+{
+    int param_no = -1;
+    map<string, int>::iterator it = param_name_map.find(param_name);
+    if (it == param_name_map.end())
+        g_error("Unknown parameter %s", param_name.c_str());
+    else
+        param_no = it->second;
+
+    return param_no;
+}
+
 void plugin_gui::xml_element_start(const char *element, const char *attributes[])
 {
     if (ignore_stack) {
@@ -146,45 +158,13 @@ void plugin_gui::xml_element_start(const char *element, const char *attributes[]
         ignore_stack = 1;
         return;
     }
-    control_container *cc = create_container_from_xml(element, attributes);
-    if (cc != NULL)
-    {
-        cc->attribs = xam;
-        cc->create(this, element, xam);
-        cc->set_std_properties();
-        gtk_container_set_border_width(cc->container, cc->get_int("border"));
-
-        container_stack.push_back(cc);
-        current_control = NULL;
-        return;
-    }
-    if (!container_stack.empty())
-    {
-        current_control = create_control_from_xml(element, attributes);
-        if (current_control)
-        {
-            current_control->control_name = element;
-            current_control->attribs = xam;
-            int param_no = -1;
-            if (xam.count("param"))
-            {
-                map<string, int>::iterator it = param_name_map.find(xam["param"]);
-                if (it == param_name_map.end())
-                    g_error("Unknown parameter %s", xam["param"].c_str());
-                else
-                    param_no = it->second;
-            }
-            if (param_no != -1)
-                current_control->param_variable = plugin->get_metadata_iface()->get_param_props(param_no)->short_name;
-            current_control->create(this, param_no);
-            current_control->set_std_properties();
-            current_control->init_xml(element);
-            current_control->set();
-            current_control->hook_params();
-            return;
-        }
-    }
-    g_error("Unxpected element %s in GUI definition\n", element);
+    control_base *cc = create_widget_from_xml(element, attributes);
+    if (cc == NULL)
+        g_error("Unxpected element %s in GUI definition\n", element);
+    
+    cc->attribs = xam;
+    cc->create(this);
+    stack.push_back(cc);
 }
 
 void plugin_gui::xml_element_end(void *data, const char *element)
@@ -195,36 +175,34 @@ void plugin_gui::xml_element_end(void *data, const char *element)
         return;
     }
     if (!strcmp(element, "if"))
-    {
         return;
-    }
-    if (gui->current_control)
+
+    control_base *control = gui->stack.back();
+    control->created();
+    
+    gui->stack.pop_back();
+    if (gui->stack.empty())
     {
-        (*gui->container_stack.rbegin())->add(gui->current_control->widget, gui->current_control);
-        gui->current_control = NULL;
-        return;
-    }
-    unsigned int ss = gui->container_stack.size();
-    if (ss > 1) {
-        gui->container_stack[ss - 2]->add(GTK_WIDGET(gui->container_stack[ss - 1]->container), gui->container_stack[ss - 1]);
+        gui->top_container = control;
+        gtk_widget_show_all(control->widget);
     }
     else
-        gui->top_container = gui->container_stack[0];
-    gui->container_stack.pop_back();
+        gui->stack.back()->add(control);
 }
 
 
 GtkWidget *plugin_gui::create_from_xml(plugin_ctl_iface *_plugin, const char *xml)
 {
-    current_control = NULL;
     top_container = NULL;
     parser = XML_ParserCreate("UTF-8");
     plugin = _plugin;
-    container_stack.clear();
+    stack.clear();
     ignore_stack = 0;
     
     param_name_map.clear();
+    read_serials.clear();
     int size = plugin->get_metadata_iface()->get_param_count();
+    read_serials.resize(size);
     for (int i = 0; i < size; i++)
         param_name_map[plugin->get_metadata_iface()->get_param_props(i)->short_name] = i;
     
@@ -276,9 +254,9 @@ GtkWidget *plugin_gui::create_from_xml(plugin_ctl_iface *_plugin, const char *xm
     gtk_table_attach(GTK_TABLE(decoTable), GTK_WIDGET(leftBox),   0, 1, 0, 1, (GtkAttachOptions)(0), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
     gtk_table_attach(GTK_TABLE(decoTable), GTK_WIDGET(rightBox),  2, 3, 0, 1, (GtkAttachOptions)(0), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
         
-    gtk_table_attach(GTK_TABLE(decoTable), GTK_WIDGET(top_container->container), 1, 2, 0, 1, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 15, 5);
+    gtk_table_attach(GTK_TABLE(decoTable), top_container->widget, 1, 2, 0, 1, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 15, 5);
     gtk_container_add( GTK_CONTAINER(eventbox), decoTable );
-    gtk_widget_set_name( GTK_WIDGET(eventbox), "Calf-whatever" );
+    gtk_widget_set_name( GTK_WIDGET(eventbox), "Calf-Plugin" );
     
     // create window with viewport
 //    GtkWidget *sw = gtk_scrolled_window_new(NULL, NULL);
@@ -313,17 +291,54 @@ void plugin_gui::send_status(const char *key, const char *value)
     }
 }
 
+void plugin_gui::remove_param_ctl(int param, param_control *ctl)
+{
+    std::multimap<int, param_control *>::iterator it = par2ctl.find(param);
+    while(it != par2ctl.end() && it->first == param)
+    {
+        if (it->second == ctl)
+        {
+            std::multimap<int, param_control *>::iterator orig = it;
+            ++orig;
+            par2ctl.erase(it, orig);
+            it = orig;
+        }
+        else
+            ++it;
+    }
+    unsigned last = params.size() - 1;
+    for (unsigned i = 0; i < params.size(); ++i)
+    {
+        if (params[i] == ctl)
+        {
+            if (i != last)
+                std::swap(params[i], params[last]);
+            params.erase(params.begin() + last, params.end());
+        }
+    }
+}
+
 void plugin_gui::on_idle()
 {
-    for (unsigned int i = 0; i < params.size(); i++)
+    set<unsigned> changed;
+    for (unsigned i = 0; i < read_serials.size(); i++)
     {
-        if (params[i]->param_no != -1)
+        int write_serial = plugin->get_write_serial(i);
+        if (write_serial - read_serials[i] > 0)
         {
-            const parameter_properties &props = *plugin->get_metadata_iface()->get_param_props(params[i]->param_no);
+            read_serials[i] = write_serial;
+            changed.insert(i);
+        }
+    }
+    for (unsigned i = 0; i < params.size(); i++)
+    {
+        int param_no = params[i]->param_no;
+        if (param_no != -1)
+        {
+            const parameter_properties &props = *plugin->get_metadata_iface()->get_param_props(param_no);
             bool is_output = (props.flags & PF_PROP_OUTPUT) != 0;
-            if (is_output) {
+            if (is_output || (param_no != -1 && changed.count(param_no)))
                 params[i]->set();
-            }
         }
         params[i]->on_idle();
     }    
@@ -346,7 +361,7 @@ void plugin_gui::refresh(int param_no, param_control *originator)
     {
         if (it->second != originator)
             it->second->set();
-        it++;
+        ++it;
     }
 }
 
@@ -390,13 +405,149 @@ void plugin_gui::show_rack_ears(bool show)
     }
 }
 
+void plugin_gui::on_automation_add(GtkWidget *widget, void *user_data)
+{
+    plugin_gui *self = (plugin_gui *)user_data;
+    self->plugin->add_automation(self->context_menu_last_designator, automation_range(0, 1, self->context_menu_param_no));
+}
+
+void plugin_gui::on_automation_delete(GtkWidget *widget, void *user_data)
+{
+    automation_menu_entry *ame = (automation_menu_entry *)user_data;
+    ame->gui->plugin->delete_automation(ame->source, ame->gui->context_menu_param_no);
+}
+
+void plugin_gui::on_automation_set_lower_or_upper(automation_menu_entry *ame, bool is_upper)
+{
+    const parameter_properties *props = plugin->get_metadata_iface()->get_param_props(context_menu_param_no);
+    float mapped = props->to_01(plugin->get_param_value(context_menu_param_no));
+    
+    automation_map mappings;
+    plugin->get_automation(context_menu_param_no, mappings);
+    automation_map::const_iterator i = mappings.find(ame->source);
+    if (i != mappings.end())
+    {
+        if (is_upper)
+            plugin->add_automation(context_menu_last_designator, automation_range(i->second.min_value, mapped, context_menu_param_no));
+        else
+            plugin->add_automation(context_menu_last_designator, automation_range(mapped, i->second.max_value, context_menu_param_no));
+    }
+}
+
+void plugin_gui::on_automation_set_lower(GtkWidget *widget, void *user_data)
+{
+    automation_menu_entry *ame = (automation_menu_entry *)user_data;
+    ame->gui->on_automation_set_lower_or_upper(ame, false);
+}
+
+void plugin_gui::on_automation_set_upper(GtkWidget *widget, void *user_data)
+{
+    automation_menu_entry *ame = (automation_menu_entry *)user_data;
+    ame->gui->on_automation_set_lower_or_upper(ame, true);
+}
+
+void plugin_gui::cleanup_automation_entries()
+{
+    for (int i = 0; i < (int)automation_menu_callback_data.size(); i++)
+        delete automation_menu_callback_data[i];
+    automation_menu_callback_data.clear();
+}
+
+void plugin_gui::on_control_popup(param_control *ctl, int param_no)
+{
+    cleanup_automation_entries();
+    if (param_no == -1)
+        return;
+    context_menu_param_no = param_no;
+    GtkWidget *menu = gtk_menu_new();
+    
+    multimap<uint32_t, automation_range> mappings;
+    plugin->get_automation(param_no, mappings);
+    
+    context_menu_last_designator = plugin->get_last_automation_source();
+    
+    GtkWidget *item;
+    if (context_menu_last_designator != 0xFFFFFFFF)
+    {
+        stringstream ss;
+        ss << "_Bind to: Ch" << (1 + (context_menu_last_designator >> 8)) << ", CC#" << (context_menu_last_designator & 127);
+        item = gtk_menu_item_new_with_mnemonic(ss.str().c_str());
+        g_signal_connect(item, "activate", (GCallback)on_automation_add, this);
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    }
+    else
+    {
+        item = gtk_menu_item_new_with_label("Send CC to automate");
+        gtk_widget_set_sensitive(item, FALSE);
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    }
+    
+    for(multimap<uint32_t, automation_range>::const_iterator i = mappings.begin(); i != mappings.end(); ++i)
+    {
+        automation_menu_entry *ame = new automation_menu_entry(this, i->first);
+        automation_menu_callback_data.push_back(ame);
+        stringstream ss;
+        ss << "Mapping: Ch" << (1 + (i->first >> 8)) << ", CC#" << (i->first & 127);
+        item = gtk_menu_item_new_with_label(ss.str().c_str());
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+        
+        GtkWidget *submenu = gtk_menu_new();        
+        gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
+        
+        item = gtk_menu_item_new_with_mnemonic("_Delete");
+        g_signal_connect(item, "activate", (GCallback)on_automation_delete, ame);
+        gtk_menu_shell_append(GTK_MENU_SHELL(submenu), item);
+
+        item = gtk_menu_item_new_with_mnemonic("Set _lower limit");
+        g_signal_connect(item, "activate", (GCallback)on_automation_set_lower, ame);
+        gtk_menu_shell_append(GTK_MENU_SHELL(submenu), item);
+
+        item = gtk_menu_item_new_with_mnemonic("Set _upper limit");
+        g_signal_connect(item, "activate", (GCallback)on_automation_set_upper, ame);
+        gtk_menu_shell_append(GTK_MENU_SHELL(submenu), item);
+        //g_signal_connect(item, "activate", (GCallback)on_automation_add, this);
+        
+    }
+    
+    gtk_widget_show_all(menu);
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time());
+}
+
+void plugin_gui::destroy_child_widgets(GtkWidget *parent)
+{
+    if (parent && GTK_IS_CONTAINER(parent))
+    {
+        GList *children = gtk_container_get_children(GTK_CONTAINER(parent));
+        for(GList *p = children; p; p = p->next)
+            gtk_widget_destroy(GTK_WIDGET(p->data));
+        g_list_free(children);
+    }
+}
+
 plugin_gui::~plugin_gui()
 {
+    cleanup_automation_entries();
     delete preset_access;
-    for (std::vector<param_control *>::iterator i = params.begin(); i != params.end(); i++)
+}
+
+/***************************** GUI environment ********************************************/
+
+bool window_update_controller::check_redraw(GtkWidget *toplevel)
+{
+    GdkWindow *gdkwin = gtk_widget_get_window(toplevel);
+    if (!gdkwin)
+        return false;
+
+    if (!gdk_window_is_viewable(gdkwin))
+        return false;
+    GdkWindowState state = gdk_window_get_state(gdkwin);
+    if (state & GDK_WINDOW_STATE_ICONIFIED)
     {
-        delete *i;
+        ++refresh_counter;
+        if (refresh_counter & 15)
+            return false;
     }
+    return true;
 }
 
 /***************************** GUI environment ********************************************/
@@ -420,4 +571,3 @@ gui_environment::~gui_environment()
     if (keyfile)
         g_key_file_free(keyfile);
 }
-
