@@ -36,13 +36,16 @@ host_session *host_session::instance = NULL;
 
 host_session::host_session(session_environment_iface *se)
 {
-    client_name = "calf";
+    client_name = "Calf Studio Gear";
+    calfjackhost_cmd = "calfjackhost";
     session_env = se;
     autoconnect_midi_index = -1;
     gui_win = NULL;
     session_manager = NULL;
     only_load_if_exists = false;
     save_file_on_next_idle_call = false;
+    quit_on_next_idle_call = 0;
+    handle_event_on_next_idle_call = NULL;
 
     main_win = session_env->create_main_window();
     main_win->set_owner(this);
@@ -54,7 +57,7 @@ std::string host_session::get_next_instance_name(const std::string &effect_name)
         return effect_name;
     for (int i = 2; ; i++)
     {
-        string tmp = string(effect_name) + i2s(i);
+        string tmp = string(effect_name) + " (" + i2s(i) + ")";
         if (!instances.count(tmp))
             return tmp;
     }
@@ -66,7 +69,7 @@ void host_session::add_plugin(string name, string preset, string instance_name)
 {
     if (instance_name.empty())
         instance_name = get_next_instance_name(name);
-    jack_host *jh = create_jack_host(name.c_str(), instance_name, main_win);
+    jack_host *jh = create_jack_host(&client, name.c_str(), instance_name, main_win);
     if (!jh) {
         string s = 
         #define PER_MODULE_ITEM(name, isSynth, jackname) jackname ", "
@@ -77,7 +80,7 @@ void host_session::add_plugin(string name, string preset, string instance_name)
         throw text_exception("Unknown plugin name; allowed are: " + s);
     }
     instances.insert(jh->instance_name);
-    jh->create(&client);
+    jh->create();
     
     plugins.push_back(jh);
     client.add(jh);
@@ -111,22 +114,62 @@ void host_session::open()
     if (!output_name.empty()) client.output_name = output_name;
     if (!midi_name.empty()) client.midi_name = midi_name;
     
-    client.open(client_name.c_str());
+    client.open(client_name.c_str(), !jack_session_id.empty() ? jack_session_id.c_str() : NULL);
+    jack_set_session_callback(client.client, session_callback, this);
     main_win->add_condition("jackhost");
     main_win->add_condition("directlink");
     main_win->add_condition("configure");
+    client.create_automation_input();
     if (!session_manager || !session_manager->is_being_restored()) 
         create_plugins_from_list();
     main_win->create();
 }
 
+void host_session::session_callback(jack_session_event_t *event, void *arg)
+{
+    printf("session callback type %d\n", (int)event->type);
+    switch(event->type)
+    {
+    case JackSessionSave:
+    case JackSessionSaveAndQuit:
+    case JackSessionSaveTemplate:
+        host_session *hs = (host_session *)arg;
+        hs->handle_event_on_next_idle_call = event;
+    }
+    // XXXKF if more than one event happen in a short sequence, the other event
+    // may be lost. This calls for implementing a proper event queue.
+}
+
+void host_session::handle_jack_session_event(jack_session_event_t *event)
+{
+    try {
+        asprintf(&event->command_line, "%s --load ${SESSION_DIR}" G_DIR_SEPARATOR_S "rack.xml --session-id %s" , calfjackhost_cmd.c_str(), event->client_uuid);
+        string fn = event->session_dir;
+        fn += "rack.xml";
+        save_file(fn.c_str());
+    }
+    catch(...)
+    {
+        event->flags = JackSessionSaveError;
+        // let the server know that the save operation failed
+        jack_session_reply(client.client, event);
+        jack_session_event_free(event);
+        throw;
+    }
+
+    if (event->type == JackSessionSaveAndQuit)
+        quit_on_next_idle_call = 1;
+    jack_session_reply(client.client, event);
+    jack_session_event_free(event);
+}
+
 void host_session::new_plugin(const char *name)
 {
-    jack_host *jh = create_jack_host(name, get_next_instance_name(name), main_win);
+    jack_host *jh = create_jack_host(&client, name, get_next_instance_name(name), main_win);
     if (!jh)
         return;
     instances.insert(jh->instance_name);
-    jh->create(&client);
+    jh->create();
     
     plugins.push_back(jh);
     client.add(jh);
@@ -243,6 +286,7 @@ void host_session::connect()
                     break;
                 }
             }
+            free(ports);
         }
     }
     if (!load_name.empty())
@@ -280,6 +324,7 @@ void host_session::close()
     main_win->on_closed();
     client.deactivate();
     client.delete_plugins();
+    client.destroy_automation_input();
     client.close();
 }
 
@@ -310,6 +355,11 @@ char *host_session::open_file(const char *name)
             {
                 add_plugin(ps.type, "", ps.instance_name);
                 pl.presets[ps.preset_offset].activate(plugins[i]);
+                for (size_t j = 0; j < ps.automation_entries.size(); ++j)
+                {
+                    const pair<string, string> &p = ps.automation_entries[j];
+                    plugins[i]->configure(p.first.c_str(), p.second.c_str());
+                }
                 main_win->refresh_plugin(plugins[i]);
             }
         }
@@ -324,35 +374,60 @@ char *host_session::open_file(const char *name)
     return NULL;
 }
 
+struct gather_automation_params: public send_configure_iface
+{
+    stringstream *xml;
+    dictionary *dict;
+    gather_automation_params(stringstream &_xml)
+    : xml(&_xml)
+    , dict(NULL)
+    {}
+    gather_automation_params(dictionary &_dict)
+    : xml(NULL)
+    , dict(&_dict)
+    {}
+    
+    virtual void send_configure(const char *key, const char *value)
+    {
+        if (xml)
+            *xml << "<automation key=\"" << key << "\" value=\"" << value << "\" />" << endl;
+        if (dict && key && value)
+            (*dict)[key] = value;
+    }
+};
+
 char *host_session::save_file(const char *name)
 {
     string i_name = stripfmt(client.input_name);
     string o_name = stripfmt(client.output_name);
     string m_name = stripfmt(client.midi_name);
-    string data;
-    data = "<?xml version=\"1.1\" encoding=\"utf-8\">\n";
-    data = "<rack>\n";
+    stringstream data;
+    data << "<?xml version=\"1.1\" encoding=\"utf-8\"?>" << endl;
+    data << "<rack>";
     for (unsigned int i = 0; i < plugins.size(); i++) {
         jack_host *p = plugins[i];
         plugin_preset preset;
         preset.plugin = p->metadata->get_id();
         preset.get_from(p);
-        data += "<plugin";
-        data += to_xml_attr("type", preset.plugin);
-        data += to_xml_attr("instance-name", p->instance_name);
+        data << "<plugin";
+        data << to_xml_attr("type", preset.plugin);
+        data << to_xml_attr("instance-name", p->instance_name);
         if (p->metadata->get_input_count())
-            data += to_xml_attr("input-index", p->get_inputs()[0].name.substr(i_name.length()));
+            data << to_xml_attr("input-index", p->get_inputs()[0].name.substr(i_name.length()));
         if (p->metadata->get_output_count())
-            data += to_xml_attr("output-index", p->get_outputs()[0].name.substr(o_name.length()));
+            data << to_xml_attr("output-index", p->get_outputs()[0].name.substr(o_name.length()));
         if (p->get_midi_port())
-            data += to_xml_attr("midi-index", p->get_midi_port()->name.substr(m_name.length()));
-        data += ">\n";
-        data += preset.to_xml();
-        data += "</plugin>\n";
+            data << to_xml_attr("midi-index", p->get_midi_port()->name.substr(m_name.length()));
+        data << ">" << endl;
+        data << preset.to_xml();
+        gather_automation_params gap(data);
+        p->send_automation_configures(&gap);
+        data << "</plugin>" << endl;
     }
-    data += "</rack>\n";
+    data << "</rack>" << endl;
+    string datastr(data.str());
     FILE *f = fopen(name, "w");
-    if (!f || 1 != fwrite(data.c_str(), data.length(), 1, f))
+    if (!f || 1 != fwrite(datastr.c_str(), datastr.length(), 1, f))
     {
         int e = errno;
         if (f)
@@ -382,9 +457,11 @@ void host_session::load(session_load_iface *stream)
         if (!strncmp(key.c_str(), "Plugin", 6))
         {
             unsigned int nplugin = atoi(key.c_str() + 6);
-            dictionary dict;
+            dictionary dict, automation;
             decode_map(dict, data);
             data = dict["preset"];
+            if (dict.count("automation"))
+                decode_map(automation, dict["automation"]);
             string instance_name;
             if (dict.count("instance_name")) instance_name = dict["instance_name"];
             if (dict.count("input_name")) client.input_nr = atoi(dict["input_name"].c_str());
@@ -398,6 +475,8 @@ void host_session::load(session_load_iface *stream)
                 add_plugin(tmp.presets[0].plugin, "", instance_name);
                 tmp.presets[0].activate(plugins[nplugin]);
                 main_win->refresh_plugin(plugins[nplugin]);
+                for(dictionary::const_iterator i = automation.begin(); i != automation.end(); ++i)
+                    plugins[nplugin]->configure(i->first.c_str(), i->second.c_str());
             }
         }
     }
@@ -422,7 +501,7 @@ void host_session::save(session_save_iface *stream)
         plugin_preset preset;
         preset.plugin = p->metadata->get_id();
         preset.get_from(p);
-        sprintf(ss, "Plugin%d", i);
+        snprintf(ss, sizeof(ss), "Plugin%d", i);
         pstr = preset.to_xml();
         tmp.clear();
         tmp["instance_name"] = p->instance_name;
@@ -433,14 +512,28 @@ void host_session::save(session_save_iface *stream)
         if (p->get_midi_port())
             tmp["midi_name"] = p->get_midi_port()->name.substr(m_name.length());
         tmp["preset"] = pstr;
+        dictionary automation;
+        gather_automation_params gap(automation);
+        p->send_automation_configures(&gap);
+        tmp["automation"] = encode_map(automation);
+
         pstr = encode_map(tmp);
         stream->write_next_item(ss, pstr);
     }
 }
 
-void host_session::sigusr1handler(int signum)
+void host_session::signal_handler(int signum)
 {
-    instance->save_file_on_next_idle_call = true;
+    switch (signum)
+    {
+    case SIGUSR1:
+        instance->save_file_on_next_idle_call = true;
+        break;
+    case SIGTERM:
+    case SIGHUP:
+        instance->quit_on_next_idle_call = signum;
+        break;
+    }
 }
 
 void host_session::on_idle()
@@ -451,17 +544,32 @@ void host_session::on_idle()
         main_win->save_file();
         printf("LADISH Level 1 support: file '%s' saved\n", get_current_filename().c_str());
     }
+
+    if (handle_event_on_next_idle_call)
+    {
+        jack_session_event_t *ev = handle_event_on_next_idle_call;
+        handle_event_on_next_idle_call = NULL;
+        handle_jack_session_event(ev);
+    }
+    if (quit_on_next_idle_call > 0)
+    {
+        printf("Quit requested through signal %d\n", quit_on_next_idle_call);
+        quit_on_next_idle_call = -quit_on_next_idle_call; // mark the event as handled but preserve signal number
+        session_env->quit_gui_loop();
+    }
 }
 
-void host_session::set_ladish_handler()
+void host_session::set_signal_handlers()
 {
     instance = this;
-    
+
     struct sigaction sa;
-    sa.sa_handler = sigusr1handler;
+    sa.sa_handler = signal_handler;
     sigemptyset(&sa.sa_mask);
     sa.sa_flags = SA_RESTART;
-    sigaction(SIGUSR1, &sa, NULL);    
+    sigaction(SIGTERM, &sa, NULL);
+    sigaction(SIGHUP,  &sa, NULL);
+    sigaction(SIGUSR1, &sa, NULL);
 }
 
 void host_session::reorder_plugins()
@@ -485,3 +593,8 @@ void host_session::set_current_filename(const std::string &name)
 {
     current_filename = name;
 }
+
+host_session::~host_session()
+{
+    delete main_win;
+}
