@@ -23,9 +23,8 @@
 #include <calf/utils.h>
 #if USE_LV2
 #include <lv2.h>
-#include <calf/lv2_event.h>
 #include <calf/lv2_state.h>
-#include <calf/lv2_uri_map.h>
+#include <calf/lv2_urid.h>
 #endif
 #include <getopt.h>
 #include <string.h>
@@ -40,6 +39,9 @@ static struct option long_options[] = {
     {"version", 0, 0, 'v'},
     {"mode", 1, 0, 'm'},
     {"path", 1, 0, 'p'},
+#if USE_LV2
+    {"data-dir", 1, 0, 'd'},
+#endif
     {0,0,0,0},
 };
 
@@ -70,22 +72,23 @@ static void add_port(string &ports, const char *symbol, const char *name, const
     ss << ind << "lv2:name \"" << name << "\" ;\n";
     if (optional)
         ss << ind << "lv2:portProperty lv2:connectionOptional ;\n";
-    if (!strcmp(type, "lv2ev:EventPort")) {
-        ss << ind << "lv2ev:supportsEvent lv2midi:MidiEvent ;\n";
+    if (!strcmp(type, "atom:AtomPort")) {
+        ss << ind << "atom:bufferType atom:Sequence ;\n"
+           << ind << "atom:supports lv2midi:MidiEvent ;" << endl;
     }
-    if (!strcmp(symbol, "in_l")) 
+    if (!strcmp(std::string(symbol, 0, 4).c_str(), "in_l")) 
         ss << ind << "lv2:designation pg:left ;\n"
            << ind << "pg:group :in ;" << endl;
     else
-    if (!strcmp(symbol, "in_r")) 
+    if (!strcmp(std::string(symbol, 0, 4).c_str(), "in_r")) 
         ss << ind << "lv2:designation pg:right ;\n"
            << ind << "pg:group :in ;" << endl;
     else
-    if (!strcmp(symbol, "out_l")) 
+    if (!strcmp(std::string(symbol, 0, 5).c_str(), "out_l")) 
         ss << ind << "lv2:designation pg:left ;\n"
            << ind << "pg:group :out ;" << endl;
     else
-    if (!strcmp(symbol, "out_r")) 
+    if (!strcmp(std::string(symbol, 0, 5).c_str(), "out_r")) 
         ss << ind << "lv2:designation pg:right ;\n"
            << ind << "pg:group :out ;" << endl;
     ss << "    ]";
@@ -145,6 +148,7 @@ static bool add_ctl_port(string &ports, const parameter_properties &pp, int pidx
     else if (type == PF_ENUM)
     {
         ss << ind << "lv2:portProperty lv2:integer ;\n";
+        ss << ind << "lv2:portProperty lv2:enumeration ;\n";
         for (int i = (int)pp.min; i <= (int)pp.max; i++)
             ss << ind << "lv2:scalePoint [ rdfs:label \"" << pp.choices[i - (int)pp.min] << "\"; rdf:value " << i <<" ] ;\n";
     }
@@ -160,12 +164,13 @@ static bool add_ctl_port(string &ports, const parameter_properties &pp, int pidx
     // XXX This value does not seem to match the definition of the property
     //if (pp.step > 1)
     //    ss << ind << "epp:rangeSteps " << pp.step << " ;\n";
-    if (unit > 0 && unit < (sizeof(units) / sizeof(char *)) && units[unit - 1] != NULL)
+    if (((pp.flags & PF_SCALEMASK) != PF_SCALE_GAIN) && unit > 0 && unit < (sizeof(units) / sizeof(char *)) && units[unit - 1] != NULL)
         ss << ind << "ue:unit " << units[unit - 1] << " ;\n";
     
     // for now I assume that the only tempo passed is the tempo the plugin should operate with
     // this may change as more complex plugins are added
-    if (unit == (PF_UNIT_BPM >> 24))
+    
+    if ((pp.flags & PF_SYNC_BPM))
         ss << ind << "lv2:designation <http://lv2plug.in/ns/ext/time#beatsPerMinute> ;\n";
     
     ss << "    ]";
@@ -173,7 +178,7 @@ static bool add_ctl_port(string &ports, const parameter_properties &pp, int pidx
     return true;
 }
 
-void make_ttl(string path_prefix)
+void make_ttl(string path_prefix, const string *data_dir)
 {
     if (path_prefix.empty())
     {
@@ -189,7 +194,7 @@ void make_ttl(string path_prefix)
         "@prefix dct: <http://purl.org/dc/terms/> .\n"
         "@prefix doap: <http://usefulinc.com/ns/doap#> .\n"
         "@prefix uiext: <http://lv2plug.in/ns/extensions/ui#> .\n"
-        "@prefix lv2ev: <http://lv2plug.in/ns/ext/event#> .\n"
+        "@prefix atom: <http://lv2plug.in/ns/ext/atom#> .\n"
         "@prefix lv2midi: <http://lv2plug.in/ns/ext/midi#> .\n"
         "@prefix lv2ctx: <http://lv2plug.in/ns/dev/contexts#> .\n"
         "@prefix strport: <http://lv2plug.in/ns/dev/string-port#> .\n"
@@ -216,7 +221,9 @@ void make_ttl(string path_prefix)
         "Delay", "Modulator", "Phaser", "Chorus", "Filter",
         "Lowpass", "Highpass", "Bandpass", "Comb", "Allpass",
         "Amplifier", "Distortion", "Waveshaper", "Dynamics", "Compressor",
-        "Expander", "Limiter", "Gate", NULL
+        "Expander", "Limiter", "Gate", "EQ", "ParaEQ", "MultiEQ", "Spatial",
+        "Spectral", "Pitch", "Envelope", "Function", "Constant",
+        NULL
     };
     
     for(const char **p = ptypes; *p; p++) {
@@ -233,22 +240,23 @@ void make_ttl(string path_prefix)
     string gtkgui_uri = "<http://calf.sourceforge.net/plugins/gui/gtk2-gui>";
     gui_header = gtkgui_uri + "\n"
         "    a uiext:GtkUI ;\n"
-        "    uiext:binary <calflv2gui.so> ;\n"
-        "    uiext:requiredFeature uiext:makeResident .\n"
+        "    lv2:extensionData uiext:showInterface ;\n"
+        "    lv2:requiredFeature uiext:makeResident ;\n"
+        "    uiext:binary <calflv2gui.so> .\n"
         "\n"
     ;
 #endif
     
-    map<string, string> id_to_label;
+    map<string, pair<string, string> > id_to_info;
     
     for (unsigned int i = 0; i < plugins.size(); i++) {
         const plugin_metadata_iface *pi = plugins[i];
         const ladspa_plugin_info &lpi = pi->get_plugin_info();
-        id_to_label[pi->get_id()] = pi->get_label();
         printf("Generating a .ttl file for plugin '%s'\n", lpi.label);
         fflush(stdout);
         string unquoted_uri = plugin_uri_prefix + string(lpi.label);
         string uri = string("<" + unquoted_uri + ">");
+        id_to_info[pi->get_id()] = make_pair(lpi.label, uri);
         string ttl;
         ttl = "@prefix : <" + unquoted_uri + "#> .\n" + header + gui_header;
         
@@ -303,14 +311,16 @@ void make_ttl(string path_prefix)
         if (pi->get_midi())
         {
             if (pi->requires_midi()) {
-                ttl += "    lv2:requiredFeature <" LV2_URI_MAP_URI "> ;\n";                
+                ttl += "    lv2:requiredFeature <" LV2_URID_MAP_URI "> ;\n";
             }
             else {
-                ttl += "    lv2:optionalFeature <" LV2_URI_MAP_URI "> ;\n";                
+                ttl += "    lv2:optionalFeature <" LV2_URID_MAP_URI "> ;\n";
             }
         }
         
-        if (pi->get_configure_vars())
+        vector<string> configure_keys;
+        pi->get_configure_vars(configure_keys);
+        if (!configure_keys.empty())
         {
             ttl += "    lv2:extensionData <" LV2_STATE__interface "> ;\n";
         }
@@ -324,23 +334,25 @@ void make_ttl(string path_prefix)
 
         string ports = "";
         int pn = 0;
-        const char *in_names[] = { "in_l", "in_r", "sidechain" };
-        const char *out_names[] = { "out_l", "out_r" };
+        const char *in_symbols[] = { "in_l", "in_r", "sidechain", "sidechain2" };
+        const char *in_names[] = { "In L", "In R", "Sidechain", "Sidechain 2" };
+        const char *out_symbols[] = { "out_l", "out_r", "out_l_2", "out_r_2", "out_l_3", "out_r_3", "out_l_4", "out_r_4" };
+        const char *out_names[] = { "Out L", "Out R", "Out L 2", "Out R 2", "Out L 3", "Out R 3", "Out L 4", "Out R 4" };
         for (int i = 0; i < pi->get_input_count(); i++)
-            if(i <= pi->get_input_count() - pi->get_inputs_optional() - 1)
-                add_port(ports, in_names[i], in_names[i], "Input", pn++);
+            if(i <= pi->get_input_count() - pi->get_inputs_optional() - 1 && !(i == 1 && pi->get_simulate_stereo_input()))
+                add_port(ports, in_symbols[i], in_names[i], "Input", pn++);
             else
-                add_port(ports, in_names[i], in_names[i], "Input", pn++, "lv2:AudioPort", true);
+                add_port(ports, in_symbols[i], in_names[i], "Input", pn++, "lv2:AudioPort", true);
         for (int i = 0; i < pi->get_output_count(); i++)
             if(i <= pi->get_output_count() - pi->get_outputs_optional() - 1)
-                add_port(ports, out_names[i], out_names[i], "Output", pn++);
+                add_port(ports, out_symbols[i], out_names[i], "Output", pn++);
             else
-                add_port(ports, out_names[i], out_names[i], "Output", pn++, "lv2:AudioPort", true);
+                add_port(ports, out_symbols[i], out_names[i], "Output", pn++, "lv2:AudioPort", true);
         for (int i = 0; i < pi->get_param_count(); i++)
             if (add_ctl_port(ports, *pi->get_param_props(i), pn, pi, i))
                 pn++;
         if (pi->get_midi()) {
-            add_port(ports, "event_in", "Event", "Input", pn++, "lv2ev:EventPort", true);
+            add_port(ports, "midi_in", "MIDI", "Input", pn++, "atom:AtomPort", true);
         }
         if (!ports.empty())
             ttl += "    lv2:port " + ports + "\n";
@@ -366,7 +378,7 @@ void make_ttl(string path_prefix)
     // Prefixes for the manifest TTL
     string ttl = presets_ttl_head;
     
-    calf_plugins::get_builtin_presets().load_defaults(true);
+    calf_plugins::get_builtin_presets().load_defaults(true, data_dir);
     calf_plugins::preset_vector &factory_presets = calf_plugins::get_builtin_presets().presets;
 
     ttl += "\n";
@@ -377,24 +389,24 @@ void make_ttl(string path_prefix)
     for (unsigned int i = 0; i < factory_presets.size(); i++)
     {
         plugin_preset &pr = factory_presets[i];
-        map<string, string>::iterator ilm = id_to_label.find(pr.plugin);
-        if (ilm == id_to_label.end())
+        map<string, pair<string, string> >::iterator ilm = id_to_info.find(pr.plugin);
+        if (ilm == id_to_info.end())
             continue;
         
         string presets_ttl;
         // if this is the first preset, add a header
-        if (!preset_data.count(ilm->second))
+        if (!preset_data.count(ilm->second.first))
             presets_ttl = presets_ttl_head;
         
         string uri = "<http://calf.sourceforge.net/factory_presets#"
             + pr.plugin + "_" + pr.get_safe_name()
             + ">";
-        ttl += string("<" + plugin_uri_prefix + ilm->second + "> lv2p:hasPreset\n    " + uri + " .\n");
+        ttl += ilm->second.second + " lv2p:hasPreset\n    " + uri + " .\n";
         
         presets_ttl += uri + 
             " a lv2p:Preset ;\n"
             "    rdfs:label \"" + pr.name + "\" ;\n"
-            "    lv2:appliesTo <" + plugin_uri_prefix + ilm->second + "> ;\n"
+            "    lv2:appliesTo " + ilm->second.second + " ;\n"
             "    lv2:port \n"
         ;
         
@@ -408,9 +420,9 @@ void make_ttl(string path_prefix)
         }
         presets_ttl += ".\n\n";
         
-        preset_data[ilm->second] += presets_ttl;
+        preset_data[ilm->second.first] += presets_ttl;
     }
-    for (map<string, string>::iterator i = preset_data.begin(); i != preset_data.end(); i++)
+    for (map<string, string>::iterator i = preset_data.begin(); i != preset_data.end(); ++i)
     {
         FILE *f = open_and_check(path_prefix + "presets-" + i->first + ".ttl");
         fprintf(f, "%s\n", i->second.c_str());
@@ -424,9 +436,9 @@ void make_ttl(string path_prefix)
         string label = plugins[i]->get_plugin_info().label;
         ttl += string("<" + plugin_uri_prefix) 
             + string(plugins[i]->get_plugin_info().label)
-	        + "> a lv2:Plugin ;\n    dct:replaces <urn:ladspa:"
-	        + i2s(plugins[i]->get_plugin_info().unique_id) + "> ;\n    "
-	        + "lv2:binary <calf.so> ; rdfs:seeAlso <" + label + ".ttl> ";
+            + "> a lv2:Plugin ;\n    dct:replaces <urn:ladspa:"
+            + i2s(plugins[i]->get_plugin_info().unique_id) + "> ;\n    "
+            + "lv2:binary <calf.so> ; rdfs:seeAlso <" + label + ".ttl> ";
         if (preset_data.count(label))
             ttl += ", <presets-" + label + ".ttl>";
         ttl += ".\n";
@@ -571,9 +583,16 @@ int main(int argc, char *argv[])
 {
     string mode = "rdf";
     string path_prefix;
+#if USE_LV2
+    string pkglibdir_path;
+#endif
     while(1) {
         int option_index;
-        int c = getopt_long(argc, argv, "hvm:p:", long_options, &option_index);
+        int c = getopt_long(argc, argv, "hvm:p:"
+#if USE_LV2
+        "d:"
+#endif
+            , long_options, &option_index);
         if (c == -1)
             break;
         switch(c) {
@@ -591,6 +610,18 @@ int main(int argc, char *argv[])
                     return 1;
                 }
                 break;
+#if USE_LV2
+            case 'd':
+                pkglibdir_path = optarg;
+                if (pkglibdir_path.empty())
+                {
+                    fprintf(stderr, "calfmakerdf: Data directory must not be empty\n");
+                    exit(1);
+                }
+                if (pkglibdir_path[path_prefix.length() - 1] != '/')
+                    pkglibdir_path += '/';
+                break;
+#endif
             case 'p':
                 path_prefix = optarg;
                 if (path_prefix.empty())
@@ -603,14 +634,12 @@ int main(int argc, char *argv[])
                 break;
         }
     }
-    if (false)
-    {
-    }
 #if USE_LV2
-    else if (mode == "ttl")
-        make_ttl(path_prefix);
+    if (mode == "ttl")
+        make_ttl(path_prefix, !pkglibdir_path.empty() ? &pkglibdir_path : NULL);
+    else
 #endif
-    else if (mode == "gui")
+    if (mode == "gui")
         make_gui(path_prefix);
     else
     {
