@@ -28,7 +28,9 @@ using namespace calf_plugins;
 
 #define SET_IF_CONNECTED(name) if (params[AM::param_##name] != NULL) *params[AM::param_##name] = name;
 
-///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+/**********************************************************************
+ * FLANGER by Krzysztof Foltman
+**********************************************************************/
 
 void flanger_audio_module::activate() {
     left.reset();
@@ -49,23 +51,6 @@ void flanger_audio_module::deactivate() {
     is_active = false;
 }
 
-bool flanger_audio_module::get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const
-{
-    if (!is_active)
-        return false;
-    if (index == par_delay && subindex < 2) 
-    {
-        set_channel_color(context, subindex);
-        return ::get_graph(*this, subindex, data, points);
-    }
-    return false;
-}
-
-float flanger_audio_module::freq_gain(int subindex, float freq, float srate) const
-{
-    return (subindex ? right : left).freq_gain(freq, srate);                
-}
-
 void flanger_audio_module::params_changed()
 {
     float dry = *params[par_dryamount];
@@ -102,8 +87,34 @@ void flanger_audio_module::params_reset()
         clear_reset = false;
     }
 }
+bool flanger_audio_module::get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
+{
+    if (!is_active or !phase or subindex >= 2)
+        return false;
+    set_channel_color(context, subindex);
+    return ::get_graph(*this, subindex, data, points, 32, 0);
+}
+bool flanger_audio_module::get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
+{
+    if (!is_active or phase)
+        return false;
+    return get_freq_gridline(subindex, pos, vertical, legend, context, true, 32, 0);
+}
+bool flanger_audio_module::get_layers(int index, int generation, unsigned int &layers) const
+{
+    layers = LG_REALTIME_GRAPH | (generation ? 0 : LG_CACHE_GRID);
+    return true;
+}
+float flanger_audio_module::freq_gain(int subindex, float freq) const
+{
+    return (subindex ? right : left).freq_gain(freq, srate);                
+}
+
+
+/**********************************************************************
+ * PHASER by Krzysztof Foltman
+**********************************************************************/
 
-///////////////////////////////////////////////////////////////////////////////////////////////
 
 phaser_audio_module::phaser_audio_module()
 : left(MaxStages, x1vals[0], y1vals[0])
@@ -134,28 +145,6 @@ void phaser_audio_module::deactivate()
     is_active = false;
 }
 
-bool phaser_audio_module::get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const
-{
-    if (!is_active)
-        return false;
-    if (subindex < 2) 
-    {
-        set_channel_color(context, subindex);
-        return ::get_graph(*this, subindex, data, points);
-    }
-    return false;
-}
-
-float phaser_audio_module::freq_gain(int subindex, float freq, float srate) const
-{
-    return (subindex ? right : left).freq_gain(freq, srate);                
-}
-
-bool phaser_audio_module::get_gridline(int index, int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
-{
-    return get_freq_gridline(subindex, pos, vertical, legend, context);
-}
-
 void phaser_audio_module::params_changed()
 {
     float dry = *params[par_dryamount];
@@ -194,8 +183,34 @@ void phaser_audio_module::params_reset()
         clear_reset = false;
     }
 }
+bool phaser_audio_module::get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
+{
+    if (!is_active or phase)
+        return false;
+    return get_freq_gridline(subindex, pos, vertical, legend, context, true, 32, 0);
+}
+bool phaser_audio_module::get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
+{
+    if (!is_active or subindex >= 2 or !phase)
+        return false;
+    set_channel_color(context, subindex);
+    return ::get_graph(*this, subindex, data, points, 32, 0);
+}
+bool phaser_audio_module::get_layers(int index, int generation, unsigned int &layers) const
+{
+    layers = LG_REALTIME_GRAPH | (generation ? 0 : LG_CACHE_GRID);
+    return true;
+}
 
-///////////////////////////////////////////////////////////////////////////////////////////////
+float phaser_audio_module::freq_gain(int subindex, float freq) const
+{
+    return (subindex ? right : left).freq_gain(freq, srate);                
+}
+
+
+/**********************************************************************
+ * ROTARY SPEAKER by Krzysztof Foltman
+**********************************************************************/
 
 rotary_speaker_audio_module::rotary_speaker_audio_module()
 {
@@ -338,10 +353,10 @@ uint32_t rotary_speaker_audio_module::process(uint32_t offset, uint32_t nsamples
     float mix = 0.5 * (1.0 - *params[par_micdistance]);
     float mix2 = *params[par_reflection];
     float mix3 = mix2 * mix2;
-    float am_depth = *params[par_am_depth];
+    double am_depth = *params[par_am_depth];
     for (unsigned int i = 0; i < nsamples; i++) {
         float in_l = ins[0][i + offset], in_r = ins[1][i + offset];
-        float in_mono = atan(0.5f * (in_l + in_r));
+        double in_mono = atan(0.5f * (in_l + in_r));
         
         int xl = pseudo_sine_scl(phase_l), yl = pseudo_sine_scl(phase_l + 0x40000000);
         int xh = pseudo_sine_scl(phase_h), yh = pseudo_sine_scl(phase_h + 0x40000000);
@@ -352,11 +367,11 @@ uint32_t rotary_speaker_audio_module::process(uint32_t offset, uint32_t nsamples
         // float out_hi_r = in_mono + delay.get_interp_1616(shift + md * 65536 - md * yh) - delay.get_interp_1616(shift + pdelta + md * xh) + delay.get_interp_1616(shift + pdelta + pdelta + md * yh);
         float fm_hi_l = delay.get_interp_1616(shift + md * xh) - mix2 * delay.get_interp_1616(shift + md * 65536 + pdelta - md * yh) + mix3 * delay.get_interp_1616(shift + md * 65536 + pdelta + pdelta - md * xh);
         float fm_hi_r = delay.get_interp_1616(shift + md * 65536 - md * yh) - mix2 * delay.get_interp_1616(shift + pdelta + md * xh) + mix3 * delay.get_interp_1616(shift + pdelta + pdelta + md * yh);
-        float out_hi_l = lerp(in_mono, damper1l.process(fm_hi_l), lerp(0.5, xh * 1.0 / 65536.0, am_depth));
-        float out_hi_r = lerp(in_mono, damper1r.process(fm_hi_r), lerp(0.5, yh * 1.0 / 65536.0, am_depth));
+        float out_hi_l = lerp(in_mono, (double)damper1l.process(fm_hi_l), lerp(0.5, xh * 1.0 / 65536.0, am_depth));
+        float out_hi_r = lerp(in_mono, (double)damper1r.process(fm_hi_r), lerp(0.5, yh * 1.0 / 65536.0, am_depth));
 
-        float out_lo_l = lerp(in_mono, delay.get_interp_1616(shift + (md * xl >> 2)), lerp(0.5, yl * 1.0 / 65536.0, am_depth)); // + delay.get_interp_1616(shift + md * 65536 + pdelta - md * yl);
-        float out_lo_r = lerp(in_mono, delay.get_interp_1616(shift + (md * yl >> 2)), lerp(0.5, xl * 1.0 / 65536.0, am_depth)); // + delay.get_interp_1616(shift + md * 65536 + pdelta - md * yl);
+        float out_lo_l = lerp(in_mono, (double)delay.get_interp_1616(shift + (md * xl >> 2)), lerp(0.5, yl * 1.0 / 65536.0, am_depth)); // + delay.get_interp_1616(shift + md * 65536 + pdelta - md * yl);
+        float out_lo_r = lerp(in_mono, (double)delay.get_interp_1616(shift + (md * yl >> 2)), lerp(0.5, xl * 1.0 / 65536.0, am_depth)); // + delay.get_interp_1616(shift + md * 65536 + pdelta - md * yl);
         
         out_hi_l = crossover2l.process(out_hi_l); // sanitize(out_hi_l);
         out_hi_r = crossover2r.process(out_hi_r); // sanitize(out_hi_r);
@@ -400,12 +415,17 @@ uint32_t rotary_speaker_audio_module::process(uint32_t offset, uint32_t nsamples
     return outputs_mask;
 }
 
-///////////////////////////////////////////////////////////////////////////////////////////////
+/**********************************************************************
+ * MULTI CHORUS by Krzysztof Foltman
+**********************************************************************/
 
 multichorus_audio_module::multichorus_audio_module()
 {
     is_active = false;
     last_r_phase = -1;
+    freq_old = freq2_old = q_old = -1;
+    redraw_graph = true;
+    redraw_sine = true;
 }
 
 void multichorus_audio_module::activate()
@@ -426,82 +446,6 @@ void multichorus_audio_module::set_sample_rate(uint32_t sr) {
     right.setup(sr);
 }
 
-bool multichorus_audio_module::get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const
-{
-    if (!is_active)
-        return false;
-    int nvoices = (int)*params[par_voices];
-    if (index == par_delay && subindex < 3) 
-    {
-        if (subindex < 2)
-            set_channel_color(context, subindex);
-        else {
-            context->set_source_rgba(0.35, 0.4, 0.2);
-            context->set_line_width(1.0);
-        }
-        return ::get_graph(*this, subindex, data, points);
-    }
-    if (index == par_rate && subindex < nvoices) {
-        const sine_multi_lfo<float, 8> &lfo = left.lfo;
-        for (int i = 0; i < points; i++) {
-            float phase = i * 2 * M_PI / points;
-            // original -65536 to 65535 value
-            float orig = subindex * lfo.voice_offset + ((lfo.voice_depth >> (30-13)) * 65536.0 * (0.95 * sin(phase) + 1)/ 8192.0) - 65536;
-            // scale to -1..1
-            data[i] = orig / 65536.0;
-        }
-        return true;
-    }
-    return false;
-}
-
-bool multichorus_audio_module::get_dot(int index, int subindex, float &x, float &y, int &size, cairo_iface *context) const
-{
-    int voice = subindex >> 1;
-    int nvoices = (int)*params[par_voices];
-    if ((index != par_rate && index != par_depth) || voice >= nvoices)
-        return false;
-
-    float unit = (1 - *params[par_overlap]);
-    float scw = 1 + unit * (nvoices - 1);
-    set_channel_color(context, subindex);
-    const sine_multi_lfo<float, 8> &lfo = (subindex & 1 ? right : left).lfo;
-    if (index == par_rate)
-    {
-        x = (double)(lfo.phase + lfo.vphase * voice) / 4096.0;
-        y = 0.95 * sin(x * 2 * M_PI);
-        y = (voice * unit + (y + 1) / 2) / scw * 2 - 1;
-    }
-    else
-    {
-        double ph = (double)(lfo.phase + lfo.vphase * voice) / 4096.0;
-        x = 0.5 + 0.5 * sin(ph * 2 * M_PI);
-        y = subindex & 1 ? -0.75 : 0.75;
-        x = (voice * unit + x) / scw;
-    }
-    return true;
-}
-
-bool multichorus_audio_module::get_gridline(int index, int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
-{
-    if (index == par_rate && !subindex)
-    {
-        pos = 0;
-        vertical = false;
-        return true;
-    }
-    if (index == par_delay)
-        return get_freq_gridline(subindex, pos, vertical, legend, context);
-    return false;
-}
-
-float multichorus_audio_module::freq_gain(int subindex, float freq, float srate) const
-{
-    if (subindex == 2)
-        return *params[par_amount] * left.post.freq_gain(freq, srate);
-    return (subindex ? right : left).freq_gain(freq, srate);                
-}
-
 void multichorus_audio_module::params_changed()
 {
     // delicious copy-pasta from flanger module - it'd be better to keep it common or something
@@ -527,10 +471,17 @@ void multichorus_audio_module::params_changed()
         right.lfo.phase += chorus_phase(r_phase * 4096);
         last_r_phase = r_phase;
     }
-    left.post.f1.set_bp_rbj(*params[par_freq], *params[par_q], srate);
-    left.post.f2.set_bp_rbj(*params[par_freq2], *params[par_q], srate);
-    right.post.f1.copy_coeffs(left.post.f1);
-    right.post.f2.copy_coeffs(left.post.f2);
+    if (*params[par_freq] != freq_old or *params[par_freq2] != freq2_old or *params[par_q] != q_old) {
+        left.post.f1.set_bp_rbj(*params[par_freq], *params[par_q], srate);
+        left.post.f2.set_bp_rbj(*params[par_freq2], *params[par_q], srate);
+        right.post.f1.copy_coeffs(left.post.f1);
+        right.post.f2.copy_coeffs(left.post.f2);
+        freq_old = *params[par_freq];
+        freq2_old = *params[par_freq2];
+        q_old = *params[par_q];
+        redraw_graph = true;
+    }
+    redraw_sine = true;
 }
 
 uint32_t multichorus_audio_module::process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask)
@@ -540,25 +491,112 @@ uint32_t multichorus_audio_module::process(uint32_t offset, uint32_t numsamples,
     return outputs_mask; // XXXKF allow some delay after input going blank
 }
 
-/// Pulsator by Markus Schmidt
-///
-/// This module provides a couple
-/// of different LFO's for modulating the level of a signal.
-///////////////////////////////////////////////////////////////////////////////////////////////
+bool multichorus_audio_module::get_layers(int index, int generation, unsigned int &layers) const
+{
+    layers = 0;
+    // frequency response
+    if (index == par_delay) {
+        layers = (generation ? 0 : LG_CACHE_GRID) | (redraw_graph ? LG_CACHE_GRAPH : 0) | LG_REALTIME_GRAPH;
+    }
+    // sine display
+    if (index == par_rate) {
+        layers = LG_REALTIME_DOT | (redraw_sine ? LG_CACHE_GRAPH : 0);
+    }
+    // dot display
+    if (index == par_depth)
+        layers = LG_REALTIME_DOT;
+    return true;
+}
+
+bool multichorus_audio_module::get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
+{
+    if (!is_active)
+        return false;
+    
+    // the filter graph (cached) in frequency response
+    if (index == par_delay and subindex == 2 and !phase) {
+        context->set_source_rgba(0.15, 0.2, 0.0, 0.8);
+        redraw_graph = false;
+        return ::get_graph(*this, subindex, data, points, 64, 0.5);
+    }
+    // the realtime graphs in frequency response
+    if (index == par_delay and subindex < 2 and phase) {
+        set_channel_color(context, subindex);
+        context->set_line_width(1);
+        return ::get_graph(*this, subindex, data, points, 64, 0.5);
+    }
+    
+    // the sine curves in modulation display
+    if (index == par_rate && subindex < (int)*params[par_voices] and !phase) {
+        const sine_multi_lfo<float, 8> &lfo = left.lfo;
+        for (int i = 0; i < points; i++) {
+            float _phase = i * 2 * M_PI / points;
+            // original -65536 to 65535 value
+            float orig = subindex * lfo.voice_offset + ((lfo.voice_depth >> (30-13)) * 65536.0 * (0.95 * sin(_phase) + 1)/ 8192.0) - 65536;
+            // scale to -1..1
+            data[i] = orig / 65536.0;
+        }
+        return true;
+    }
+    redraw_sine = false;
+    return false;
+}
+
+bool multichorus_audio_module::get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const
+{
+    int voice = subindex >> 1;
+    int nvoices = (int)*params[par_voices];
+    if (!is_active
+     or !phase
+     or (index != par_rate and index != par_depth)
+     or voice >= nvoices)
+        return false;
+
+    float unit = (1 - *params[par_overlap]);
+    float scw = 1 + unit * (nvoices - 1);
+    const sine_multi_lfo<float, 8> &lfo = (subindex & 1 ? right : left).lfo;
+    
+    // the display with the sine curves
+    if (index == par_rate) {
+        x = (double)(lfo.phase + lfo.vphase * voice) / 4096.0;
+        y = 0.95 * sin(x * 2 * M_PI);
+        y = (voice * unit + (y + 1) / 2) / scw * 2 - 1;
+    }
+    // the tiny dot display
+    if (index == par_depth) {
+        double ph = (double)(lfo.phase + lfo.vphase * voice) / 4096.0;
+        x = 0.5 + 0.5 * sin(ph * 2 * M_PI);
+        y = subindex & 1 ? -0.5 : 0.5;
+        x = (voice * unit + x) / scw;
+    }
+    return true;
+}
+
+bool multichorus_audio_module::get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
+{
+    if (index == par_delay and !phase and is_active)
+        return get_freq_gridline(subindex, pos, vertical, legend, context, true, 64, 0.5);
+    return false;
+}
+
+float multichorus_audio_module::freq_gain(int subindex, float freq) const
+{
+    if (subindex == 2)
+        return *params[par_amount] * left.post.freq_gain(freq, srate);
+    return (subindex ? right : left).freq_gain(freq, srate);                
+}
+
+/**********************************************************************
+ * PULSATOR by Markus Schmidt
+**********************************************************************/
 
 pulsator_audio_module::pulsator_audio_module()
 {
     is_active = false;
     srate = 0;
-//    last_generation = 0;
-    clip_inL    = 0.f;
-    clip_inR    = 0.f;
-    clip_outL   = 0.f;
-    clip_outR   = 0.f;
-    meter_inL  = 0.f;
-    meter_inR  = 0.f;
-    meter_outL = 0.f;
-    meter_outR = 0.f;
+    mode_old    = -1;
+    amount_old  = -1;
+    offset_old  = -1;
 }
 
 void pulsator_audio_module::activate()
@@ -585,50 +623,45 @@ void pulsator_audio_module::params_changed()
         lfoL.set_phase(0.f);
         lfoR.set_phase(0.f);
     }
+    if (*params[param_mode]   != mode_old
+     or *params[param_amount] != amount_old
+     or *params[param_offset] != offset_old
+     or clear_reset) {
+        mode_old     = *params[param_mode];
+        amount_old   = *params[param_amount];
+        offset_old   = *params[param_offset];
+        redraw_graph = true;
+    }
 }
 
 void pulsator_audio_module::set_sample_rate(uint32_t sr)
 {
     srate = sr;
+    int meter[] = {param_meter_inL, param_meter_inR,  param_meter_outL, param_meter_outR};
+    int clip[] = {param_clip_inL, param_clip_inR, param_clip_outL, param_clip_outR};
+    meters.init(params, meter, clip, 4, sr);
 }
 
 uint32_t pulsator_audio_module::process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask)
 {
-    bool bypass = *params[param_bypass] > 0.5f;
+    bool bypassed = bypass.update(*params[param_bypass] > 0.5f, numsamples);
     uint32_t samples = numsamples + offset;
-    if(bypass) {
+    if(bypassed) {
         // everything bypassed
         while(offset < samples) {
             outs[0][offset] = ins[0][offset];
             outs[1][offset] = ins[1][offset];
             ++offset;
         }
-        // displays, too
-        clip_inL    = 0.f;
-        clip_inR    = 0.f;
-        clip_outL   = 0.f;
-        clip_outR   = 0.f;
-        meter_inL  = 0.f;
-        meter_inR  = 0.f;
-        meter_outL = 0.f;
-        meter_outR = 0.f;
-        
         // LFO's should go on
         lfoL.advance(numsamples);
         lfoR.advance(numsamples);
         
+        float values[] = {0, 0, 0, 0};
+        meters.process(values);
     } else {
-        
-        clip_inL    -= std::min(clip_inL,  numsamples);
-        clip_inR    -= std::min(clip_inR,  numsamples);
-        clip_outL   -= std::min(clip_outL, numsamples);
-        clip_outR   -= std::min(clip_outR, numsamples);
-        meter_inL = 0.f;
-        meter_inR = 0.f;
-        meter_outL = 0.f;
-        meter_outR = 0.f;
-        
         // process
+        uint32_t orig_offset = offset;
         while(offset < samples) {
             // cycle through samples
             float outL = 0.f;
@@ -659,96 +692,257 @@ uint32_t pulsator_audio_module::process(uint32_t offset, uint32_t numsamples, ui
             outs[0][offset] = outL;
             outs[1][offset] = outR;
             
-            // clip LED's
-            if(inL > 1.f) {
-                clip_inL  = srate >> 3;
-            }
-            if(inR > 1.f) {
-                clip_inR  = srate >> 3;
-            }
-            if(outL > 1.f) {
-                clip_outL = srate >> 3;
-            }
-            if(outR > 1.f) {
-                clip_outR = srate >> 3;
-            }
-            // set up in / out meters
-            if(inL > meter_inL) {
-                meter_inL = inL;
-            }
-            if(inR > meter_inR) {
-                meter_inR = inR;
-            }
-            if(outL > meter_outL) {
-                meter_outL = outL;
-            }
-            if(outR > meter_outR) {
-                meter_outR = outR;
-            }
-            
             // next sample
             ++offset;
             
             // advance lfo's
             lfoL.advance(1);
             lfoR.advance(1);
+            
+            float values[] = {inL, inR, outL, outR};
+            meters.process(values);
+
         } // cycle trough samples
+        bypass.crossfade(ins, outs, 2, orig_offset, numsamples);
     }
-    // draw meters
-    SET_IF_CONNECTED(clip_inL)
-    SET_IF_CONNECTED(clip_inR)
-    SET_IF_CONNECTED(clip_outL)
-    SET_IF_CONNECTED(clip_outR)
-    SET_IF_CONNECTED(meter_inL)
-    SET_IF_CONNECTED(meter_inR)
-    SET_IF_CONNECTED(meter_outL)
-    SET_IF_CONNECTED(meter_outR)
-    // whatever has to be returned x)
+    meters.fall(numsamples);
     return outputs_mask;
 }
 
-bool pulsator_audio_module::get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const
+bool pulsator_audio_module::get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
 {
-    if (!is_active) {
+    if (!is_active or phase or subindex > 1) {
+        redraw_graph = false;
         return false;
-    } else if(index == param_freq) {
-        if(subindex == 0) {
-            context->set_source_rgba(0.35, 0.4, 0.2, 1);
-            return lfoL.get_graph(data, points, context, mode);
-        }
-        if(subindex == 1) {
-            context->set_source_rgba(0.35, 0.4, 0.2, 0.5);
-            return lfoR.get_graph(data, points, context, mode);
-        }
     }
-    return false;
+    set_channel_color(context, subindex);
+    return (subindex ? lfoR : lfoL).get_graph(data, points, context, mode);
 }
 
-bool pulsator_audio_module::get_dot(int index, int subindex, float &x, float &y, int &size, cairo_iface *context) const
+bool pulsator_audio_module::get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const
 {
-    if (!is_active) {
+    if (!is_active or !phase or subindex > 1)
         return false;
-    } else if(index == param_freq) {
-        if(subindex == 0) {
-            context->set_source_rgba(0.35, 0.4, 0.2, 1);
-            return lfoL.get_dot(x, y, size, context);
-        }
-        if(subindex == 1) {
-            context->set_source_rgba(0.35, 0.4, 0.2, 0.5);
-            return lfoR.get_dot(x, y, size, context);
-        }
-        return false;
-    }
-    return false;
+    set_channel_color(context, subindex);
+    return (subindex ? lfoR : lfoL).get_dot(x, y, size, context);
 }
 
-bool pulsator_audio_module::get_gridline(int index, int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
+bool pulsator_audio_module::get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
 {
-   if (index == param_freq && !subindex)
-    {
-        pos = 0;
-        vertical = false;
-        return true;
+    if (phase or subindex)
+        return false;
+    pos = 0;
+    vertical = false;
+    return true;
+}
+bool pulsator_audio_module::get_layers(int index, int generation, unsigned int &layers) const
+{
+    layers = LG_REALTIME_DOT | (generation ? 0 : LG_CACHE_GRID) | ((redraw_graph || !generation) ? LG_CACHE_GRAPH : 0);
+    return true;
+}
+
+
+/**********************************************************************
+ * RING MODULATOR by Markus Schmidt
+**********************************************************************/
+
+ringmodulator_audio_module::ringmodulator_audio_module()
+{
+    is_active = false;
+    srate = 0;
+}
+
+void ringmodulator_audio_module::activate()
+{
+    is_active = true;
+    lfo1.activate();
+    lfo2.activate();
+    modL.activate();
+    modR.activate();
+    params_changed();
+}
+void ringmodulator_audio_module::deactivate()
+{
+    is_active = false;
+    lfo1.deactivate();
+    lfo2.deactivate();
+    modL.deactivate();
+    modR.deactivate();
+}
+
+void ringmodulator_audio_module::params_changed()
+{
+    lfo1.set_params(*params[param_lfo1_freq], *params[param_lfo1_mode], 0, srate, 1);
+    lfo2.set_params(*params[param_lfo2_freq], *params[param_lfo2_mode], 0, srate, 1);
+    modL.set_params(*params[param_mod_freq] * pow(pow(2, 1.0 / 1200.0), *params[param_mod_detune] / 2),
+                    *params[param_mod_mode], 0, srate, 1);
+    modR.set_params(*params[param_mod_freq] * pow(pow(2, 1.0 / 1200.0), *params[param_mod_detune] / -2),
+                    *params[param_mod_mode], *params[param_mod_phase], srate, 1);
+    
+    clear_reset = false;
+    
+    if (*params[param_lfo1_reset] >= 0.5) {
+        clear_reset = true;
+        lfo1.set_phase(0.f);
     }
-    return false;
+    if (*params[param_lfo2_reset] >= 0.5) {
+        clear_reset = true;
+        lfo2.set_phase(0.f);
+    }
+    
+    //redraw_graph = true;
+}
+
+void ringmodulator_audio_module::set_sample_rate(uint32_t sr)
+{
+    srate = sr;
+    int meter[] = {param_meter_inL, param_meter_inR,  param_meter_outL, param_meter_outR};
+    int clip[] = {param_clip_inL, param_clip_inR, param_clip_outL, param_clip_outR};
+    meters.init(params, meter, clip, 4, sr);
+}
+
+uint32_t ringmodulator_audio_module::process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask)
+{
+    bool bypassed = bypass.update(*params[param_bypass] > 0.5f, numsamples);
+    uint32_t samples = numsamples + offset;
+    float led1 = 0;
+    float led2 = 0;
+    if(bypassed) {
+        // everything bypassed
+        while(offset < samples) {
+            outs[0][offset] = ins[0][offset];
+            outs[1][offset] = ins[1][offset];
+            ++offset;
+        }
+        // LFO's should go on
+        lfo1.advance(numsamples);
+        lfo1.advance(numsamples);
+        modL.advance(numsamples);
+        modR.advance(numsamples);
+        
+        float values[] = {0, 0, 0, 0};
+        meters.process(values);
+    } else {
+        // process
+        uint32_t orig_offset = offset;
+        while(offset < samples) {
+            // cycle through samples
+            
+            // set oscillators
+            // mod frequency
+            float freq = 0;
+            if (*params[param_lfo1_mod_freq_active] > 0.5) {
+                freq = (*params[param_lfo1_mod_freq_hi]
+                      - *params[param_lfo1_mod_freq_lo])
+                        * ((lfo1.get_value() + 1) / 2.)
+                      + *params[param_lfo1_mod_freq_lo];
+                modL.set_freq(freq);
+                modR.set_freq(freq);
+            }
+            // mod detune
+            if (*params[param_lfo1_mod_detune_active] > 0.5) {
+                float detune = (*params[param_lfo1_mod_detune_hi]
+                              - *params[param_lfo1_mod_detune_lo])
+                              * ((lfo1.get_value() + 1) / 2.)
+                              + *params[param_lfo1_mod_detune_lo];
+                modL.set_freq((freq ? freq : *params[param_mod_freq]) * pow(pow(2, 1.0 / 1200.0), detune / 2));
+                modR.set_freq((freq ? freq : *params[param_mod_freq]) * pow(pow(2, 1.0 / 1200.0), detune / -2));
+            }
+            // lfo1 frequency
+            if (*params[param_lfo2_lfo1_freq_active] > 0.5) {
+                lfo1.set_freq((*params[param_lfo2_lfo1_freq_hi]
+                             - *params[param_lfo2_lfo1_freq_lo])
+                             * ((lfo2.get_value() + 1) / 2.)
+                             + *params[param_lfo2_lfo1_freq_lo]);
+            }
+            // mod amount
+            float mod_amount = *params[param_mod_amount];
+            if (*params[param_lfo2_mod_amount_active] > 0.5) {
+                mod_amount = (*params[param_lfo2_mod_amount_hi]
+                            - *params[param_lfo2_mod_amount_lo])
+                            * ((lfo2.get_value() + 1) / 2.)
+                            + *params[param_lfo2_mod_amount_lo];
+            }
+            
+            float outL = 0.f;
+            float outR = 0.f;
+            float inL = ins[0][offset];
+            float inR = ins[1][offset];
+            
+            // in level
+            inR *= *params[param_level_in];
+            inL *= *params[param_level_in];
+            
+            // modulator
+            float modulL = modL.get_value() * mod_amount;
+            float modulR = modR.get_value() * mod_amount;
+            
+            float procL = inL * modulL;
+            float procR = inR * modulR;
+            
+            outL = *params[param_mod_listen] > 0.5 ? modulL : procL + inL * (1 - mod_amount);
+            outR = *params[param_mod_listen] > 0.5 ? modulR : procR + inR * (1 - mod_amount);
+            
+            outL *=  *params[param_level_out];
+            outR *=  *params[param_level_out];
+            
+            // send to output
+            outs[0][offset] = outL;
+            outs[1][offset] = outR;
+            
+            // next sample
+            ++offset;
+            
+            // leds
+            led1 = std::max(led1, lfo1.get_value() / 2 + 0.5f);
+            led2 = std::max(led2, lfo2.get_value() / 2 + 0.5f);
+            
+            // advance lfo's
+            lfo1.advance(1);
+            lfo2.advance(1);
+            modL.advance(1);
+            modR.advance(1);
+            
+            float values[] = {inL, inR, outL, outR};
+            meters.process(values);
+
+        } // cycle trough samples
+        bypass.crossfade(ins, outs, 2, orig_offset, numsamples);
+    }
+    *params[param_lfo1_activity] = led1;
+    *params[param_lfo2_activity] = led2;
+    meters.fall(numsamples);
+    return outputs_mask;
+}
+
+bool ringmodulator_audio_module::get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
+{
+    if (!is_active or phase or subindex > 1) {
+        redraw_graph = false;
+        return false;
+    }
+    set_channel_color(context, subindex);
+    return (subindex ? lfo2 : lfo1).get_graph(data, points, context, mode);
+}
+
+bool ringmodulator_audio_module::get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const
+{
+    if (!is_active or !phase or subindex > 1)
+        return false;
+    set_channel_color(context, subindex);
+    return (subindex ? lfo2 : lfo1).get_dot(x, y, size, context);
+}
+
+bool ringmodulator_audio_module::get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
+{
+    if (phase or subindex)
+        return false;
+    pos = 0;
+    vertical = false;
+    return true;
+}
+bool ringmodulator_audio_module::get_layers(int index, int generation, unsigned int &layers) const
+{
+    layers = LG_REALTIME_DOT | (generation ? 0 : LG_CACHE_GRID) | ((redraw_graph || !generation) ? LG_CACHE_GRAPH : 0);
+    return true;
 }
