@@ -35,7 +35,10 @@ monosynth_audio_module::monosynth_audio_module()
 {
 }
 
-void monosynth_audio_module::activate() {
+void monosynth_audio_module::reset()
+{
+    last_stretch1 = 0;
+    stopping = false;
     running = false;
     output_pos = 0;
     queue_note_on = -1;
@@ -45,11 +48,15 @@ void monosynth_audio_module::activate() {
     modwheel_value_int = 0;
     inertia_cutoff.set_now(*params[par_cutoff]);
     inertia_pressure.set_now(0);
+    osc1.phasedelta = 0;
+    osc2.phasedelta = 0;
     filter.reset();
     filter2.reset();
     stack.clear();
     last_pwshift1 = last_pwshift2 = 0;
     last_stretch1 = 65536;
+    last_xfade = 0;
+    last_unison = 0.0;
     queue_note_on_and_off = false;
     prev_wave1 = -1;
     prev_wave2 = -1;
@@ -60,6 +67,11 @@ void monosynth_audio_module::activate() {
     lfo_clock = 0.f;
 }
 
+void monosynth_audio_module::activate()
+{
+    reset();
+}
+
 waveform_family<MONOSYNTH_WAVE_BITS> *monosynth_audio_module::waves;
 
 void monosynth_audio_module::precalculate_waves(progress_report_iface *reporter)
@@ -117,10 +129,12 @@ void monosynth_audio_module::precalculate_waves(progress_report_iface *reporter)
     for (int i = 0; i < S; i++) {
         data[i] = (min(1.f, (float)(i / 64.f))) * (1.0 - i * 1.0 / S) * (-1 + fmod (i * i * 8/ (S * S * 1.0), 2.0));
     }
+    normalize_waveform(data, S);
     waves[wave_skewsaw].make(bl, data);
     for (int i = 0; i < S; i++) {
         data[i] = (min(1.f, (float)(i / 64.f))) * (1.0 - i * 1.0 / S) * (fmod (i * i * 8/ (S * S * 1.0), 2.0) < 1.0 ? -1.0 : +1.0);
     }
+    normalize_waveform(data, S);
     waves[wave_skewsqr].make(bl, data);
 
     if (reporter)
@@ -135,6 +149,7 @@ void monosynth_audio_module::precalculate_waves(progress_report_iface *reporter)
             data[i] = -0.5 * sin(3 * M_PI * p * p);
         }
     }
+    normalize_waveform(data, S);
     waves[wave_test1].make(bl, data);
     for (int i = 0; i < S; i++) {
         data[i] = exp(-i * 1.0 / HS) * sin(i * M_PI / HS) * cos(2 * M_PI * i / HS);
@@ -146,14 +161,17 @@ void monosynth_audio_module::precalculate_waves(progress_report_iface *reporter)
         int ii = HS;
         data[i] = (ii * 1.0 / HS) * sin(i * 3 * M_PI / HS + 2 * M_PI * sin(M_PI / 4 + i * 4 * M_PI / HS)) * sin(i * 5 * M_PI / HS + 2 * M_PI * sin(M_PI / 8 + i * 6 * M_PI / HS));
     }
+    normalize_waveform(data, S);
     waves[wave_test3].make(bl, data);
     for (int i = 0; i < S; i++) {
         data[i] = sin(i * 2 * M_PI / HS + sin(i * 2 * M_PI / HS + 0.5 * M_PI * sin(i * 18 * M_PI / HS)) * sin(i * 1 * M_PI / HS + 0.5 * M_PI * sin(i * 11 * M_PI / HS)));
     }
+    normalize_waveform(data, S);
     waves[wave_test4].make(bl, data);
     for (int i = 0; i < S; i++) {
         data[i] = sin(i * 2 * M_PI / HS + 0.2 * M_PI * sin(i * 13 * M_PI / HS) + 0.1 * M_PI * sin(i * 37 * M_PI / HS)) * sin(i * M_PI / HS + 0.2 * M_PI * sin(i * 15 * M_PI / HS));
     }
+    normalize_waveform(data, S);
     waves[wave_test5].make(bl, data);
     for (int i = 0; i < S; i++) {
         if (i < HS)
@@ -167,24 +185,29 @@ void monosynth_audio_module::precalculate_waves(progress_report_iface *reporter)
         else
             data[i] = sin(i * 8 * M_PI / HS) * (S - i) / (S / 8);
     }
+    normalize_waveform(data, S);
     waves[wave_test6].make(bl, data);
     for (int i = 0; i < S; i++) {
         int j = i >> (MONOSYNTH_WAVE_BITS - 11);
         data[i] = (j ^ 0x1D0) * 1.0 / HS - 1;
     }
+    normalize_waveform(data, S);
     waves[wave_test7].make(bl, data);
     for (int i = 0; i < S; i++) {
         int j = i >> (MONOSYNTH_WAVE_BITS - 11);
         data[i] = -1 + 0.66 * (3 & ((j >> 8) ^ (j >> 10) ^ (j >> 6)));
     }
+    normalize_waveform(data, S);
     waves[wave_test8].make(bl, data);
     if (reporter)
         reporter->report_progress(100, "");
     
 }
 
-bool monosynth_audio_module::get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const
+bool monosynth_audio_module::get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
 {
+    if (!phase)
+        return false;
     monosynth_audio_module::precalculate_waves(NULL);
     // printf("get_graph %d %p %d wave1=%d wave2=%d\n", index, data, points, wave1, wave2);
     if (index == par_wave1 || index == par_wave2) {
@@ -232,20 +255,21 @@ bool monosynth_audio_module::get_graph(int index, int subindex, float *data, int
             return false;
         for (int i = 0; i < points; i++)
         {
-            typedef complex<double> cfloat;
             double freq = 20.0 * pow (20000.0 / 20.0, i * 1.0 / points);
             
-            const dsp::biquad_d1_lerp<float> &f = subindex ? filter2 : filter;
+            const dsp::biquad_d1_lerp &f = subindex ? filter2 : filter;
             float level = f.freq_gain(freq, srate);
             if (!is_stereo_filter())
                 level *= filter2.freq_gain(freq, srate);
+            else
+                set_channel_color(context, subindex);
             level *= fgain;
             
             data[i] = log(level) / log(1024.0) + 0.5;
         }
         return true;
     }
-    return get_static_graph(index, subindex, *params[index], data, points, context);
+    return false;
 }
 
 void monosynth_audio_module::calculate_buffer_oscs(float lfo1)
@@ -278,6 +302,20 @@ void monosynth_audio_module::calculate_buffer_oscs(float lfo1)
     float rnd_start = 1 - *params[par_window1] * 0.5f;
     float scl = rnd_start < 1.0 ? 1.f / (1 - rnd_start) : 0.f;
     
+    static const int muls[8] = { 33, -47, 53, -67, 87, -101, 121, -139 };
+    float unison = *params[par_o2unison] + moddest[moddest_o2unisonamp] * 0.01;
+    float unison_scale = 1.0, unison_delta = 0.0, last_unison_scale = 1.0, unison_scale_delta = 0.0;
+    if (unison > 0)
+    {
+        float freq = fabs(*params[par_o2unisonfrq] / muls[7]);
+        if (moddest[moddest_o2unisondetune] != 0)
+            freq *= pow(2.0, moddest[moddest_o2unisondetune]);
+        unison_osc.set_freq(freq, srate);
+        last_unison_scale = 1.0 / (1.0 + 2 * last_unison);
+        unison_scale = 1.0 / (1.0 + 2 * unison);
+        unison_delta = (unison - last_unison) * (1.0 / step_size);
+        unison_scale_delta = (unison_scale - last_unison_scale) * (1.0 / step_size);
+    }
     for (uint32_t i = 0; i < step_size; i++) 
     {
         //buffer[i] = lerp(osc1.get_phaseshifted(shift1, mix1), osc2.get_phaseshifted(shift2, mix2), cur_xfade);
@@ -288,7 +326,19 @@ void monosynth_audio_module::calculate_buffer_oscs(float lfo1)
         if (o1phase < 0)
             o1phase = 0;
         float r = 1.0 - o1phase * o1phase;
-        buffer[i] = lerp(r * osc1.get_phasedist(stretch1, shift1, mix1), osc2.get_phaseshifted(shift2, mix2), cur_xfade);
+        float osc1val = osc1.get_phasedist(stretch1, shift1, mix1);
+        float osc2val = osc2.get_phaseshifted(shift2, mix2);
+        if (unison > 0 || last_unison > 0)
+        {
+            for (int j = 0; j < 8; j++)
+                osc2val += last_unison * osc2.get_phaseshifted2(shift2, unison_osc.phase * muls[j], mix2);
+            osc2val *= last_unison_scale;
+
+            unison_osc.step();
+            last_unison += unison_delta;
+            last_unison_scale += unison_scale_delta;
+        }
+        buffer[i] = lerp(r * osc1val, osc2val, cur_xfade);
         osc1.advance();
         osc2.advance();
         shift1 += shift_delta1;
@@ -297,6 +347,7 @@ void monosynth_audio_module::calculate_buffer_oscs(float lfo1)
         cur_xfade += xfade_step;
     }
     last_xfade = new_xfade;
+    last_unison = unison;
 }
 
 void monosynth_audio_module::calculate_buffer_ser()
@@ -367,6 +418,7 @@ void monosynth_audio_module::delayed_note_on()
         if (legato >= 2)
             porta_time = -1.f;
         last_xfade = xfade;
+        unison_osc.phase = rand() << 16;
         osc1.reset();
         osc2.reset();
         filter.reset();
@@ -720,8 +772,8 @@ void monosynth_audio_module::set_frequency()
         p1 = pow(2.0, moddest[moddest_o1detune] * (1.0 / 1200.0));
     if (moddest[moddest_o2detune] != 0)
         p2 = pow(2.0, moddest[moddest_o2detune] * (1.0 / 1200.0));
-    osc1.set_freq(freq * (1 - detune_scaled) * p1 * inertia_pitchbend.get_last() * lfo_bend, srate);
-    osc2.set_freq(freq * (1 + detune_scaled) * p2 * inertia_pitchbend.get_last() * lfo_bend * xpose, srate);
+    osc1.set_freq(freq * (1 - detune_scaled) * p1 * inertia_pitchbend.get_last() * lfo_bend * xpose1, srate);
+    osc2.set_freq(freq * (1 + detune_scaled) * p2 * inertia_pitchbend.get_last() * lfo_bend * xpose2, srate);
 }
 
 
@@ -735,7 +787,8 @@ void monosynth_audio_module::params_changed()
     wave1 = dsp::clip(dsp::fastf2i_drm(*params[par_wave1]), 0, (int)wave_count - 1);
     wave2 = dsp::clip(dsp::fastf2i_drm(*params[par_wave2]), 0, (int)wave_count - 1);
     detune = pow(2.0, *params[par_detune] / 1200.0);
-    xpose = pow(2.0, *params[par_osc2xpose] / 12.0);
+    xpose1 = pow(2.0, *params[par_osc1xpose] / 12.0);
+    xpose2 = pow(2.0, *params[par_osc2xpose] / 12.0);
     xfade = *params[par_oscmix];
     legato = dsp::fastf2i_drm(*params[par_legato]);
     master.set_inertia(*params[par_master]);
@@ -784,4 +837,3 @@ uint32_t monosynth_audio_module::process(uint32_t offset, uint32_t nsamples, uin
         
     return had_data;
 }
-
