@@ -42,7 +42,7 @@ void organ_audio_module::activate()
     panic_flag = false;
 }
 
-void organ_audio_module::post_instantiate()
+void organ_audio_module::post_instantiate(uint32_t)
 {
     dsp::organ_voice_base::precalculate_waves(progress_report);
 }
@@ -69,47 +69,50 @@ void organ_audio_module::params_changed() {
     polyphony_limit = dsp::clip(dsp::fastf2i_drm(*params[par_polyphony]), 1, 32);
     if (polyphony_limit < old_poly)
         trim_voices();
-    
+    redraw = true;
     update_params();
 }
-
-bool organ_audio_module::get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const
+bool organ_audio_module::get_layers(int index, int generation, unsigned int &layers) const
 {
-    if (index == par_master) {
-        organ_voice_base::precalculate_waves(progress_report);
-        if (subindex)
-            return false;
-        float *waveforms[9];
-        int S[9], S2[9];
-        enum { small_waves = organ_voice_base::wave_count_small};
-        for (int i = 0; i < 9; i++)
+    layers = LG_REALTIME_GRAPH;
+    return true;
+}
+bool organ_audio_module::get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
+{
+    if (index != par_master or subindex or !phase)
+        return false;
+    
+    organ_voice_base::precalculate_waves(progress_report);
+    float *waveforms[9];
+    int S[9], S2[9];
+    enum { small_waves = organ_voice_base::wave_count_small};
+    for (int i = 0; i < 9; i++)
+    {
+        int wave = dsp::clip((int)(parameters->waveforms[i]), 0, (int)organ_voice_base::wave_count - 1);
+        if (wave >= small_waves)
         {
-            int wave = dsp::clip((int)(parameters->waveforms[i]), 0, (int)organ_voice_base::wave_count - 1);
-            if (wave >= small_waves)
-            {
-                waveforms[i] = organ_voice_base::get_big_wave(wave - small_waves).original;
-                S[i] = ORGAN_BIG_WAVE_SIZE;
-                S2[i] = ORGAN_WAVE_SIZE / 64;
-            }
-            else
-            {
-                waveforms[i] = organ_voice_base::get_wave(wave).original;
-                S[i] = S2[i] = ORGAN_WAVE_SIZE;
-            }
+            waveforms[i] = organ_voice_base::get_big_wave(wave - small_waves).original;
+            S[i] = ORGAN_BIG_WAVE_SIZE;
+            S2[i] = ORGAN_WAVE_SIZE / 64;
         }
-        for (int i = 0; i < points; i++)
+        else
         {
-            float sum = 0.f;
-            for (int j = 0; j < 9; j++)
-            {
-                float shift = parameters->phase[j] * S[j] / 360.0;
-                sum += parameters->drawbars[j] * waveforms[j][int(parameters->harmonics[j] * i * S2[j] / points + shift) & (S[j] - 1)];
-            }
-            data[i] = sum * 2 / (9 * 8);
+            waveforms[i] = organ_voice_base::get_wave(wave).original;
+            S[i] = S2[i] = ORGAN_WAVE_SIZE;
         }
-        return true;
     }
-    return false;
+    for (int i = 0; i < points; i++)
+    {
+        float sum = 0.f;
+        for (int j = 0; j < 9; j++)
+        {
+            float shift = parameters->phase[j] * S[j] / 360.0;
+            sum += parameters->drawbars[j] * waveforms[j][int(parameters->harmonics[j] * i * S2[j] / points + shift) & (S[j] - 1)];
+        }
+        data[i] = sum * 2 / (9 * 8);
+    }
+    return true;
+    
 }
 
 uint32_t organ_audio_module::message_run(const void *valid_inputs, void *output_ports)
@@ -698,7 +701,7 @@ void organ_voice::update_pitch()
     dphase.set(dsp::midi_note_to_phase(note, 100 * parameters->global_transpose + parameters->global_detune, sample_rate) * inertia_pitchbend.get_last());
 }
 
-void organ_voice::render_block() {
+void organ_voice::render_block(int snapshot) {
     if (note == -1)
         return;
 
@@ -731,7 +734,7 @@ void organ_voice::render_block() {
         uint32_t rate = (dphase * hm).get();
         if (waveid >= wave_count_small)
         {
-            float *data = (*big_waves)[waveid - wave_count_small].get_level(rate >> (ORGAN_BIG_WAVE_BITS - ORGAN_WAVE_BITS + ORGAN_BIG_WAVE_SHIFT));
+            data = (*big_waves)[waveid - wave_count_small].get_level(rate >> (ORGAN_BIG_WAVE_BITS - ORGAN_WAVE_BITS + ORGAN_BIG_WAVE_SHIFT));
             if (!data)
                 continue;
             hm.set(hm.get() >> ORGAN_BIG_WAVE_SHIFT);
@@ -983,7 +986,7 @@ bool drawbar_organ::check_percussion() {
 void drawbar_organ::pitch_bend(int amt)
 {
     parameters->pitch_bend = pow(2.0, (amt * parameters->pitch_bend_range) / (1200.0 * 8192.0));
-    for (list<voice *>::iterator i = active_voices.begin(); i != active_voices.end(); i++)
+    for (voice_array::iterator i = active_voices.begin(); i != active_voices.end(); ++i)
     {
         organ_voice *v = dynamic_cast<organ_voice *>(*i);
         v->update_pitch();
