@@ -14,14 +14,13 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  */
-#include <stdio.h>
-#include <assert.h>
 #include <config.h>
 #include <calf/osctl.h>
 #include <calf/utils.h>
+#include <stdio.h>
 #include <sstream>
 
 using namespace std;
@@ -34,7 +33,7 @@ string encode_map(const dictionary &data)
     osctl::string_buffer sb;
     osc_stream<osctl::string_buffer> str(sb);
     str << (uint32_t)data.size();
-    for(dictionary::const_iterator i = data.begin(); i != data.end(); i++)
+    for(dictionary::const_iterator i = data.begin(); i != data.end(); ++i)
     {
         str << i->first << i->second;
     }
@@ -70,6 +69,11 @@ std::string xml_escape(const std::string &src)
     return dest;
 }
 
+std::string to_xml_attr(const std::string &key, const std::string &value)
+{
+    return " " + key + "=\"" + xml_escape(value) + "\"";
+}
+
 std::string load_file(const std::string &src)
 {
     std::string str;
@@ -81,28 +85,39 @@ std::string load_file(const std::string &src)
         char buffer[1024];
         int len = fread(buffer, 1, sizeof(buffer), f);
         if (len < 0)
+        {
+            fclose(f);
             throw file_exception(src);
+        }
         str += string(buffer, len);
     }
+    fclose(f);
     return str;
 }
 
 std::string i2s(int value)
 {
     char buf[32];
-    sprintf(buf, "%d", value);
+    snprintf(buf, sizeof(buf), "%d", value);
     
     return std::string(buf);
 }
 
 std::string f2s(double value)
 {
-    // XXXKF might not work with some locale settings
     stringstream ss;
     ss << value;
     return ss.str();
 }
 
+std::string ff2s(double value)
+{
+    string s = f2s(value);
+    if (s.find('.') == string::npos)
+        s += ".0";
+    return s;
+}
+
 std::string indent(const std::string &src, const std::string &indent)
 {
     std::string dest;
@@ -119,4 +134,22 @@ std::string indent(const std::string &src, const std::string &indent)
     return dest;
 }
 
+//////////////////////////////////////////////////////////////////////////////////
+
+file_exception::file_exception(const std::string &f)
+: message(strerror(errno))
+, filename(f)
+, container(filename + ":" + message)
+{ 
+    text = container.c_str();
+}
+
+file_exception::file_exception(const std::string &f, const std::string &t)
+: message(t)
+, filename(f)
+, container(filename + ":" + message)
+{
+    text = container.c_str();
+}
+
 }
